@@ -13,7 +13,7 @@
 #include <cdi.h>
 int      vlistInqVarMissvalUsed(int vlistID, int varID);
 #ifndef DBL_IS_NAN
-#if  defined  (HAVE_ISNAN)
+#if  defined  (HAVE_DECL_ISNAN)
 #  define DBL_IS_NAN(x)     (isnan(x))
 #elif  defined  (FP_NAN)
 #  define DBL_IS_NAN(x)     (fpclassify(x) == FP_NAN)
@@ -56,7 +56,7 @@ int complevel = 0;              // Compression level
 static
 void version(void)
 {
-  fprintf(stderr, "CDI version 1.7.1\n");
+  fprintf(stderr, "CDI version 1.8\n");
 #if defined (COMPILER)
   fprintf(stderr, "Compiler: %s\n", COMPILER);
 #endif
@@ -111,13 +111,14 @@ void version(void)
   1.4.2  22 Mar 2005 : change level from int to double
   1.4.3  11 Apr 2005 : change date and time format to ISO
   1.5.0  22 Nov 2005 : IEG support
-  1.5.1  21 Feb 2006 : add option -s for short info
-  1.6.0   1 Aug 2006 : add option -z szip for SZIP compression of GRIB records
+  1.5.1  21 Feb 2006 : added option -s for short info
+  1.6.0   1 Aug 2006 : added option -z szip for SZIP compression of GRIB records
   1.6.1  27 Feb 2007 : short info with ltype for GENERIC zaxis
   1.6.2   3 Jan 2008 : changes for CDI library version 1.1.0 (compress)
   1.6.3  26 Mar 2008 : call streamDefTimestep also if ntsteps = 0 (buf fix)
-  1.7.0  11 Apr 2008 : add option -z zip for deflate compression of netCDF4 variables
-  1.7.1   1 Nov 2009 : add option -z jpeg for JPEG compression of GRIB2 records
+  1.7.0  11 Apr 2008 : added option -z zip for deflate compression of netCDF4 variables
+  1.7.1   1 Nov 2009 : added option -z jpeg for JPEG compression of GRIB2 records
+  1.7.2  14 Nov 2012 : added optional compression level -z zip[_1-9]
 */
 }
 
@@ -144,7 +145,7 @@ void usage(void)
   fprintf(stderr, "    -V             Print version number\n");
   fprintf(stderr, "    -z szip        SZIP compression of GRIB1 records\n");
   fprintf(stderr, "       jpeg        JPEG compression of GRIB2 records\n");
-  fprintf(stderr, "        zip        Deflate compression of netCDF4 variables\n");
+  fprintf(stderr, "        zip[_1-9]  Deflate compression of netCDF4 variables\n");
   fprintf(stderr, "\n");
   fprintf(stderr, "  Report bugs to <http://code.zmaw.de/projects/cdi>\n");
 }
@@ -158,25 +159,25 @@ void printInfo(int gridtype, int vdate, int vtime, char *varname, double level,
   int i, ivals = 0, imiss = 0;
   double arrmean, arrmin, arrmax;
   char vdatestr[32], vtimestr[32];
- 
+
   if ( ! rec )
   {
     if ( vardis )
-      fprintf(stdout, 
-    "   Rec :       Date  Time    Varname      Level    Size    Miss :     Minimum        Mean     Maximum\n");
+      fprintf(stdout,
+    "   Rec :       Date     Time   Level Gridsize    Miss :     Minimum        Mean     Maximum : Parameter name\n");
 /*   ----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8----+----9----+----0----+ */
     else
-      fprintf(stdout, 
-    "   Rec :       Date  Time    Param        Level    Size    Miss :     Minimum        Mean     Maximum\n");
+      fprintf(stdout,
+    "   Rec :       Date     Time   Level Gridsize    Miss :     Minimum        Mean     Maximum : Parameter ID\n");
 /*   ----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8----+----9----+----0----+ */
   }
 
   date2str(vdate, vdatestr, sizeof(vdatestr));
   time2str(vtime, vtimestr, sizeof(vtimestr));
 
-  fprintf(stdout, "%6d :%s %s %-10s %7g ", ++rec, vdatestr, vtimestr, varname, level);
+  fprintf(stdout, "%6d :%s %s %7g ", ++rec, vdatestr, vtimestr, level);
 
-  fprintf(stdout, "%7d ", datasize);
+  fprintf(stdout, "%8d ", datasize);
 
   fprintf(stdout, "%7d :", nmiss);
 
@@ -222,7 +223,7 @@ void printInfo(int gridtype, int vdate, int vtime, char *varname, double level,
 
       if ( datasize > 0 ) arrmean /= datasize;
 
-      fprintf(stdout, "%#12.5g%#12.5g%#12.5g\n", arrmin, arrmean, arrmax);
+      fprintf(stdout, "%#12.5g%#12.5g%#12.5g", arrmin, arrmean, arrmax);
     }
   else
     {
@@ -249,13 +250,16 @@ void printInfo(int gridtype, int vdate, int vtime, char *varname, double level,
 
       if ( nvals_r > 0 ) arrmean_r = arrsum_r / nvals_r;
       if ( nvals_i > 0 ) arrmean_i = arrsum_i / nvals_i;
-      fprintf(stdout, "  -  (%#12.5g,%#12.5g)  -\n", arrmean_r, arrmean_i);
+      fprintf(stdout, "  -  (%#12.5g,%#12.5g)  -", arrmean_r, arrmean_i);
     }
 
+  fprintf(stdout, " : %-11s\n", varname);
+
   if ( imiss != nmiss && nmiss > 0 )
     fprintf(stdout, "Found %d of %d missing values!\n", imiss, nmiss);
 }
 
+#define MAXCHARS 82
 
 static
 void printShortinfo(int streamID, int vlistID, int vardis)
@@ -268,7 +272,7 @@ void printShortinfo(int streamID, int vlistID, int vardis)
   int nrecs, nvars, nzaxis, ntsteps;
   int levelID, levelsize;
   int tsID, ntimeout;
-  int timeID, taxisID;
+  int tsteptype, taxisID;
   int nbyte, nbyte0;
   int index;
   char varname[CDI_MAX_NAME];
@@ -286,10 +290,10 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 
       if ( vardis )
 	fprintf(stdout,
-		"   Var : Institut Source   Varname     Time Typ  Grid Size Num  Levels Num\n");
+		"   Var : Institut Source   Ttype    Levels Num  Gridsize Num Dtype : Parameter name\n");
       else
 	fprintf(stdout,
-		"   Var : Institut Source   Param       Time Typ  Grid Size Num  Levels Num\n");
+		"   Var : Institut Source   Ttype    Levels Num  Gridsize Num Dtype : Parameter ID\n");
 
       nvars = vlistNvars(vlistID);
 
@@ -299,39 +303,45 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 	  gridID  = vlistInqVarGrid(vlistID, varID);
 	  zaxisID = vlistInqVarZaxis(vlistID, varID);
 
-	  cdiParamToString(param, paramstr, sizeof(paramstr));
-
-	  if ( vardis ) vlistInqVarName(vlistID, varID, varname);
-
-	  gridsize = gridInqSize(gridID);
-
 	  fprintf(stdout, "%6d : ", varID + 1);
 
+	  /* institute info */
 	  instptr = institutInqNamePtr(vlistInqVarInstitut(vlistID, varID));
 	  if ( instptr )
-	    fprintf(stdout, "%-9s", instptr);
+	    fprintf(stdout, "%-8s ", instptr);
 	  else
 	    fprintf(stdout, "unknown  ");
 
+	  /* source info */
 	  modelptr = modelInqNamePtr(vlistInqVarModel(vlistID, varID));
 	  if ( modelptr )
-	    fprintf(stdout, "%-9s", modelptr);
+	    fprintf(stdout, "%-8s ", modelptr);
 	  else
 	    fprintf(stdout, "unknown  ");
 
-	  if ( vardis )
-	    fprintf(stdout, "%-12s", varname);
-	  else
-	    fprintf(stdout, "%-12s", paramstr);
+	  /* tsteptype */
+	  tsteptype = vlistInqVarTsteptype(vlistID, varID);
+	  if      ( tsteptype == TSTEP_CONSTANT ) fprintf(stdout, "%-8s ", "constant");
+	  else if ( tsteptype == TSTEP_INSTANT  ) fprintf(stdout, "%-8s ", "instant");
+	  else if ( tsteptype == TSTEP_INSTANT2 ) fprintf(stdout, "%-8s ", "instant");
+	  else if ( tsteptype == TSTEP_INSTANT3 ) fprintf(stdout, "%-8s ", "instant");
+	  else if ( tsteptype == TSTEP_MIN      ) fprintf(stdout, "%-8s ", "min");
+	  else if ( tsteptype == TSTEP_MAX      ) fprintf(stdout, "%-8s ", "max");
+	  else if ( tsteptype == TSTEP_ACCUM    ) fprintf(stdout, "%-8s ", "accum");
+	  else                                    fprintf(stdout, "%-8s ", "unknown");
 
-	  timeID = vlistInqVarTime(vlistID, varID);
-	  if ( timeID == TIME_CONSTANT )
-	    fprintf(stdout, "con ");
-	  else
-	    fprintf(stdout, "var ");
+	  /* layer info */
+	  levelsize = zaxisInqSize(zaxisID);
+	  fprintf(stdout, "%6d ", levelsize);
+	  fprintf(stdout, "%3d ", vlistZaxisIndex(vlistID, zaxisID) + 1);
 
+	  /* grid info */
+	  gridsize = gridInqSize(gridID);
+	  fprintf(stdout, "%9d ", gridsize);
+	  fprintf(stdout, "%3d ", vlistGridIndex(vlistID, gridID) + 1);
+
+	  /* datatype */
 	  datatype = vlistInqVarDatatype(vlistID, varID);
-
 	  if      ( datatype == DATATYPE_PACK   ) strcpy(pstr, "P0");
 	  else if ( datatype > 0 && datatype <= 32  ) sprintf(pstr, "P%d", datatype);
 	  else if ( datatype == DATATYPE_CPX32  ) strcpy(pstr, "C32");
@@ -349,26 +359,30 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 	  fprintf(stdout, " %-3s", pstr);
 
 	  if ( vlistInqVarCompType(vlistID, varID) == COMPRESS_NONE )
-	    fprintf(stdout, " ");
+	    fprintf(stdout, "  ");
 	  else
-	    fprintf(stdout, "z");
+	    fprintf(stdout, "z ");
 
-	  fprintf(stdout, "%9d", gridsize);
+	  /* parameter info */
+	  fprintf(stdout, ": ");
 
-	  fprintf(stdout, " %3d ", vlistGridIndex(vlistID, gridID) + 1);
+	  cdiParamToString(param, paramstr, sizeof(paramstr));
 
-	  levelsize = zaxisInqSize(zaxisID);
-	  fprintf(stdout, " %6d", levelsize);
-	  fprintf(stdout, " %3d", vlistZaxisIndex(vlistID, zaxisID) + 1);
+	  if ( vardis ) vlistInqVarName(vlistID, varID, varname);
+
+          if ( vardis )
+	    fprintf(stdout, "%-11s ", varname);
+	  else
+	    fprintf(stdout, "%-11s ", paramstr);
 
 	  fprintf(stdout, "\n");
 	}
 
-      fprintf(stdout, "   Horizontal grids :\n");
+      fprintf(stdout, "   Grid coordinates :\n");
       printGridInfo(vlistID);
 
       nzaxis = vlistNzaxis(vlistID);
-      fprintf(stdout, "   Vertical grids :\n");
+      fprintf(stdout, "   Vertical coordinates :\n");
       for ( index = 0; index < nzaxis; index++)
 	{
 	  zaxisID   = vlistZaxis(vlistID, index);
@@ -387,7 +401,7 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 	  nbyte = nbyte0;
 	  for ( levelID = 0; levelID < levelsize; levelID++ )
 	    {
-	      if ( nbyte > 80 )
+	      if ( nbyte > MAXCHARS )
 		{
 		  fprintf(stdout, "\n");
 		  fprintf(stdout, "%*s", nbyte0, "");
@@ -404,7 +418,7 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 	      nbyte0 = fprintf(stdout, "%33s : ", "bounds");
 	      for ( levelID = 0; levelID < levelsize; levelID++ )
 		{
-		  if ( nbyte > 80 )
+		  if ( nbyte > MAXCHARS )
 		    {
 		      fprintf(stdout, "\n");
 		      fprintf(stdout, "%*s", nbyte0, "");
@@ -424,13 +438,13 @@ void printShortinfo(int streamID, int vlistID, int vardis)
       if ( ntsteps != 0 )
 	{
 	  if ( ntsteps == CDI_UNDEFID )
-	    fprintf(stdout, "   Time axis :  unlimited steps\n");
+	    fprintf(stdout, "   Time coordinate :  unlimited steps\n");
 	  else
-	    fprintf(stdout, "   Time axis :  %d step%s\n", ntsteps, ntsteps == 1 ? "" : "s");
+	    fprintf(stdout, "   Time coordinate :  %d step%s\n", ntsteps, ntsteps == 1 ? "" : "s");
 
 	  if ( taxisID != CDI_UNDEFID )
 	    {
-	      int calendar, unit;
+	      int calendar, tunits;
 
 	      if ( taxisInqType(taxisID) == TAXIS_RELATIVE )
 		{
@@ -443,20 +457,26 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 		  fprintf(stdout, "     RefTime = %4.4d-%2.2d-%2.2d %2.2d:%2.2d:%2.2d",
 			  year, month, day, hour, minute, second);
 
-		  unit = taxisInqTunit(taxisID);
-		  if ( unit != CDI_UNDEFID )
+		  tunits = taxisInqTunit(taxisID);
+		  if ( tunits != CDI_UNDEFID )
 		    {
-		      if ( unit == TUNIT_YEAR )
+		      if ( tunits == TUNIT_YEAR )
 			fprintf(stdout, "  Units = years");
-		      else if ( unit == TUNIT_MONTH )
+		      else if ( tunits == TUNIT_MONTH )
 			fprintf(stdout, "  Units = months");
-		      else if ( unit == TUNIT_DAY )
+		      else if ( tunits == TUNIT_DAY )
 			fprintf(stdout, "  Units = days");
-		      else if ( unit == TUNIT_HOUR )
+		      else if ( tunits == TUNIT_12HOURS )
+			fprintf(stdout, "  Units = 12hours");
+		      else if ( tunits == TUNIT_6HOURS )
+			fprintf(stdout, "  Units = 6hours");
+		      else if ( tunits == TUNIT_3HOURS )
+			fprintf(stdout, "  Units = 3hours");
+		      else if ( tunits == TUNIT_HOUR )
 			fprintf(stdout, "  Units = hours");
-		      else if ( unit == TUNIT_MINUTE )
+		      else if ( tunits == TUNIT_MINUTE )
 			fprintf(stdout, "  Units = minutes");
-		      else if ( unit == TUNIT_SECOND )
+		      else if ( tunits == TUNIT_SECOND )
 			fprintf(stdout, "  Units = seconds");
 		      else
 			fprintf(stdout, "  Units = unknown");
@@ -720,10 +740,13 @@ void defineCompress(const char *arg)
       comptype = COMPRESS_GZIP;
       complevel = 6;
     }
-  else if ( strncmp(arg, "zip", len) == 0 )
+  else if ( strncmp(arg, "zip", 3) == 0 )
     {
       comptype = COMPRESS_ZIP;
-      complevel = 1;
+      if ( len == 5 && arg[3] == '_' && isdigit(arg[4]) )
+	complevel = atoi(&arg[4]);
+      else
+        complevel = 1;
     }
   else
     fprintf(stderr, "%s compression unsupported!\n", arg);
@@ -853,7 +876,7 @@ int main(int argc, char *argv[])
       int recID;
       int tsID;
       int ntsteps = 0;
-      int taxisID;
+      int taxisID1, taxisID2 = CDI_UNDEFID;
       int gridtype;
       int vlistID1, vlistID2 = CDI_UNDEFID;
 
@@ -874,13 +897,18 @@ int main(int argc, char *argv[])
 	}
 
       nvars   = vlistNvars(vlistID1);
-      taxisID = vlistInqTaxis(vlistID1);
+      taxisID1 = vlistInqTaxis(vlistID1);
       ntsteps = vlistNtsteps(vlistID1);
 
       if ( Debug ) fprintf(stderr, "nvars   = %d\n", nvars);
       if ( Debug ) fprintf(stderr, "ntsteps = %d\n", ntsteps);
 
-      if ( fname2 ) vlistID2 = vlistDuplicate(vlistID1);
+      if ( fname2 )
+        {
+          vlistID2 = vlistDuplicate(vlistID1);
+          taxisID2 = taxisDuplicate(taxisID1);
+          vlistDefTaxis(vlistID2, taxisID2);
+        }
 
       for ( varID = 0; varID < nvars; varID++)
 	{
@@ -943,10 +971,12 @@ int main(int argc, char *argv[])
       while ( (nrecs = streamInqTimestep(streamID1, tsID)) > 0 )
 	{
 	  if ( fname2 /* && ntsteps != 0*/ )
-	    streamDefTimestep(streamID2, tsID);
-
-	  vdate = taxisInqVdate(taxisID);
-	  vtime = taxisInqVtime(taxisID);
+            {
+              taxisCopyTimestep(taxisID2, taxisID1);
+              streamDefTimestep(streamID2, tsID);
+            }
+	  vdate = taxisInqVdate(taxisID1);
+	  vtime = taxisInqVtime(taxisID1);
 
 	  if ( Debug )
 	    fprintf(stdout, "tsID = %d nrecs = %d date = %d time = %d\n", tsID, nrecs, vdate, vtime);
@@ -993,7 +1023,7 @@ int main(int argc, char *argv[])
 	    {
 	      for ( varID = 0; varID < nvars; varID++ )
 		{
-		  if ( vlistInqVarTime(vlistID1, varID) == TIME_CONSTANT && tsID > 0 ) continue;
+		  if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT && tsID > 0 ) continue;
 
 		  number   = vlistInqVarNumber(vlistID1, varID);
 		  gridID   = vlistInqVarGrid(vlistID1, varID);
@@ -1038,6 +1068,7 @@ int main(int argc, char *argv[])
 	{
 	  streamClose(streamID2);
 	  vlistDestroy(vlistID2);
+	  taxisDestroy(taxisID2);
 	}
       streamClose(streamID1);
     }
@@ -1047,3 +1078,12 @@ int main(int argc, char *argv[])
 
   return (0);
 }
+/*
+ * Local Variables:
+ * c-file-style: "Java"
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * show-trailing-whitespace: t
+ * require-trailing-newline: t
+ * End:
+ */
