@@ -112,6 +112,11 @@ extern "C" {
 #define  DATATYPE_TXT           253
 #define  DATATYPE_CPX           254
 
+/* Chunks */
+
+#define  CHUNK_AUTO               1  /* use default chunk size                  */
+#define  CHUNK_GRID               2
+#define  CHUNK_LINES              3
 
 /* GRID types */
 
@@ -152,16 +157,33 @@ extern "C" {
 #define  ZAXIS_ATMOSPHERE        15  /* Entire atmosphere                       */
 #define  ZAXIS_REFERENCE         16  /* zaxis reference number                  */
 
+/* TIME types */
+
+#define  TIME_CONSTANT            0  /* obsolate, use TSTEP_CONSTANT            */
+#define  TIME_VARIABLE            1  /* obsolate, use TSTEP_INSTANT             */
+
+/* TSTEP types */
+
+#define  TSTEP_CONSTANT           0
+#define  TSTEP_INSTANT            1
+#define  TSTEP_AVG                2
+#define  TSTEP_ACCUM              3
+#define  TSTEP_MAX                4
+#define  TSTEP_MIN                5
+#define  TSTEP_DIFF               6
+#define  TSTEP_RMS                7
+#define  TSTEP_SD                 8
+#define  TSTEP_COV                9
+#define  TSTEP_RATIO             10
+#define  TSTEP_RANGE             11
+#define  TSTEP_INSTANT2          12
+#define  TSTEP_INSTANT3          13
+
 /* TAXIS types */
 
 #define  TAXIS_ABSOLUTE           1
 #define  TAXIS_RELATIVE           2
 
-/* TIME types */
-
-#define  TIME_CONSTANT            1
-#define  TIME_VARIABLE            2
-
 /* TUNIT types */
 
 #define  TUNIT_SECOND             1
@@ -175,18 +197,6 @@ extern "C" {
 #define  TUNIT_6HOURS             9
 #define  TUNIT_12HOURS           10
 
-/* TSTEP types */
-
-#define  TSTEP_INSTANT            1
-#define  TSTEP_AVG                2
-#define  TSTEP_ACCUM              3
-#define  TSTEP_MAX                4
-#define  TSTEP_MIN                5
-#define  TSTEP_DIFF               6
-#define  TSTEP_RANGE              7
-#define  TSTEP_INSTANT2           8
-#define  TSTEP_INSTANT3           9
-
 /* CALENDAR types */
 
 #define  CALENDAR_STANDARD        0  /* don't change this value (used also in cgribexlib)! */
@@ -196,9 +206,33 @@ extern "C" {
 #define  CALENDAR_366DAYS         4
 #define  CALENDAR_NONE            5
 
+/* parallel IO IOMode */
+
+#define  PIO_NONE                 0
+#define  PIO_MPI                  1
+#define  PIO_WRITER               2
+#define  PIO_ASYNCH               3
+#define  PIO_FPGUARD              4
+
+#define  PIO_MINIOMODE                  PIO_NONE
+#define  PIO_MAXIOMODE                  PIO_FPGUARD
+#define  PIO_MINIOMODEWITHSPECIALPROCS  PIO_WRITER
+
+/* parallel IO routines */
+
+void     pioEndDef             ( void );
+void     pioEndTimestepping    ( void );
+void     pioFinalize           ( void );
+int      pioInit               ( int, int, int, int, int * );
+int      pioInqVarDecoChunk    ( int, int );
+int      pioInqVarDecoOff      ( int, int );
+void     pioNamespaceSetActive ( int );
+void     pioWriteTimestep      ( int, int, int );
 
 /* CDI control routines */
 
+void    cdiReset(void);
+
 char   *cdiStringError(int cdiErrno);
 
 void    cdiDebug(int debug);
@@ -250,6 +284,7 @@ void    streamDefVlist(int streamID, int vlistID);
 
 /*      streamInqVlist: Get the Vlist of a stream */
 int     streamInqVlist(int streamID);
+int     streamInqVlistIDorig(int streamID);
 
 /*      streamInqFiletype: Get the filetype */
 int     streamInqFiletype(int streamID);
@@ -283,6 +318,7 @@ char   *streamFilesuffix(int filetype);
 int     streamNtsteps(int streamID);
 off_t   streamNvals(int streamID);
 
+int     streamInqNvars ( int streamID );
 
 /* STREAM var I/O routines */
 
@@ -291,12 +327,14 @@ void    streamReadVar(int streamID, int varID, double *data_vec, int *nmiss);
 
 /*      streamWriteVar: Write a variable */
 void    streamWriteVar(int streamID, int varID, const double *data_vec, int nmiss);
+void    streamWriteVarF(int streamID, int varID, const float *data_vec, int nmiss);
 
 /*      streamReadVarSlice: Read a horizontal slice of a variable */
 void    streamReadVarSlice(int streamID, int varID, int levelID, double *data_vec, int *nmiss);
 
 /*      streamWriteVarSlice: Write a horizontal slice of a variable */
 void    streamWriteVarSlice(int streamID, int varID, int levelID, const double *data_vec, int nmiss);
+void    streamWriteVarSliceF(int streamID, int varID, int levelID, const float *data_vec, int nmiss);
 
 
 /* STREAM record I/O routines */
@@ -305,6 +343,7 @@ void    streamInqRecord(int streamID, int *varID, int *levelID);
 void    streamDefRecord(int streamID, int  varID, int  levelID);
 void    streamReadRecord(int streamID, double *data_vec, int *nmiss);
 void    streamWriteRecord(int streamID, const double *data_vec, int nmiss);
+void    streamWriteRecordF(int streamID, const float *data_vec, int nmiss);
 void    streamCopyRecord(int streamIDdest, int streamIDsrc);
 
 void    streamInqGinfo(int streamID, int *intnum, float *fltnum);
@@ -378,15 +417,20 @@ int     vlistInqModel(int vlistID);
 /* VLIST VAR routines */
 
 /*      vlistDefVar: Create a new Variable */
-int     vlistDefVar(int vlistID, int gridID, int zaxisID, int timeID);
+int     vlistDefVar(int vlistID, int gridID, int zaxisID, int tsteptype);
 
 void    vlistChangeVarGrid(int vlistID, int varID, int gridID);
 void    vlistChangeVarZaxis(int vlistID, int varID, int zaxisID);
 
-void    vlistInqVar(int vlistID, int varID, int *gridID, int *zaxisID, int *timeID);
+void    vlistInqVar(int vlistID, int varID, int *gridID, int *zaxisID, int *tsteptype);
 int     vlistInqVarGrid(int vlistID, int varID);
 int     vlistInqVarZaxis(int vlistID, int varID);
-int     vlistInqVarTime(int vlistID, int varID);
+
+/* used in MPIOM */
+int     vlistInqVarID(int vlistID, int code);
+
+int     vlistInqVarTsteptype(int vlistID, int varID);
+void    vlistDefVarTsteptype(int vlistID, int varID, int tsteptype);
 
 void    vlistDefVarCompType(int vlistID, int varID, int comptype);
 int     vlistInqVarCompType(int vlistID, int varID);
@@ -411,6 +455,9 @@ void    vlistDefVarDatatype(int vlistID, int varID, int datatype);
 /*      vlistInqVarDatatype: Get the data type of a Variable */
 int     vlistInqVarDatatype(int vlistID, int varID);
 
+void    vlistDefVarChunkType(int vlistID, int varID, int chunktype);
+int     vlistInqVarChunkType(int vlistID, int varID);
+
 void    vlistDefVarXYZ(int vlistID, int varID, int xyz);
 int     vlistInqVarXYZ(int vlistID, int varID);
 
@@ -458,8 +505,6 @@ double  vlistInqVarScalefactor(int vlistID, int varID);
 void    vlistDefVarAddoffset(int vlistID, int varID, double addoffset);
 double  vlistInqVarAddoffset(int vlistID, int varID);
 
-void    vlistDefVarTsteptype(int vlistID, int varID, int tsteptype);
-int     vlistInqVarTsteptype(int vlistID, int varID);
 void    vlistDefVarTimave(int vlistID, int varID, int timave);
 int     vlistInqVarTimave(int vlistID, int varID);
 void    vlistDefVarTimaccu(int vlistID, int varID, int timaccu);
@@ -476,6 +521,9 @@ int     vlistFindLevel(int vlistID, int fvarID, int flevelID);
 int     vlistMergedVar(int vlistID, int varID);
 int     vlistMergedLevel(int vlistID, int varID, int levelID);
 
+/*     Ensemble info routines */
+void    vlistDefVarEnsemble(int vlistID, int varID, int ensID, int ensCount, int forecast_type);
+int     vlistInqVarEnsemble(int vlistID, int varID, int *ensID, int *ensCount, int *forecast_type);
 
 /* VLIST attributes */
 
