@@ -21,18 +21,30 @@
 #include "extra.h"
 #include "ieg.h"
 #include "vlist.h"
+#include "resource_handle.h"
+#include "pio_util.h"
+
+#include "namespace.h"
+#include "pio_interface.h"
+#include "pio_rpc.h"
+#include "pio_comm.h"
+
+#include <string.h>
 
 #define  MAX_FNAMES  3
 
+extern resOps streamOps;
+
+
 FILE *popen(const char *command, const char *type);
 int pclose(FILE *stream);
 
 void cdiPrintDefaults(void)
 {
-  fprintf (stderr, "default instID     :  %d\n", cdiDefaultInstID); 
-  fprintf (stderr, "default modelID    :  %d\n", cdiDefaultModelID); 
-  fprintf (stderr, "default tableID    :  %d\n", cdiDefaultTableID); 
-  fprintf (stderr, "default missval    :  %g\n", cdiDefaultMissval); 
+  fprintf (stderr, "default instID     :  %d\n", cdiDefaultInstID);
+  fprintf (stderr, "default modelID    :  %d\n", cdiDefaultModelID);
+  fprintf (stderr, "default tableID    :  %d\n", cdiDefaultTableID);
+  fprintf (stderr, "default missval    :  %g\n", cdiDefaultMissval);
 }
 
 
@@ -77,7 +89,8 @@ void cdiDebug(int level)
 #define IsBigendian()  ( u_byteorder.c[sizeof(long) - 1] )
 
 
-static int getByteorder(int byteswap)
+static
+int getByteorder(int byteswap)
 {
   static union {unsigned long l; unsigned char c[sizeof(long)];} u_byteorder = {1};
   int byteorder = -1;
@@ -97,7 +110,8 @@ static int getByteorder(int byteswap)
 }
 
 
-static int getFiletype(const char *filename, int *byteorder)
+static
+int getFiletype(const char *filename, int *byteorder)
 {
   int filetype = CDI_EUFTYPE;
   int fileID;
@@ -232,7 +246,7 @@ int get_fnames(const char *argument, char *fnames[], int max_fnames)
     {
       pch = &argument[i+1];
       len -= (i+1);
-      if ( len && ( memcmp(argument, "filelist:", i) == 0 || 
+      if ( len && ( memcmp(argument, "filelist:", i) == 0 ||
 		    memcmp(argument, "flist:", i) == 0 ) )
 	{
 	  for ( i = 0; i < len; ++i ) if ( pch[i] == ',' ) nfiles++;
@@ -253,7 +267,7 @@ int get_fnames(const char *argument, char *fnames[], int max_fnames)
 		{
 		  if ( line[0] == '#' || line[0] == '\0' ||
 		       line[0] == ' ' ) continue;
-		  
+
 		  if ( nfiles >= max_fnames )
 		    {
 		      Warning("Too many input files (limit: %d)", max_fnames);
@@ -262,7 +276,7 @@ int get_fnames(const char *argument, char *fnames[], int max_fnames)
 		  fnames[nfiles] = strdupx(line);
 		  nfiles++;
 		}
-	      
+
 	      fclose(fp);
 
 	      if ( nfiles == 0 ) Error("No input file found in %s", pch);
@@ -270,10 +284,10 @@ int get_fnames(const char *argument, char *fnames[], int max_fnames)
 	  else
 	    {
 	      char xline[65536];
-	      	      
+
 	      strcpy(xline, pch);
 	      for ( i = 0; i < len; i++ ) if ( xline[i] == ',' ) xline[i] = 0;
-	      
+
 	      nfiles++;
 	      if ( nfiles >= max_fnames )
 		{
@@ -293,14 +307,13 @@ int get_fnames(const char *argument, char *fnames[], int max_fnames)
 	{
 	  char command[4096];
 	  FILE *pfp;
-	  
+
 	  strcpy(command, "ls ");
 	  strcat(command, pch);
 
 	  pfp = popen(command, "r");
-	  if ( pfp == NULL )
-	    SysError("popen %s failed", command);
-	  
+	  if ( pfp == NULL ) SysError("popen %s failed", command);
+
 	  nfiles = 0;
 	  while ( _readline_(pfp, line, MAX_LINE) )
 	    {
@@ -321,7 +334,7 @@ int get_fnames(const char *argument, char *fnames[], int max_fnames)
     }
 
   num_fnames = nfiles;
-  
+
   return (num_fnames);
 }
 
@@ -379,7 +392,7 @@ int getByteswap(int byteorder)
 
 @Prototype void streamDefByteorder(int streamID, int byteorder)
 @Parameter
-    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenRead} or @fref{streamOpenWrite}.
+    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenWrite}.
     @Item  byteorder The byte order of a dataset, one of the CDI constants @func{CDI_BIGENDIAN} and
                      @func{CDI_LITTLEENDIAN}.
 
@@ -391,16 +404,21 @@ with the file format type @func{FILETYPE_SRV}, @func{FILETYPE_EXT} or @func{FILE
 */
 void streamDefByteorder(int streamID, int byteorder)
 {
-  int filetype, fileID;
+  int filetype;
   stream_t *streamptr;
 
   streamptr = stream_to_pointer(streamID);
 
   stream_check_ptr(__func__, streamptr);
 
+  if ( reshGetStatus ( streamID, &streamOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
   streamptr->byteorder = byteorder;
   filetype = streamptr->filetype;
-  fileID   = streamptr->fileID;
 
   switch (filetype)
     {
@@ -578,7 +596,13 @@ int cdiInqContents(int streamID)
       vlistID = streamInqVlist(streamID);
       taxisID = vlistInqTaxis(vlistID);
       if ( taxisID != -1 )
-	ptaxisCopy(taxisPtr(taxisID), &streamptr->tsteps[0].taxis);
+        {
+          taxis_t *taxisptr1 = &streamptr->tsteps[0].taxis;
+          taxis_t *taxisptr2 = taxisPtr(taxisID);
+          ptaxisCopy(taxisptr2, taxisptr1);
+          if ( taxisptr1->name     ) taxisptr2->name = taxisptr1->name;
+          if ( taxisptr1->longname ) taxisptr2->longname = taxisptr1->longname;
+        }
     }
 
   return (status);
@@ -592,80 +616,113 @@ int streamOpen(const char *filename, const char *filemode, int filetype)
   int status;
   Record *record = NULL;
   stream_t *streamptr = NULL;
+  int hasLocalFile = namespaceHasLocalFile ( namespaceGetActive ());
 
   if ( CDI_Debug )
     Message("Open %s mode %c file %s", strfiletype(filetype), (int) *filemode, filename);
 
   if ( ! filename || ! filemode || filetype < 0 ) return (CDI_EINVAL);
 
-  switch (filetype)
+  xdebug("START, filename = %s, filemode = %s, filetype = %d, "
+         "namespaceHasLocalFile(%d)=%s",
+         filename, filemode, filetype, namespaceGetActive(),
+	 namespaceHasLocalFile ( namespaceGetActive ()) ? "true":"false");
+
+  if ( hasLocalFile )
     {
+      switch (filetype)
+	{
 #if  defined  (HAVE_LIBGRIB)
-    case FILETYPE_GRB:
-    case FILETYPE_GRB2:
-      {
-        fileID = gribOpen(filename, filemode);
-	record = (Record *) malloc(sizeof(Record));
-	record->buffer = NULL;
-	break;
-      }
+	case FILETYPE_GRB:
+	case FILETYPE_GRB2:
+	  {
+	    fileID = gribOpen(filename, filemode);
+            if ( fileID < 0 ) fileID = CDI_ESYSTEM;
+            xdebug("%s OPENED, fileID=%d", filename, fileID);
+	    record = (Record *) malloc(sizeof(Record));
+	    record->buffer = NULL;
+	    break;
+	  }
 #endif
 #if  defined  (HAVE_LIBSERVICE)
-    case FILETYPE_SRV:
-      {
-        fileID = fileOpen(filename, filemode);
-	record = (Record *) malloc(sizeof(Record));
-	record->buffer = NULL;
-	record->srvp   = srvNew();
-	break;
-      }
+	case FILETYPE_SRV:
+	  {
+	    fileID = fileOpen(filename, filemode);
+            if ( fileID < 0 ) fileID = CDI_ESYSTEM;
+	    record = (Record *) malloc(sizeof(Record));
+	    record->buffer = NULL;
+	    record->srvp   = srvNew();
+	    break;
+	  }
 #endif
 #if  defined  (HAVE_LIBEXTRA)
-    case FILETYPE_EXT:
-      {
-        fileID = fileOpen(filename, filemode);
-	record = (Record *) malloc(sizeof(Record));
-	record->buffer = NULL;
-	record->extp   = extNew();
-	break;
-      }
+	case FILETYPE_EXT:
+	  {
+	    fileID = fileOpen(filename, filemode);
+            if ( fileID < 0 ) fileID = CDI_ESYSTEM;
+	    record = (Record *) malloc(sizeof(Record));
+	    record->buffer = NULL;
+	    record->extp   = extNew();
+	    break;
+	  }
 #endif
 #if  defined  (HAVE_LIBIEG)
-    case FILETYPE_IEG:
-      {
-        fileID = fileOpen(filename, filemode);
-	record = (Record *) malloc(sizeof(Record));
-	record->buffer = NULL;
-	record->iegp   = iegNew();
-	break;
-      }
+	case FILETYPE_IEG:
+	  {
+	    fileID = fileOpen(filename, filemode);
+            if ( fileID < 0 ) fileID = CDI_ESYSTEM;
+	    record = (Record *) malloc(sizeof(Record));
+	    record->buffer = NULL;
+	    record->iegp   = iegNew();
+	    break;
+	  }
 #endif
 #if  defined  (HAVE_LIBNETCDF)
-    case FILETYPE_NC:
-      {
-	fileID = cdfOpen(filename, filemode);
-	break;
-      }
-    case FILETYPE_NC2:
-      {
-	fileID = cdfOpen64(filename, filemode);
-	break;
-      }
-    case FILETYPE_NC4:
-    case FILETYPE_NC4C:
-      {
-	fileID = cdf4Open(filename, filemode, &filetype);
-	break;
-      }
- #endif
-    default:
-      {
-	if ( CDI_Debug ) Message("%s support not compiled in!", strfiletype(filetype));
-	return (CDI_ELIBNAVAIL);
-      }
+	case FILETYPE_NC:
+	  {
+	    fileID = cdfOpen(filename, filemode);
+	    break;
+	  }
+	case FILETYPE_NC2:
+	  {
+	    fileID = cdfOpen64(filename, filemode);
+	    break;
+	  }
+	case FILETYPE_NC4:
+	case FILETYPE_NC4C:
+	  {
+	    fileID = cdf4Open(filename, filemode, &filetype);
+	    break;
+	  }
+#endif
+	default:
+	  {
+	    if ( CDI_Debug ) Message("%s support not compiled in!", strfiletype(filetype));
+	    return (CDI_ELIBNAVAIL);
+	  }
+	}
     }
+#ifdef USE_MPI
+  else if ( tolower ( * filemode ) == 'w' )
+    {
+      statusCode nspStatus = namespaceInqResStatus ();
+      switch ( nspStatus )
+        {
+        case STAGE_DEFINITION:
+          break;
+        case STAGE_TIMELOOP:
+          pioBufferFuncCall(STREAMOPEN, 2, filename, filetype);
+          break;
+        case STAGE_CLEANUP:
+          xabort ( "TRANSITION TO IO PROCESSES ALREADY FINISHED." );
+        break;
+        default:
+          xabort ( "INTERNAL ERROR" );
+        }
+    }
+#endif
 
-  if ( fileID < 0 )
+  if ( fileID < 0 && hasLocalFile )
     {
       streamID = fileID;
     }
@@ -674,7 +731,7 @@ int streamOpen(const char *filename, const char *filemode, int filetype)
       streamptr = stream_new_entry();
       streamID  = streamptr->self;
 
-      if ( streamID < 0 ) return(CDI_ELIMIT);
+      if ( streamID < 0 ) return (CDI_ELIMIT);
 
       streamptr->record   = record;
       streamptr->filetype = filetype;
@@ -697,7 +754,7 @@ int streamOpen(const char *filename, const char *filemode, int filetype)
 	  vlistptr->ntsteps = streamNtsteps(streamID);
 	}
     }
- 
+
   return (streamID);
 }
 
@@ -806,7 +863,7 @@ int streamOpenA(const char *filename, const char *filemode, int filetype)
       vlistptr = vlist_to_pointer(streamptr->vlistID);
       vlistptr->ntsteps = cdiInqTimeSize(streamID);
     }
- 
+
   switch (filetype)
     {
 #if  defined  (HAVE_LIBGRIB)
@@ -969,13 +1026,15 @@ int streamOpenRead(const char *filenames)
   const char *filename;
   stream_t *streamptr = NULL;
 
+  cdiInitialize();
+
   //num_fnames = get_fnames(filenames, fnames, MAX_FNAMES);
 
   if ( num_fnames == 0 )
     filename = filenames;
   else
     {
-      int i; 
+      int i;
       for ( i = 0; i < num_fnames; ++i ) printf("fnames: %d %s\n", i, fnames[i]);
       filename = fnames[0];
     }
@@ -1011,6 +1070,8 @@ int streamOpenAppend(const char *filename)
   int streamID;
   stream_t *streamptr;
 
+  cdiInitialize();
+
   filetype = getFiletype(filename, &byteorder);
 
   if ( filetype < 0 ) return (filetype);
@@ -1053,7 +1114,7 @@ open stream. Otherwise, a negative number with the error status is returned.
 @EndList
 
 @Example
-Here is an example using @func{streamOpenWrite} to create a new netCDF file 
+Here is an example using @func{streamOpenWrite} to create a new netCDF file
 named @func{foo.nc} for writing:
 
 @Source
@@ -1069,6 +1130,8 @@ if ( streamID < 0 ) handle_error(streamID);
 */
 int streamOpenWrite(const char *filename, int filetype)
 {
+  cdiInitialize();
+
   return (streamOpen(filename, "w", filetype));
 }
 
@@ -1104,68 +1167,89 @@ void streamClose(int streamID)
   filetype = streamptr->filetype;
   vlistID  = streamptr->vlistID;
 
-  if ( fileID == CDI_UNDEFID )
-    Warning("File %s not open!", streamptr->filename);
-  else
-    switch (filetype)
-      {
+  if ( namespaceHasLocalFile ( namespaceGetActive ()))
+    {
+      if ( fileID == CDI_UNDEFID )
+        Warning("File %s not open!", streamptr->filename);
+      else
+	switch (filetype)
+	  {
 #if  defined  (HAVE_LIBGRIB)
-      case FILETYPE_GRB:
-      case FILETYPE_GRB2:
-	{
-	  gribClose(fileID);
-	  gribContainersDelete(streamID);
-	  break;
-	}
+	  case FILETYPE_GRB:
+	  case FILETYPE_GRB2:
+	    {
+	      gribClose(fileID);
+	      gribContainersDelete(streamID);
+	      break;
+	    }
 #endif
 #if  defined  (HAVE_LIBSERVICE)
-      case FILETYPE_SRV:
-	{
-	  fileClose(fileID);
-	  srvDelete(streamptr->record->srvp);
-	  break;
-	}
+	  case FILETYPE_SRV:
+	    {
+	      fileClose(fileID);
+	      srvDelete(streamptr->record->srvp);
+	      break;
+	    }
 #endif
 #if  defined  (HAVE_LIBEXTRA)
-      case FILETYPE_EXT:
-	{
-	  fileClose(fileID);
-	  extDelete(streamptr->record->extp);
-	  break;
-	}
+	  case FILETYPE_EXT:
+	    {
+	      fileClose(fileID);
+	      extDelete(streamptr->record->extp);
+	      break;
+	    }
 #endif
 #if  defined  (HAVE_LIBIEG)
-      case FILETYPE_IEG:
-	{
-	  fileClose(fileID);
-	  iegDelete(streamptr->record->iegp);
-	  break;
-	}
+	  case FILETYPE_IEG:
+	    {
+	      fileClose(fileID);
+	      iegDelete(streamptr->record->iegp);
+	      break;
+	    }
 #endif
 #if  defined  (HAVE_LIBNETCDF)
-      case FILETYPE_NC:
-      case FILETYPE_NC2:
-      case FILETYPE_NC4:
-      case FILETYPE_NC4C:
-	{
-	  cdfClose(fileID);
-	  break;
-	}
+	  case FILETYPE_NC:
+	  case FILETYPE_NC2:
+	  case FILETYPE_NC4:
+	  case FILETYPE_NC4C:
+	    {
+	      cdfClose(fileID);
+	      break;
+	    }
+#endif
+	  default:
+	    {
+	      Error("%s support not compiled in!", strfiletype(filetype));
+	      break;
+	    }
+	  }
+    }
+#ifdef USE_MPI
+  else
+    {
+      statusCode nspStatus = namespaceInqResStatus ();
+      switch ( nspStatus )
+        {
+        case STAGE_DEFINITION:
+          break;
+        case STAGE_TIMELOOP:
+          pioBufferFuncCall(STREAMCLOSE, 1, streamID);
+          break;
+        case STAGE_CLEANUP:
+        break;
+        default:
+          xabort ( "INTERNAL ERROR" );
+        }
+    }
 #endif
-      default:
-	{
-	  Error("%s support not compiled in!", strfiletype(filetype));
-	  break;
-	}
-      }
 
   if ( streamptr->record )
-    {
-      if ( streamptr->record->buffer )
-	free(streamptr->record->buffer);
+      {
+	  if ( streamptr->record->buffer )
+              free(streamptr->record->buffer);
 
-      free(streamptr->record);
-    }  
+	  free(streamptr->record);
+      }
 
   streamptr->filetype = 0;
   if ( streamptr->filename ) free(streamptr->filename);
@@ -1186,7 +1270,7 @@ void streamClose(int streamID)
       if ( streamptr->tsteps[index].recIDs )
 	free(streamptr->tsteps[index].recIDs);
     }
-    
+
   if ( streamptr->tsteps ) free(streamptr->tsteps);
 
   if ( streamptr->nfiles > 0 )
@@ -1232,6 +1316,8 @@ void streamSync(int streamID)
   int nvars;
   stream_t *streamptr;
 
+  if ( !namespaceHasLocalFile ( namespaceGetActive ())) return;
+
   streamptr = stream_to_pointer(streamID);
 
   stream_check_ptr(__func__, streamptr);
@@ -1280,7 +1366,7 @@ void streamSync(int streamID)
 
 @Prototype int streamDefTimestep(int streamID, int tsID)
 @Parameter
-    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenRead} or @fref{streamOpenWrite}.
+    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenWrite}.
     @Item  tsID      Timestep identifier.
 
 @Description
@@ -1296,7 +1382,10 @@ int streamDefTimestep(int streamID, int tsID)
   int newtsID;
   int taxisID;
   int vlistID;
+  int time_is_varying;
   stream_t *streamptr;
+  taxis_t *taxisptr1;
+  taxis_t *taxisptr2;
 
   streamptr = stream_to_pointer(streamID);
 
@@ -1306,12 +1395,18 @@ int streamDefTimestep(int streamID, int tsID)
   stream_check_ptr(__func__, streamptr);
 
   vlistID = streamInqVlist(streamID);
-  taxisID = vlistInqTaxis(vlistID);
-  if ( taxisID == CDI_UNDEFID )
+
+  time_is_varying = vlistHasTime(vlistID);
+
+  if ( time_is_varying )
     {
-      Warning("taxisID undefined for fileID = %d! Using absolute time axis.", streamID);
-      taxisID = taxisCreate(TAXIS_ABSOLUTE);
-      vlistDefTaxis(vlistID, taxisID);
+      taxisID = vlistInqTaxis(vlistID);
+      if ( taxisID == CDI_UNDEFID )
+        {
+          Warning("taxisID undefined for fileID = %d! Using absolute time axis.", streamID);
+          taxisID = taxisCreate(TAXIS_ABSOLUTE);
+          vlistDefTaxis(vlistID, taxisID);
+        }
     }
 
   newtsID = tstepsNewEntry(streamID);
@@ -1321,15 +1416,26 @@ int streamDefTimestep(int streamID, int tsID)
 
   streamptr->curTsID = tsID;
 
-  ptaxisCopy(&streamptr->tsteps[tsID].taxis, taxisPtr(taxisID));
+  if ( time_is_varying )
+    {
+      taxisptr1 = taxisPtr(taxisID);
+      taxisptr2 = &streamptr->tsteps[tsID].taxis;
+      ptaxisCopy(taxisptr2, taxisptr1);
+      if ( tsID == 0 )
+        {
+          if ( taxisptr1->name     ) taxisptr2->name = taxisptr1->name;
+          if ( taxisptr1->longname ) taxisptr2->longname = taxisptr1->longname;
+        }
+    }
 
   streamptr->ntsteps = tsID + 1;
 
-  if ( (streamptr->filetype == FILETYPE_NC  ||
-	streamptr->filetype == FILETYPE_NC2 ||
-	streamptr->filetype == FILETYPE_NC4 ||
-	streamptr->filetype == FILETYPE_NC4C)
-       && vlistHasTime(vlistID) )
+  if ( namespaceHasLocalFile(namespaceGetActive())
+       && (streamptr->filetype == FILETYPE_NC  ||
+           streamptr->filetype == FILETYPE_NC2 ||
+           streamptr->filetype == FILETYPE_NC4 ||
+           streamptr->filetype == FILETYPE_NC4C)
+       && time_is_varying )
     cdfDefTimestep(streamID, tsID);
 
   cdiCreateRecords(streamID, tsID);
@@ -1454,7 +1560,7 @@ int streamInqTimestep(int streamID, int tsID)
 
 @Prototype void streamReadVar(int streamID, int varID, double *data, int *nmiss)
 @Parameter
-    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenRead} or @fref{streamOpenWrite}.
+    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenRead}.
     @Item  varID     Variable identifier.
     @Item  data      Pointer to the location into which the data values are read.
                      The caller must allocate space for the returned values.
@@ -1532,23 +1638,8 @@ void streamReadVar(int streamID, int varID, double *data, int *nmiss)
     }
 }
 
-/*
-@Function  streamWriteVar
-@Title     Write a variable
 
-@Prototype void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
-@Parameter
-    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenRead} or @fref{streamOpenWrite}.
-    @Item  varID     Variable identifier.
-    @Item  data      Pointer to a block of data values to be written.
-    @Item  nmiss     Number of missing values.
-
-@Description
-The function streamWriteVar writes the values of one time step of a variable 
-to an open dataset.
-@EndFunction
-*/
-void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
+void stream_write_var(int streamID, int varID, int memtype, const void *data, int nmiss)
 {
   int filetype;
   stream_t *streamptr;
@@ -1557,11 +1648,34 @@ void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
 
   check_parg(data);
 
+  /* begin deike */
+#ifdef USE_MPI
+  if ( ! namespaceHasLocalFile ( namespaceGetActive ()))
+    {
+      statusCode nspStatus = namespaceInqResStatus ();
+      switch ( nspStatus )
+        {
+        case STAGE_DEFINITION:
+          xabort ( "DEFINITION STAGE: PARALLEL WRITING NOT POSSIBLE." );
+          break;
+        case STAGE_TIMELOOP:
+          pioBufferData ( streamID, varID, data, nmiss );
+          return;
+        case STAGE_CLEANUP:
+          xabort ( "CLEANUP STAGE: PARALLEL WRITING NOT POSSIBLE." );
+          break;
+        default:
+          xabort ( "INTERNAL ERROR" );
+        }
+    }
+#endif
+  /* end deike */
+
   streamptr = stream_to_pointer(streamID);
 
   stream_check_ptr(__func__, streamptr);
 
-  streamDefineTaxis(streamID);
+  // streamDefineTaxis(streamID);
 
   filetype = streamptr->filetype;
 
@@ -1571,6 +1685,7 @@ void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
     case FILETYPE_GRB:
     case FILETYPE_GRB2:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("grbWriteVar not implemented for memtype float!");
         grbWriteVarDP(streamID, varID, data, nmiss);
 	break;
       }
@@ -1578,6 +1693,7 @@ void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
 #if  defined  (HAVE_LIBSERVICE)
     case FILETYPE_SRV:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("srvWriteVar not implemented for memtype float!");
         srvWriteVarDP(streamID, varID, data);
 	break;
       }
@@ -1585,6 +1701,7 @@ void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
 #if  defined  (HAVE_LIBEXTRA)
     case FILETYPE_EXT:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("extWriteVar not implemented for memtype float!");
         extWriteVarDP(streamID, varID, data);
 	break;
       }
@@ -1592,6 +1709,7 @@ void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
 #if  defined  (HAVE_LIBIEG)
     case FILETYPE_IEG:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("iegWriteVar not implemented for memtype float!");
         iegWriteVarDP(streamID, varID, data);
 	break;
       }
@@ -1603,7 +1721,7 @@ void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
     case FILETYPE_NC4C:
       {
 	if ( streamptr->accessmode == 0 ) cdfEndDef(streamID);
-        cdfWriteVarDP(streamID, varID, data, nmiss);
+        cdf_write_var(streamID, varID, memtype, data, nmiss);
 	break;
       }
 #endif
@@ -1615,13 +1733,56 @@ void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
     }
 }
 
+/*
+@Function  streamWriteVar
+@Title     Write a variable
+
+@Prototype void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
+@Parameter
+    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenWrite}.
+    @Item  varID     Variable identifier.
+    @Item  data      Pointer to a block of double precision floating point data values to be written.
+    @Item  nmiss     Number of missing values.
+
+@Description
+The function streamWriteVar writes the values of one time step of a variable to an open dataset.
+The values are converted to the external data type of the variable, if necessary.
+@EndFunction
+*/
+void streamWriteVar(int streamID, int varID, const double *data, int nmiss)
+{
+  stream_write_var(streamID, varID, MEMTYPE_DOUBLE, data, nmiss);
+}
+
+/*
+@Function  streamWriteVarF
+@Title     Write a variable
+
+@Prototype void streamWriteVarF(int streamID, int varID, const float *data, int nmiss)
+@Parameter
+    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenWrite}.
+    @Item  varID     Variable identifier.
+    @Item  data      Pointer to a block of single precision floating point data values to be written.
+    @Item  nmiss     Number of missing values.
+
+@Description
+The function streamWriteVarF writes the values of one time step of a variable to an open dataset.
+The values are converted to the external data type of the variable, if necessary.
+Only support for netCDF was implemented in this function.
+@EndFunction
+*/
+void streamWriteVarF(int streamID, int varID, const float *data, int nmiss)
+{
+  stream_write_var(streamID, varID, MEMTYPE_FLOAT, data, nmiss);
+}
+
 /*
 @Function  streamReadVarSlice
 @Title     Read a horizontal slice of a variable
 
 @Prototype void streamReadVarSlice(int streamID, int varID, int levelID, double *data, int *nmiss)
 @Parameter
-    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenRead} or @fref{streamOpenWrite}.
+    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenRead}.
     @Item  varID     Variable identifier.
     @Item  levelID   Level identifier.
     @Item  data      Pointer to the location into which the data values are read.
@@ -1636,7 +1797,6 @@ from an open dataset.
 void streamReadVarSlice(int streamID, int varID, int levelID, double *data, int *nmiss)
 {
   int filetype;
-  int ierr = 0;
   stream_t *streamptr;
 
   if ( CDI_Debug ) Message("streamID = %d  varID = %d", streamID, varID);
@@ -1689,7 +1849,8 @@ void streamReadVarSlice(int streamID, int varID, int levelID, double *data, int
     case FILETYPE_NC4:
     case FILETYPE_NC4C:
       {
-        ierr = cdfReadVarSliceDP(streamID, varID, levelID, data, nmiss);
+        /* FIXME: status value ignored */
+        int ierr = cdfReadVarSliceDP(streamID, varID, levelID, data, nmiss);
 	break;
       }
 #endif
@@ -1701,27 +1862,10 @@ void streamReadVarSlice(int streamID, int varID, int levelID, double *data, int
     }
 }
 
-/*
-@Function  streamWriteVarSlice
-@Title     Write a horizontal slice of a variable
 
-@Prototype void streamWriteVarSlice(int streamID, int varID, int levelID, const double *data, int nmiss)
-@Parameter
-    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenRead} or @fref{streamOpenWrite}.
-    @Item  varID     Variable identifier.
-    @Item  levelID   Level identifier.
-    @Item  data      Pointer to a block of data values to be written.
-    @Item  nmiss     Number of missing values.
-
-@Description
-The function streamWriteVarSlice writes the values of a horizontal slice of a 
-variable to an open dataset.
-@EndFunction
-*/
-void streamWriteVarSlice(int streamID, int varID, int levelID, const double *data, int nmiss)
+void stream_write_var_slice(int streamID, int varID, int levelID, int memtype, const void *data, int nmiss)
 {
   int filetype;
-  int ierr = 0;
   stream_t *streamptr;
 
   if ( CDI_Debug ) Message("streamID = %d varID = %d", streamID, varID);
@@ -1740,6 +1884,7 @@ void streamWriteVarSlice(int streamID, int varID, int levelID, const double *dat
     case FILETYPE_GRB:
     case FILETYPE_GRB2:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("grbWriteVarSlice not implemented for memtype float!");
         grbWriteVarSliceDP(streamID, varID, levelID, data, nmiss);
 	break;
       }
@@ -1747,6 +1892,7 @@ void streamWriteVarSlice(int streamID, int varID, int levelID, const double *dat
 #if  defined  (HAVE_LIBSERVICE)
     case FILETYPE_SRV:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("srvWriteVarSlice not implemented for memtype float!");
         srvWriteVarSliceDP(streamID, varID, levelID, data);
 	break;
       }
@@ -1754,6 +1900,7 @@ void streamWriteVarSlice(int streamID, int varID, int levelID, const double *dat
 #if  defined  (HAVE_LIBEXTRA)
     case FILETYPE_EXT:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("extWriteVarSlice not implemented for memtype float!");
         extWriteVarSliceDP(streamID, varID, levelID, data);
 	break;
       }
@@ -1761,6 +1908,7 @@ void streamWriteVarSlice(int streamID, int varID, int levelID, const double *dat
 #if  defined  (HAVE_LIBIEG)
     case FILETYPE_IEG:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("iegWriteVarSlice not implemented for memtype float!");
         iegWriteVarSliceDP(streamID, varID, levelID, data);
 	break;
       }
@@ -1771,8 +1919,9 @@ void streamWriteVarSlice(int streamID, int varID, int levelID, const double *dat
     case FILETYPE_NC4:
     case FILETYPE_NC4C:
       {
+        int ierr = 0;
 	if ( streamptr->accessmode == 0 ) cdfEndDef(streamID);
-        ierr = cdfWriteVarSliceDP(streamID, varID, levelID, data, nmiss);
+        ierr = cdf_write_var_slice(streamID, varID, levelID, memtype, data, nmiss);
 	break;
       }
 #endif
@@ -1784,6 +1933,51 @@ void streamWriteVarSlice(int streamID, int varID, int levelID, const double *dat
     }
 }
 
+/*
+@Function  streamWriteVarSlice
+@Title     Write a horizontal slice of a variable
+
+@Prototype void streamWriteVarSlice(int streamID, int varID, int levelID, const double *data, int nmiss)
+@Parameter
+    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenWrite}.
+    @Item  varID     Variable identifier.
+    @Item  levelID   Level identifier.
+    @Item  data      Pointer to a block of double precision floating point data values to be written.
+    @Item  nmiss     Number of missing values.
+
+@Description
+The function streamWriteVarSlice writes the values of a horizontal slice of a variable to an open dataset.
+The values are converted to the external data type of the variable, if necessary.
+@EndFunction
+*/
+void streamWriteVarSlice(int streamID, int varID, int levelID, const double *data, int nmiss)
+{
+  stream_write_var_slice(streamID, varID, levelID, MEMTYPE_DOUBLE, data, nmiss);
+}
+
+/*
+@Function  streamWriteVarSliceF
+@Title     Write a horizontal slice of a variable
+
+@Prototype void streamWriteVarSliceF(int streamID, int varID, int levelID, const float *data, int nmiss)
+@Parameter
+    @Item  streamID  Stream ID, from a previous call to @fref{streamOpenWrite}.
+    @Item  varID     Variable identifier.
+    @Item  levelID   Level identifier.
+    @Item  data      Pointer to a block of single precision floating point data values to be written.
+    @Item  nmiss     Number of missing values.
+
+@Description
+The function streamWriteVarSliceF writes the values of a horizontal slice of a variable to an open dataset.
+The values are converted to the external data type of the variable, if necessary.
+Only support for netCDF was implemented in this function.
+@EndFunction
+*/
+void streamWriteVarSliceF(int streamID, int varID, int levelID, const float *data, int nmiss)
+{
+  stream_write_var_slice(streamID, varID, levelID, MEMTYPE_FLOAT, data, nmiss);
+}
+
 
 void streamWriteContents(int streamID, char *cname)
 {
@@ -1791,8 +1985,8 @@ void streamWriteContents(int streamID, char *cname)
   int tsID, recID, varID, levelID;
   long recsize;
   int nrecs, nvars;
-  int code, gridID, zaxisID, timeID, datatype;
-  int ngrids, nzaxis;
+  int code, gridID, zaxisID, tsteptype, datatype;
+  int ngrids;
   int filetype, gridtype;
   int xsize, ysize;
   int date, time;
@@ -1830,28 +2024,27 @@ void streamWriteContents(int streamID, char *cname)
       ysize    = gridInqYsize(gridID);
       fprintf(cnp, "%4d:%4d:%4d:%4d\n", i+1, gridtype, xsize, ysize);
     }
-  nzaxis = vlistNzaxis(vlistID);
 
   fprintf(cnp, "#\n");
 
-  fprintf(cnp, "varID:code:gridID:zaxisID:timeID:datatype\n");
+  fprintf(cnp, "varID:code:gridID:zaxisID:tsteptype:datatype\n");
 
   nvars = vlistNvars(vlistID);
   for ( varID = 0; varID < nvars; varID++ )
     {
-      code     = vlistInqVarCode(vlistID, varID);
-      gridID   = vlistInqVarGrid(vlistID, varID);
-      zaxisID  = vlistInqVarZaxis(vlistID, varID);
-      timeID   = vlistInqVarTime(vlistID, varID);
-      datatype = vlistInqVarDatatype(vlistID, varID);
+      code      = vlistInqVarCode(vlistID, varID);
+      gridID    = vlistInqVarGrid(vlistID, varID);
+      zaxisID   = vlistInqVarZaxis(vlistID, varID);
+      tsteptype = vlistInqVarTsteptype(vlistID, varID);
+      datatype  = vlistInqVarDatatype(vlistID, varID);
       fprintf(cnp, "%4d:%4d:%4d:%4d:%4d:%4d:\n",
-	      varID+1, code, gridID, zaxisID, timeID, datatype);
+	      varID+1, code, gridID, zaxisID, tsteptype, datatype);
     }
 
   fprintf(cnp, "#\n");
 
   fprintf(cnp, "tsID:nrecs:date:time\n");
-  
+
   tsID = 0;
   while (1)
     {
@@ -1968,7 +2161,7 @@ off_t   streamNvals(int streamID)
 
 @Prototype void streamDefVlist(int streamID, int vlistID)
 @Parameter
-    @Item  streamID Stream ID, from a previous call to @fref{streamOpenRead} or @fref{streamOpenWrite}.
+    @Item  streamID Stream ID, from a previous call to @fref{streamOpenWrite}.
     @Item  vlistID  Variable list ID, from a previous call to @fref{vlistCreate}.
 
 @Description
@@ -1985,10 +2178,31 @@ void streamDefVlist(int streamID, int vlistID)
   streamptr = stream_to_pointer(streamID);
 
   stream_check_ptr(__func__, streamptr);
+#ifdef USE_MPI
+  if (!namespaceHasLocalFile(namespaceGetActive()))
+  {
+    statusCode nspStatus = namespaceInqResStatus ();
+    switch ( nspStatus )
+      {
+      case STAGE_DEFINITION:
+        break;
+      case STAGE_TIMELOOP:
+        pioBufferFuncCall(STREAMDEFVLIST, 2, streamID, vlistID);
+        break;
+      case STAGE_CLEANUP:
+        xabort ( "TRANSITION TO IO PROCESSES ALREADY FINISHED." );
+        break;
+      default:
+        xabort ( "INTERNAL ERROR" );
+      }
+  }
+
+#endif
 
   if ( streamptr->vlistID == CDI_UNDEFID )
     {
       streamptr->vlistID = vlistDuplicate(vlistID);
+      streamptr->vlistIDorig = vlistID ;
 
       nvars = vlistNvars(vlistID);
       for ( varID = 0; varID < nvars; varID++ )
@@ -1997,10 +2211,12 @@ void streamDefVlist(int streamID, int vlistID)
 	  zaxisID = vlistInqVarZaxis(vlistID, varID);
 	  streamNewVar(streamID, gridID, zaxisID);
 	  if ( streamptr->have_missval )
-	    vlistDefVarMissval(streamptr->vlistID, varID, vlistInqVarMissval(vlistID, varID));
+	    vlistDefVarMissval(streamptr->vlistID, varID, 
+                               vlistInqVarMissval(vlistID, varID));
 	}
 
-      if ( streamptr->filemode == 'w' )
+      if (namespaceHasLocalFile(namespaceGetActive())
+          && streamptr->filemode == 'w' )
 	{
 	  if ( streamptr->filetype == FILETYPE_NC  ||
 	       streamptr->filetype == FILETYPE_NC2 ||
@@ -2043,17 +2259,35 @@ int streamInqVlist(int streamID)
   stream_t *streamptr;
 
   streamptr = stream_to_pointer(streamID);
-  
+
   stream_check_ptr(__func__, streamptr);
 
   return (streamptr->vlistID);
 }
 
 
+int streamInqVlistIDorig(int streamID)
+{
+  stream_t *streamptr;
+
+  streamptr = stream_to_pointer(streamID);
+
+  stream_check_ptr(__func__, streamptr);
+
+  return (streamptr->vlistIDorig);
+}
+
+
 void streamDefCompType(int streamID, int comptype)
 {
   stream_t *streamptr;
 
+  if ( reshGetStatus ( streamID, &streamOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
   streamptr = stream_to_pointer(streamID);
 
   stream_check_ptr(__func__, streamptr);
@@ -2066,6 +2300,12 @@ void streamDefCompLevel(int streamID, int complevel)
 {
   stream_t *streamptr;
 
+  if ( reshGetStatus ( streamID, &streamOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
   streamptr = stream_to_pointer(streamID);
 
   stream_check_ptr(__func__, streamptr);
