@@ -11,15 +11,17 @@ void   cdfInqHistoryString(int streamID, char *history);
 
 void   cdfEndDef(int streamID);
 int    cdfDefRecord(int streamID);
+
 int    cdfCopyRecord(int streamIDdest, int streamIDsrc);
+
 int    cdfReadRecord(int streamID, double *data, int *nmiss);
-void   cdfWriteRecord(int streamID, const double *data, int nmiss);
+void   cdf_write_record(int streamID, int memtype, const void *data, int nmiss);
 
 void   cdfReadVarDP(int streamID, int varID, double *data, int *nmiss);
-void   cdfWriteVarDP(int streamID, int varID, const double *data, int nmiss);
+void   cdf_write_var(int streamID, int varID, int memtype, const void *data, int nmiss);
 
 int    cdfReadVarSliceDP(int streamID, int varID, int levelID, double *data, int *nmiss);
-int    cdfWriteVarSliceDP(int streamID, int varID, int levelID, const double *data, int nmiss);
+int    cdf_write_var_slice(int streamID, int varID, int levelID, int memtype, const void *data, int nmiss);
 
 #endif
 /*
