@@ -3,7 +3,7 @@
 #endif
 
 #include <stdio.h>
-#include <float.h>  /* FLT_EPSILON */
+// #include <float.h>  /* FLT_EPSILON */
 
 #include "dmemory.h"
 #include "cdi.h"
@@ -25,7 +25,7 @@ typedef struct {
   int level1;
   int level2;
   int ltype;
-} compvar_t; 
+} compvar_t;
 
 
 #if  defined  (HAVE_LIBCGRIBEX)
@@ -51,9 +51,7 @@ int cgribexGetGridType(int *isec2)
 
   return (gridtype);
 }
-#endif
 
-#if  defined  (HAVE_LIBCGRIBEX)
 static
 int cgribexGetIsRotated(int *isec2)
 {
@@ -63,12 +61,10 @@ int cgribexGetIsRotated(int *isec2)
     {
       isRotated = 1;
     }
-
+ 
   return (isRotated);
 }
-#endif
 
-#if  defined  (HAVE_LIBCGRIBEX)
 static
 int cgribexGetZaxisHasBounds(int grb_ltype)
 {
@@ -87,9 +83,7 @@ int cgribexGetZaxisHasBounds(int grb_ltype)
 
   return (lbounds);
 }
-#endif
 
-#if  defined  (HAVE_LIBCGRIBEX)
 static
 int cgribexGetTimeUnit(int *isec1)
 {
@@ -115,9 +109,7 @@ int cgribexGetTimeUnit(int *isec1)
 
   return (timeunit);
 }
-#endif
 
-#if  defined  (HAVE_LIBCGRIBEX)
 static
 int cgribexTimeIsFC(int *isec1)
 {
@@ -128,9 +120,7 @@ int cgribexTimeIsFC(int *isec1)
 
   return (isFC);
 }
-#endif
 
-#if  defined  (HAVE_LIBCGRIBEX)
 static
 int cgribexGetTsteptype(int timerange)
 {
@@ -156,21 +146,22 @@ int cgribexGetTsteptype(int timerange)
 
   return (tsteptype);
 }
-#endif
 
-#if  defined  (HAVE_LIBCGRIBEX)
 static
-void cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid)
+void cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid, int iret)
 {
   int gridtype;
 
   gridtype = cgribexGetGridType(isec2);
 
-  if ( streamptr->unreduced && gridtype == GRID_GAUSSIAN_REDUCED )
+  if ( streamptr->unreduced && gridtype == GRID_GAUSSIAN_REDUCED && iret != -801 )
     {
+      int ilat, nlon = 0;
+      for ( ilat = 0; ilat < ISEC2_NumLat; ++ilat )
+        if ( ISEC2_RowLon(ilat) > nlon ) nlon = ISEC2_RowLon(ilat);
       gridtype = GRID_GAUSSIAN;
-      ISEC2_NumLon = 2*ISEC2_NumLat;
-      ISEC4_NumValues = ISEC2_NumLon*ISEC2_NumLat;
+      ISEC2_NumLon = nlon;
+      ISEC4_NumValues = nlon*ISEC2_NumLat;
     }
 
   memset(grid, 0, sizeof(grid_t));
@@ -193,16 +184,22 @@ void cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid)
 	  {
 	    if ( grid->xsize > 1 )
 	      {
-		if ( ISEC2_ResFlag && ISEC2_LonIncr > 0 )
-		  if ( ISEC2_LonIncr*(grid->xsize-1) > 360000 ) ISEC2_LonIncr = 0;
+                int recompinc = TRUE;
 
 		if ( ISEC2_ResFlag && ISEC2_LonIncr > 0 )
-		  grid->xinc = ISEC2_LonIncr * 0.001;
-		else
-		  grid->xinc = (ISEC2_LastLon - ISEC2_FirstLon) * 0.001 / (grid->xsize - 1);
+                  {
+                    if ( abs(ISEC2_LastLon - (ISEC2_FirstLon+ISEC2_LonIncr*(grid->xsize-1))) <= 2 )
+                      {
+                        recompinc = FALSE;
+                        grid->xinc = ISEC2_LonIncr * 0.001;
+                      }
+                  }
+
+		/* recompute xinc if necessary */
+                if ( recompinc ) grid->xinc = (ISEC2_LastLon - ISEC2_FirstLon) * 0.001 / (grid->xsize-1);
 
 		/* correct xinc if necessary */
-		if ( ISEC2_FirstLon == 0 && ISEC2_LastLon > 354000 )
+		if ( ISEC2_FirstLon == 0 && ISEC2_LastLon > 354000 && ISEC2_LastLon < 360000 )
 		  {
 		    double xinc = 360. / grid->xsize;
 
@@ -222,13 +219,19 @@ void cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid)
 	  {
 	    if ( grid->ysize > 1 )
 	      {
-		if ( ISEC2_ResFlag && ISEC2_LatIncr > 0 )
-		  if ( ISEC2_LatIncr*(grid->ysize-1) > 180000 ) ISEC2_LatIncr = 0;
+                int recompinc = TRUE;
 
 		if ( ISEC2_ResFlag && ISEC2_LatIncr > 0 )
-		  grid->yinc = ISEC2_LatIncr * 0.001;
-		else
-		  grid->yinc = (ISEC2_LastLat - ISEC2_FirstLat) * 0.001 / (grid->ysize - 1);
+                  {
+                    if ( abs(ISEC2_LastLat - (ISEC2_FirstLat+ISEC2_LatIncr*(grid->ysize-1))) <= 2 )
+                      {
+                        recompinc = FALSE;
+                        grid->yinc = ISEC2_LatIncr * 0.001;
+                      }
+                  }
+
+		/* recompute yinc if necessary */
+                if ( recompinc ) grid->yinc = (ISEC2_LastLat - ISEC2_FirstLat) * 0.001 / (grid->ysize - 1);
 	      }
 	    grid->yfirst = ISEC2_FirstLat * 0.001;
 	    grid->ylast  = ISEC2_LastLat  * 0.001;
@@ -346,12 +349,10 @@ void cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid)
   grid->yvals = NULL;
   grid->type  = gridtype;
 }
-#endif
 
-#if  defined  (HAVE_LIBCGRIBEX)
 static
 void cgribexAddRecord(int streamID, int param, int *isec1, int *isec2, double *fsec2, double *fsec3,
-		      int *isec4, long recsize, off_t position, int datatype, int comptype, int lmv)
+		      int *isec4, long recsize, off_t position, int datatype, int comptype, int lmv, int iret)
 {
   int zaxistype;
   int gridID = CDI_UNDEFID, varID;
@@ -388,7 +389,7 @@ void cgribexAddRecord(int streamID, int param, int *isec1, int *isec2, double *f
   (*record).ilevel2  = level2;
   (*record).ltype    = ISEC1_LevelType;
 
-  cgribexGetGrid(streamptr, isec2, isec4, &grid);
+  cgribexGetGrid(streamptr, isec2, isec4, &grid, iret);
 
   gridID = varDefGrid(vlistID, grid, 0);
 
@@ -415,6 +416,14 @@ void cgribexAddRecord(int streamID, int param, int *isec1, int *isec2, double *f
 
   varDefCompType(varID, comptype);
 
+  if ( ISEC1_LocalFLag )
+    {
+      if      ( ISEC1_CenterID == 78  && isec1[36] == 253 ) // DWD local extension
+        varDefEnsembleInfo(varID, isec1[54], isec1[53], isec1[52]);
+      else if ( ISEC1_CenterID == 252 && isec1[36] ==   1 ) // MPIM local extension
+        varDefEnsembleInfo(varID, isec1[38], isec1[39], isec1[37]);
+    }
+
   if ( lmv ) varDefMissval(varID, FSEC3_MissVal);
 
   if ( varInqInst(varID) == CDI_UNDEFID )
@@ -455,9 +464,7 @@ void cgribexAddRecord(int streamID, int param, int *isec1, int *isec2, double *f
     Message("varID = %d  param = %d  zaxistype = %d  gridID = %d  levelID = %d",
 	    varID, param, zaxistype, gridID, levelID);
 }
-#endif
 
-#if  defined  (HAVE_LIBCGRIBEX)
 static
 void MCH_get_undef(int *isec1, double *undef_pds, double *undef_eps)
 {
@@ -482,18 +489,18 @@ void MCH_get_undef(int *isec1, double *undef_pds, double *undef_eps)
     }
   }
 }
-#endif
 
-#if  defined  (HAVE_LIBCGRIBEX)
 static
 void cgribexDecodeHeader(int *isec0, int *isec1, int *isec2, double *fsec2,
 			 int *isec3, double *fsec3, int *isec4, double *fsec4, 
-			 int *gribbuffer, int recsize, int *lmv)
+			 int *gribbuffer, int recsize, int *lmv, int *iret)
 {
-  int iret = 0, ipunp = 0, iword = 0;
+  int ipunp = 0, iword = 0;
+
+  memset(isec1, 0, 256*sizeof(int));
 
   gribExDP(isec0, isec1, isec2, fsec2, isec3, fsec3, isec4, fsec4,
-	   ipunp, (int *) gribbuffer, recsize, &iword, "J", &iret);
+	   ipunp, (int *) gribbuffer, recsize, &iword, "J", iret);
 
   *lmv = 0;
 
@@ -513,7 +520,7 @@ int cgribexScanTimestep1(int streamID)
 #if  defined  (HAVE_LIBCGRIBEX)
   int *isec0, *isec1, *isec2, *isec3, *isec4;
   double fsec2[512], fsec3[2], *fsec4 = NULL;
-  int lmv = 0;
+  int lmv = 0, iret = 0;
   off_t recpos = 0;
   unsigned char *gribbuffer = NULL;
   long buffersize = 0;
@@ -608,7 +615,7 @@ int cgribexScanTimestep1(int streamID)
 	}
 
       cgribexDecodeHeader(isec0, isec1, isec2, fsec2, isec3, fsec3, isec4, fsec4,
-			  (int *) gribbuffer, recsize, &lmv);
+			  (int *) gribbuffer, recsize, &lmv, &iret);
 
       param = cdiEncodeParam(ISEC1_Parameter, ISEC1_CodeTable, 255);
       if ( ISEC1_LevelType == 100 ) ISEC1_Level1 *= 100;
@@ -696,7 +703,7 @@ int cgribexScanTimestep1(int streamID)
 	Message("%4d %8d %4d  %8d %8d %6d", nrecs, (int)recpos, param, level1, vdate, vtime);
 
       cgribexAddRecord(streamID, param, isec1, isec2, fsec2, fsec3,
-		       isec4, recsize, recpos, datatype, comptype, lmv);
+		       isec4, recsize, recpos, datatype, comptype, lmv, iret);
     }
 
   streamptr->rtsteps = 1;
@@ -759,7 +766,7 @@ int cgribexScanTimestep1(int streamID)
 	  streamptr->ntsteps = 0;
 	  for ( varID = 0; varID < streamptr->nvars; varID++ )
 	    {
-	      vlistDefVarTime(vlistID, varID, TIME_CONSTANT);
+	      vlistDefVarTsteptype(vlistID, varID, TSTEP_CONSTANT);
 	    }
 	}
     }
@@ -777,7 +784,7 @@ int cgribexScanTimestep2(int streamID)
 #if  defined  (HAVE_LIBCGRIBEX)
   int *isec0, *isec1, *isec2, *isec3, *isec4;
   double fsec2[512], fsec3[2], *fsec4 = NULL;
-  int lmv = 0;
+  int lmv = 0, iret = 0;
   off_t recpos = 0;
   unsigned char *gribbuffer = NULL;
   long buffersize = 0;
@@ -817,7 +824,7 @@ int cgribexScanTimestep2(int streamID)
 
   gribbuffer = (unsigned char *) streamptr->record->buffer;
   buffersize = streamptr->record->buffersize;
-                                          
+
   tsID = streamptr->rtsteps;
   if ( tsID != 1 )
     Error("Internal problem! unexpeceted timestep %d", tsID+1);
@@ -833,14 +840,12 @@ int cgribexScanTimestep2(int streamID)
   streamptr->tsteps[1].nrecs = 0;
   for ( recID = 0; recID < nrecords; recID++ )
     streamptr->tsteps[1].recIDs[recID] = -1;
-      
+
   for ( recID = 0; recID < nrecords; recID++ )
     {
       varID = streamptr->tsteps[0].records[recID].varID;
-      streamptr->tsteps[tsID].records[recID].position = 
-	streamptr->tsteps[0].records[recID].position;
-      streamptr->tsteps[tsID].records[recID].size     = 
-	streamptr->tsteps[0].records[recID].size;
+      streamptr->tsteps[tsID].records[recID].position =	streamptr->tsteps[0].records[recID].position;
+      streamptr->tsteps[tsID].records[recID].size     =	streamptr->tsteps[0].records[recID].size;
     }
 
   rindex = 0;
@@ -876,7 +881,7 @@ int cgribexScanTimestep2(int streamID)
 	}
 
       cgribexDecodeHeader(isec0, isec1, isec2, fsec2, isec3, fsec3, isec4, fsec4,
-			  (int *) gribbuffer, recsize, &lmv);
+			  (int *) gribbuffer, recsize, &lmv, &iret);
 
       param = cdiEncodeParam(ISEC1_Parameter, ISEC1_CodeTable, 255);
       if ( ISEC1_LevelType == 100 ) ISEC1_Level1 *= 100;
@@ -911,7 +916,7 @@ int cgribexScanTimestep2(int streamID)
       if ( ISEC1_AvgNum )
 	{
 	  if (  taxis->numavg && warn_numavg &&
-		(taxis->numavg != ISEC1_AvgNum) )
+        	(taxis->numavg != ISEC1_AvgNum) )
 	    {
 	  /*
 	      Warning("Changing numavg from %d to %d not supported!",
@@ -959,7 +964,7 @@ int cgribexScanTimestep2(int streamID)
 	    {
 	      streamptr->tsteps[tsID].records[recID].used = TRUE;
 	      streamptr->tsteps[tsID].recIDs[rindex] = recID;
-	    }    
+	    }
 	}
       else
 	{
@@ -977,7 +982,7 @@ int cgribexScanTimestep2(int streamID)
 	    {
 	      streamptr->tsteps[tsID].records[recID].used = TRUE;
 	      streamptr->tsteps[tsID].recIDs[rindex] = recID;
-	    }    
+	    }
 	}
 
       if ( CDI_Debug )
@@ -1021,7 +1026,7 @@ int cgribexScanTimestep2(int streamID)
       if ( ! streamptr->tsteps[tsID].records[recID].used )
 	{
 	  varID = streamptr->tsteps[tsID].records[recID].varID;
-	  vlistDefVarTime(vlistID, varID, TIME_CONSTANT);
+          vlistDefVarTsteptype(vlistID, varID, TSTEP_CONSTANT);
 	}
       else
 	{
@@ -1056,7 +1061,7 @@ int cgribexScanTimestep(int streamID)
 #if  defined  (HAVE_LIBCGRIBEX)
   int *isec0, *isec1, *isec2, *isec3, *isec4;
   double fsec2[512], fsec3[2], *fsec4 = NULL;
-  int lmv = 0;
+  int lmv = 0, iret = 0;
   long recsize = 0;
   off_t recpos = 0;
   unsigned char *gribbuffer;
@@ -1142,8 +1147,8 @@ int cgribexScanTimestep(int streamID)
 	  rstatus = gribRead(fileID, gribbuffer, &readsize);
 	  if ( rstatus )
 	    {
-	      Error("Inconsistent timestep %d (GRIB record %d/%d)!", tsID+1, rindex+1,
-		    streamptr->tsteps[tsID].recordSize);
+	      Warning("Inconsistent timestep %d (GRIB record %d/%d)!", tsID+1, rindex+1,
+                      streamptr->tsteps[tsID].recordSize);
 	      break;
 	    }
 
@@ -1158,7 +1163,7 @@ int cgribexScanTimestep(int streamID)
 	    }
 
 	  cgribexDecodeHeader(isec0, isec1, isec2, fsec2, isec3, fsec3, isec4, fsec4,
-			      (int *) gribbuffer, recsize, &lmv);
+			      (int *) gribbuffer, recsize, &lmv, &iret);
 
 	  param = cdiEncodeParam(ISEC1_Parameter, ISEC1_CodeTable, 255);
 	  if ( ISEC1_LevelType == 100 ) ISEC1_Level1 *= 100;
@@ -1207,7 +1212,7 @@ int cgribexScanTimestep(int streamID)
 		  taxis->numavg = ISEC1_AvgNum;
 		}
 	    }
-	  
+
 	  datetime.date  = vdate;
 	  datetime.time  = vtime;
 	  compVar.param  = param;
@@ -1250,7 +1255,7 @@ int cgribexScanTimestep(int streamID)
 		  cdiParamToString(param, paramstr, sizeof(paramstr));
 
 		  if ( memcmp(&datetime, &datetime0, sizeof(DateTime)) != 0 ) break;
-		  
+
 		  if ( CDI_Debug )
 		    Warning("Param=%s level=%d already exist, skipped!", paramstr, level1);
 
@@ -1260,7 +1265,7 @@ int cgribexScanTimestep(int streamID)
 		{
 		  streamptr->tsteps[tsID].records[recID].used = TRUE;
 		  streamptr->tsteps[tsID].recIDs[rindex] = recID;
-		}    
+		}
 	    }
 
 	  if ( CDI_Debug )
@@ -1279,7 +1284,7 @@ int cgribexScanTimestep(int streamID)
 		      streamptr->tsteps[tsID].records[recID].ilevel, level1);
 	      Error("Invalid, unsupported or inconsistent record structure");
 	    }
-	  
+
 	  streamptr->tsteps[tsID].records[recID].position = recpos;
 	  streamptr->tsteps[tsID].records[recID].size = recsize;
 
@@ -1384,7 +1389,7 @@ int cgribexDecode(unsigned char *gribbuffer, int gribsize, double *data, int gri
 
 	  if ( gribsize <= 0 )
 	    Error("Decompression problem!");
-	  
+
 	  free(itmpbuffer);
 	}
       else
@@ -1422,7 +1427,6 @@ int cgribexDecode(unsigned char *gribbuffer, int gribsize, double *data, int gri
   return (status);
 }
 
-
 #if  defined  (HAVE_LIBCGRIBEX)
 static
 void cgribexDefInstitut(int *isec1, int vlistID, int varID)
@@ -1462,20 +1466,18 @@ static
 void cgribexDefParam(int *isec1, int param)
 {
   int pdis, pcat, pnum;
-  static int lwarn = TRUE;
 
   cdiDecodeParam(param, &pnum, &pcat, &pdis);
 
-  if ( pdis != 255 && lwarn )
+  if ( pnum < 0 ) pnum = -pnum;
+
+  if ( pdis != 255 )
     {
       char paramstr[32];
       cdiParamToString(param, paramstr, sizeof(paramstr));
-      Warning("Can not convert GRIB2 parameter (%s) to GRIB1!", paramstr);
-      lwarn = FALSE;
+      Warning("Can't convert GRIB2 parameter ID (%s) to GRIB1, set to %d.%d!", paramstr, pnum, pcat);
     }
 
-  if ( pnum < 0 ) pnum = -pnum;
-
   ISEC1_CodeTable = pcat;
   ISEC1_Parameter = pnum;
 }
@@ -1496,7 +1498,7 @@ int cgribexDefTimerange(int tsteptype, int factor, int calendar,
   cdiDecodeDate(vdate, &year, &month, &day);
   cdiDecodeTime(vtime, &hour, &minute, &second);
   encode_juldaysec(calendar, year, month, day, hour, minute, &julday2, &secofday2);
-  
+
   (void) julday_sub(julday1, secofday1, julday2, secofday2, &days, &secs);
 
   if ( !(int) fmod(days*86400.0 + secs, factor) )
@@ -1535,7 +1537,7 @@ int cgribexDefDateTime(int *isec1, int timeunit, int date, int time)
   century =  year / 100;
 
   ISEC1_Year = year - century*100;
-  
+
   if ( year < 0 )
     {
       century = -century;
@@ -1653,7 +1655,7 @@ void cgribexDefGrid(int *isec1, int *isec2, int *isec4, int gridID)
   memset(isec2, 0, 16*sizeof(int));
 
   ISEC1_Sec2Or3Flag = 128;
-  
+
   gridtype = gridInqType(gridID);
 
   ISEC1_GridDefinition = 255;
@@ -1697,8 +1699,7 @@ void cgribexDefGrid(int *isec1, int *isec2, int *isec4, int gridID)
       lcurvi = TRUE;
     }
 
-  ISEC2_Reduced = FALSE;
-
+  ISEC2_Reduced  = FALSE;
   ISEC2_ScanFlag = 0;
 
   switch (gridtype)
@@ -1776,8 +1777,8 @@ void cgribexDefGrid(int *isec1, int *isec2, int *isec4, int gridID)
 	    ISEC2_LastLon  = NINT(xlast*1000);
 	    ISEC2_LonIncr  = NINT(xinc*1000);
 	  }
-	if ( fabs(xinc*1000 - ISEC2_LonIncr) > FLT_EPSILON )
-	  ISEC2_LonIncr = 0;
+
+	// if ( fabs(xinc*1000 - ISEC2_LonIncr) > FLT_EPSILON ) ISEC2_LonIncr = 0;
 
 	if ( gridtype == GRID_GAUSSIAN || gridtype == GRID_GAUSSIAN_REDUCED )
           {
@@ -1788,8 +1789,7 @@ void cgribexDefGrid(int *isec1, int *isec2, int *isec4, int gridID)
 	else
 	  {
 	    ISEC2_LatIncr = NINT(yinc*1000);
-	    if ( fabs(yinc*1000 - ISEC2_LatIncr) > FLT_EPSILON )
-	      ISEC2_LatIncr = 0;
+	    // if ( fabs(yinc*1000 - ISEC2_LatIncr) > FLT_EPSILON ) ISEC2_LatIncr = 0;
 
 	    if ( ISEC2_LatIncr < 0 ) ISEC2_LatIncr = -ISEC2_LatIncr;
 	  }
@@ -1975,8 +1975,7 @@ void cgribexDefLevel(int *isec1, int *isec2, double *fsec2, int zaxisID, int lev
 	vctsize = zaxisInqVctSize(zaxisID);
 	if ( vctsize == 0 && warning )
 	  {
-	    Warning("VCT missing. ( param = %d, zaxisID = %d )",
-		    ISEC1_Parameter, zaxisID);
+	    Warning("VCT missing. ( param = %d, zaxisID = %d )", ISEC1_Parameter, zaxisID);
 	    warning = 0;
 	  }
 	if ( vctsize > 255 )
@@ -2066,17 +2065,30 @@ void cgribexDefLevel(int *isec1, int *isec2, double *fsec2, int zaxisID, int lev
       }
     case ZAXIS_DEPTH_BELOW_LAND:
       {
+	char units[128];
+	double factor;
+
+	zaxisInqUnits(zaxisID, units);
+
+        if      ( memcmp(units, "mm", 2) == 0 ) factor =   0.1;
+        else if ( memcmp(units, "cm", 2) == 0 ) factor =   1;
+        else if ( memcmp(units, "dm", 2) == 0 ) factor =  10;
+        else                                    factor = 100; // meter
+
 	if ( zaxisInqLbounds(zaxisID, NULL) && zaxisInqUbounds(zaxisID, NULL) )
 	  {
+            double level1, level2;
+            level1 = zaxisInqLbound(zaxisID, levelID);
+            level2 = zaxisInqUbound(zaxisID, levelID);
 	    ISEC1_LevelType = GRIB1_LTYPE_LANDDEPTH_LAYER;
-	    ISEC1_Level1    = (int) zaxisInqLbound(zaxisID, levelID);
-	    ISEC1_Level2    = (int) zaxisInqUbound(zaxisID, levelID);
+	    ISEC1_Level1    = (int) (level1*factor);
+	    ISEC1_Level2    = (int) (level2*factor);
 	  }
 	else
 	  {
 	    level = zaxisInqLevel(zaxisID, levelID);
 
-	    ilevel = (int) level;
+	    ilevel = (int) (level*factor);
 	    ISEC1_LevelType = GRIB1_LTYPE_LANDDEPTH;
 	    ISEC1_Level1    = ilevel;
 	    ISEC1_Level2    = 0;
@@ -2152,11 +2164,38 @@ void cgribexDefaultSec4(int *isec4)
 
   for ( i = 2; i <= 10; ++i ) isec4[i] = 0;
 }
+
+static
+void cgribexDefEnsembleVar(int *isec1, int vlistID, int varID)
+{
+  int ensID, ensCount, forecast_type;
+
+  /* For Ensemble info  */
+
+  //Put1Byte(isec1[36]);        /* MPIM local GRIB use definition identifier  */
+                                /*    (extension identifier)                  */
+  //Put1Byte(isec1[37]);        /* type of ensemble forecast                  */
+  //Put2Byte(isec1[38]);        /* individual ensemble member                 */
+  //Put2Byte(isec1[39]);        /* number of forecasts in ensemble            */
+
+  if ( vlistInqVarEnsemble(vlistID, varID, &ensID, &ensCount, &forecast_type) )
+    {
+      if ( ISEC1_CenterID == 252 )
+        {
+          ISEC1_LocalFLag = 1;
+          isec1[36] = 1;
+
+          isec1[37] =  forecast_type;
+          isec1[38] =  ensID;
+          isec1[39] =  ensCount;
+        }
+    }
+}
 #endif
 
 
 size_t cgribexEncode(int varID, int levelID, int vlistID, int gridID, int zaxisID,
-		     int vdate, int vtime, int tsteptype, int numavg, 
+		     int vdate, int vtime, int tsteptype, int numavg,
 		     long datasize, const double *data, int nmiss, unsigned char *gribbuffer, size_t gribbuffersize)
 {
   size_t nbytes = 0;
@@ -2168,7 +2207,7 @@ size_t cgribexEncode(int varID, int levelID, int vlistID, int gridID, int zaxisI
   int datatype;
   int param;
 
-  memset(isec1, 0, 32*sizeof(int));
+  memset(isec1, 0, 256*sizeof(int));
   fsec2[0] = 0; fsec2[1] = 0;
 
   gribsize = gribbuffersize / sizeof(int);
@@ -2189,6 +2228,8 @@ size_t cgribexEncode(int varID, int levelID, int vlistID, int gridID, int zaxisI
   cgribexDefLevel(isec1, isec2, fsec2, zaxisID, levelID);
   cgribexDefMask(isec3);
 
+  cgribexDefEnsembleVar(isec1, vlistID, varID);
+
   ISEC4_NumValues = gridInqSize(gridID);
   ISEC4_NumBits   = grbBitsPerValue(datatype);
 
