@@ -218,7 +218,6 @@ int extCopyRecord(int streamID2, int streamID1)
 
 int extDefRecord(int streamID)
 {
-  int fileID;
   int gridID;
   int header[4];
   int status = 0;
@@ -230,7 +229,6 @@ int extDefRecord(int streamID)
 
   stream_check_ptr(__func__, streamptr);
 
-  fileID   = streamInqFileID(streamID);
   gridID   = streamptr->record->gridID;
   extp     = streamptr->record->extp;
 
@@ -309,7 +307,7 @@ void extAddRecord(int streamID, int param, int level, int xysize,
   leveltype = ZAXIS_GENERIC;
 
   varAddRecord(recID, param, gridID, leveltype, 0, level, 0,
-	       extInqDatatype(prec, number), &varID, &levelID, 0, 0, 0, NULL, NULL, NULL);
+	       extInqDatatype(prec, number), &varID, &levelID, UNDEFID, 0, 0, NULL, NULL, NULL);
 
   (*record).varID   = varID;
   (*record).levelID = levelID;
@@ -486,7 +484,7 @@ void extScanTimestep1(int streamID)
 	  streamptr->ntsteps = 0;
 	  for ( varID = 0; varID < streamptr->nvars; varID++ )
 	    {
-	      vlistDefVarTime(vlistID, varID, TIME_CONSTANT);
+	      vlistDefVarTsteptype(vlistID, varID, TSTEP_CONSTANT);
 	    }
 	}
     }
@@ -498,7 +496,7 @@ int extScanTimestep2(int streamID)
   int header[4];
   int status;
   int fileID;
-  int rxysize = 0;
+  // int rxysize = 0;
   int param = 0;
   int rcode = 0, rlevel = 0, vdate = 0, vtime = 0;
   int tsID;
@@ -565,7 +563,7 @@ int extScanTimestep2(int streamID)
       vtime  = 0;
       rcode  = header[1];
       rlevel = header[2];
-      rxysize = header[3];
+      // rxysize = header[3];
 
       param = cdiEncodeParam(rcode, 255, 255);
 
@@ -632,7 +630,7 @@ int extScanTimestep2(int streamID)
       if ( ! streamptr->tsteps[tsID].records[recID].used )
 	{
 	  varID = streamptr->tsteps[tsID].records[recID].varID;
-	  vlistDefVarTime(vlistID, varID, TIME_CONSTANT);
+          vlistDefVarTsteptype(vlistID, varID, TSTEP_CONSTANT);
 	}
       else
 	{
@@ -687,7 +685,7 @@ int extScanTimestep(int streamID)
   int status;
   int fileID;
   int tsID;
-  int rxysize = 0;
+  // int rxysize = 0;
   int param = 0;
   int rcode = 0, rlevel = 0, vdate = 0, vtime = 0;
   long recsize = 0;
@@ -750,7 +748,7 @@ int extScanTimestep(int streamID)
 	  vtime  = 0;
 	  rcode  = header[1];
 	  rlevel = header[2];
-	  rxysize = header[3];
+	  // rxysize = header[3];
 
 	  param = cdiEncodeParam(rcode, 255, 255);
 
@@ -856,7 +854,6 @@ void extReadVarDP(int streamID, int varID, double *data, int *nmiss)
   int tsid;
   int recID;
   int i;
-  int status;
   double missval;
   extrec_t *extp;
   stream_t *streamptr;
@@ -882,7 +879,7 @@ void extReadVarDP(int streamID, int varID, double *data, int *nmiss)
       recID = streamptr->vars[varID].level[levID];
       recpos = streamptr->tsteps[tsid].records[recID].position;
       fileSetPos(fileID, recpos, SEEK_SET);
-      status = extRead(fileID, extp);
+      extRead(fileID, extp);
       extInqHeader(extp, header);
       extInqDataDP(extp, &data[levID*gridsize]);
     }
@@ -919,7 +916,6 @@ void extReadVarSliceDP(int streamID, int varID, int levID, double *data, int *nm
   int tsid;
   int recID;
   int i;
-  int status;
   double missval;
   extrec_t *extp;
   stream_t *streamptr;
@@ -944,7 +940,7 @@ void extReadVarSliceDP(int streamID, int varID, int levID, double *data, int *nm
   recID = streamptr->vars[varID].level[levID];
   recpos = streamptr->tsteps[tsid].records[recID].position;
   fileSetPos(fileID, recpos, SEEK_SET);
-  status = extRead(fileID, extp);
+  extRead(fileID, extp);
   extInqHeader(extp, header);
   extInqDataDP(extp, data);
  
