@@ -14,6 +14,10 @@
 #include "cgribex.h"  /* gribZip gribGetZip gribGinfo */
 #include "gribapi.h"
 
+#ifdef USE_MPI
+#include "pio.h"
+#include "pio_comm.h"
+#endif
 
 
 int grib1ltypeToZaxisType(int grib_ltype)
@@ -106,8 +110,6 @@ int grbInqRecord(int streamID, int *varID, int *levelID)
 
 int grbDefRecord(int streamID)
 {
-  int fileID;
-  int gridID;
   int status = 0;
   stream_t *streamptr;
 
@@ -115,9 +117,6 @@ int grbDefRecord(int streamID)
 
   stream_check_ptr(__func__, streamptr);
 
-  fileID = streamInqFileID(streamID);
-  gridID = streamptr->record->gridID;
-
   return (status);
 }
 
@@ -484,7 +483,7 @@ size_t grbSzip(int filetype, unsigned char *gribbuffer, size_t gribbuffersize)
   buffer = (unsigned char *) malloc(buffersize);
 
   /*  memcpy(buffer, gribbuffer, gribbuffersize); */
-  
+
   if ( filetype == FILETYPE_GRB )
     {
       nbytes = gribZip(gribbuffer, (long) gribbuffersize, buffer, (long) buffersize);
@@ -495,7 +494,7 @@ size_t grbSzip(int filetype, unsigned char *gribbuffer, size_t gribbuffersize)
       lszip_warn = 0;
       nbytes = gribbuffersize;
     }
-      
+
   free(buffer);
 
   return (nbytes);
@@ -555,8 +554,13 @@ int grbWriteVarSliceDP(int streamID, int varID, int levelID, const double *data,
   else
 #endif
     {
+#if defined (GRIBCONTAINER2D)
+      gribContainer_t **gribContainers =  (gribContainer_t **) streamptr->gribContainers;
+      gc = (void *) &gribContainers[varID][levelID];
+#else
       gribContainer_t *gribContainers =  (gribContainer_t *) streamptr->gribContainers;
       gc = (void *) &gribContainers[varID];
+#endif
     }
 
   if ( streamptr->comptype == COMPRESS_JPEG )
@@ -578,7 +582,15 @@ int grbWriteVarSliceDP(int streamID, int varID, int levelID, const double *data,
   if ( streamptr->comptype == COMPRESS_SZIP )
     nbytes = grbSzip(filetype, gribbuffer, nbytes);
 
-  nwrite = fileWrite(fileID, gribbuffer, nbytes);
+  /* begin deike */
+#ifdef USE_MPI
+  if ( commInqIOMode () != PIO_NONE )
+    nwrite = pioFileWrite ( fileID, tsID, gribbuffer, nbytes );
+  else
+#endif
+  /* end deike */
+    nwrite = fileWrite(fileID, gribbuffer, nbytes);
+
   if ( nwrite != nbytes ) perror(__func__);
 
   if ( gribbuffer ) free(gribbuffer);
@@ -649,7 +661,7 @@ int grbCopyRecord(int streamID2, int streamID1)
   nbytes = recsize;
 
   izip = gribGetZip(recsize, gribbuffer, &unzipsize);
-  
+
   if ( izip == 0 )
     if ( streamptr2->comptype == COMPRESS_SZIP )
       nbytes = grbSzip(filetype, gribbuffer, nbytes);
@@ -689,7 +701,6 @@ void streamInqGinfo(int streamID, int *intnum, float *fltnum)
   int recID, vrecID, tsID;
   int filetype;
   void *gribbuffer;
-  long recsize;
   long gribbuffersize;
   off_t recpos;
   int zip;
@@ -707,7 +718,6 @@ void streamInqGinfo(int streamID, int *intnum, float *fltnum)
       vrecID  = streamptr->tsteps[tsID].curRecID;
       recID   = streamptr->tsteps[tsID].recIDs[vrecID];
       recpos  = streamptr->tsteps[tsID].records[recID].position;
-      recsize = streamptr->tsteps[tsID].records[recID].size;
       zip     = streamptr->tsteps[tsID].records[recID].zip;
 
       gribbuffer = streamptr->record->buffer;
