@@ -209,12 +209,12 @@ int iegGetZaxisType(int iegleveltype)
 }
 
 
-void iegDefTime(int *pdb, int date, int time, int timeID)
+void iegDefTime(int *pdb, int date, int time, int taxisID)
 {
   int year, month, day, hour, minute, second;
   int timetype = -1;
 
-  if ( timeID != -1 ) timetype = taxisInqType(timeID);
+  if ( taxisID != -1 ) timetype = taxisInqType(taxisID);
 
   if ( timetype == TAXIS_ABSOLUTE || timetype == TAXIS_RELATIVE )
     {
@@ -579,7 +579,7 @@ int iegCopyRecord(int streamID2, int streamID1)
 int iegDefRecord(int streamID)
 {
   int status = 0;
-  int fileID, vlistID;
+  int vlistID;
   int gridID;
   int date, time;
   int datatype;
@@ -594,7 +594,6 @@ int iegDefRecord(int streamID)
 
   stream_check_ptr(__func__, streamptr);
 
-  fileID  = streamInqFileID(streamID);
   vlistID = streamInqVlist(streamID);
   iegp    = streamptr->record->iegp;
   byteorder = streamptr->byteorder;
@@ -802,7 +801,7 @@ void iegAddRecord(int streamID, int param, int *pdb, int *gdb, double *vct,
   datatype = iegInqDatatype(prec);
 
   varAddRecord(recID, param, gridID, leveltype, lbounds, level1, level2,
-	       datatype, &varID, &levelID, 0, 0, 0, NULL, NULL, NULL);
+	       datatype, &varID, &levelID, UNDEFID, 0, 0, NULL, NULL, NULL);
 
   (*record).varID   = varID;
   (*record).levelID = levelID;
@@ -1003,7 +1002,7 @@ void iegScanTimestep1(int streamID)
 	  streamptr->ntsteps = 0;
 	  for ( varID = 0; varID < streamptr->nvars; varID++ )
 	    {
-	      vlistDefVarTime(vlistID, varID, TIME_CONSTANT);
+	      vlistDefVarTsteptype(vlistID, varID, TSTEP_CONSTANT);
 	    }
 	}
     }
@@ -1153,7 +1152,7 @@ int iegScanTimestep2(int streamID)
       if ( ! streamptr->tsteps[tsID].records[recID].used )
 	{
 	  varID = streamptr->tsteps[tsID].records[recID].varID;
-	  vlistDefVarTime(vlistID, varID, TIME_CONSTANT);
+          vlistDefVarTsteptype(vlistID, varID, TSTEP_CONSTANT);
 	}
       else
 	{
@@ -1376,7 +1375,6 @@ void iegReadVarDP(int streamID, int varID, double *data, int *nmiss)
   int tsid;
   int recID;
   int i;
-  int status;
   double missval;
   iegrec_t *iegp;
   stream_t *streamptr;
@@ -1402,7 +1400,7 @@ void iegReadVarDP(int streamID, int varID, double *data, int *nmiss)
       recID = streamptr->vars[varID].level[levID];
       recpos = streamptr->tsteps[tsid].records[recID].position;
       fileSetPos(fileID, recpos, SEEK_SET);
-      status = iegRead(fileID, iegp);
+      iegRead(fileID, iegp);
       iegInqDataDP(iegp, &data[levID*gridsize]);
     }
   fileSetPos(fileID, currentfilepos, SEEK_SET);
@@ -1425,7 +1423,6 @@ void iegReadVarSliceDP(int streamID, int varID, int levID, double *data, int *nm
   int tsid;
   int recID;
   int i;
-  int status;
   double missval;
   iegrec_t *iegp;
   stream_t *streamptr;
@@ -1450,7 +1447,7 @@ void iegReadVarSliceDP(int streamID, int varID, int levID, double *data, int *nm
   recID = streamptr->vars[varID].level[levID];
   recpos = streamptr->tsteps[tsid].records[recID].position;
   fileSetPos(fileID, recpos, SEEK_SET);
-  status = iegRead(fileID, iegp);
+  iegRead(fileID, iegp);
   iegInqDataDP(iegp, data);
 
   fileSetPos(fileID, currentfilepos, SEEK_SET);
@@ -1536,11 +1533,11 @@ void iegWriteVarSliceDP(int streamID, int varID, int levID, const double *data)
   int fileID;
   int gridID;
   int zaxisID;
-  double level;
+  /* double level; */
   int datatype;
-  int tsID;
+  /* int tsID; */
   int vlistID;
-  int param, date, time, datasize;
+  /* int param, date, time, datasize; */
   iegrec_t *iegp;
   stream_t *streamptr;
 
@@ -1549,18 +1546,18 @@ void iegWriteVarSliceDP(int streamID, int varID, int levID, const double *data)
   iegp     = streamptr->record->iegp;
   vlistID  = streamInqVlist(streamID);
   fileID   = streamInqFileID(streamID);
-  tsID     = streamptr->curTsID;
+  /* tsID     = streamptr->curTsID; */
   gridID   = vlistInqVarGrid(vlistID, varID);
   zaxisID  = vlistInqVarZaxis(vlistID, varID);
-  level    = zaxisInqLevel(zaxisID, levID);
+  /* level    = zaxisInqLevel(zaxisID, levID); */
 
   if ( CDI_Debug )
     Message("gridID = %d zaxisID = %d", gridID, zaxisID);
 
-  param = vlistInqVarParam(vlistID, varID);
-  date = streamptr->tsteps[tsID].taxis.vdate;
-  time = streamptr->tsteps[tsID].taxis.vtime;
-  datasize = gridInqSize(gridID);
+  /* param = vlistInqVarParam(vlistID, varID); */
+  /* date = streamptr->tsteps[tsID].taxis.vdate; */
+  /* time = streamptr->tsteps[tsID].taxis.vtime; */
+  /* datasize = gridInqSize(gridID); */
 
   datatype = vlistInqVarDatatype(vlistID, varID);
 
