@@ -126,7 +126,7 @@ void cdiInitRecord(int streamID)
 
 void streamInqRecord(int streamID, int *varID, int *levelID)
 {
-  int rec = 0;
+  /* int rec = 0; */
   int recID, tsID, rindex;
   int lindex;
   stream_t *streamptr;
@@ -165,8 +165,8 @@ void streamInqRecord(int streamID, int *varID, int *levelID)
   streamptr->curTsID = tsID;
   streamptr->tsteps[tsID].curRecID = rindex;
 
-  rec = recID + 1;
   /*
+  rec = recID + 1;
   filetype = streamptr->filetype;
 
   switch ( filetype )
@@ -360,7 +360,7 @@ void streamReadRecord(int streamID, double *data, int *nmiss)
 }
 
 
-void streamWriteRecord(int streamID, const double *data, int nmiss)
+void stream_write_record(int streamID, int memtype, const void *data, int nmiss)
 {
   int status = 0;
   int filetype;
@@ -380,6 +380,7 @@ void streamWriteRecord(int streamID, const double *data, int nmiss)
     case FILETYPE_GRB:
     case FILETYPE_GRB2:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("grbWriteRecord not implemented for memtype float!");
         status = grbWriteRecord(streamID, data, nmiss);
 	break;
       }
@@ -387,6 +388,7 @@ void streamWriteRecord(int streamID, const double *data, int nmiss)
 #if  defined  (HAVE_LIBSERVICE)
     case FILETYPE_SRV:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("srvWriteRecord not implemented for memtype float!");
         status = srvWriteRecord(streamID, data);
 	break;
       }
@@ -394,6 +396,7 @@ void streamWriteRecord(int streamID, const double *data, int nmiss)
 #if  defined  (HAVE_LIBEXTRA)
     case FILETYPE_EXT:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("extWriteRecord not implemented for memtype float!");
         status = extWriteRecord(streamID, data);
 	break;
       }
@@ -401,6 +404,7 @@ void streamWriteRecord(int streamID, const double *data, int nmiss)
 #if  defined  (HAVE_LIBIEG)
     case FILETYPE_IEG:
       {
+        if ( memtype == MEMTYPE_FLOAT ) Error("iegWriteRecord not implemented for memtype float!");
         status = iegWriteRecord(streamID, data);
 	break;
       }
@@ -411,7 +415,7 @@ void streamWriteRecord(int streamID, const double *data, int nmiss)
     case FILETYPE_NC4:
     case FILETYPE_NC4C:
       {
-	cdfWriteRecord(streamID, data, nmiss);
+	cdf_write_record(streamID, memtype, data, nmiss);
 	break;
       }
 #endif
@@ -424,11 +428,21 @@ void streamWriteRecord(int streamID, const double *data, int nmiss)
 }
 
 
+void streamWriteRecord(int streamID, const double *data, int nmiss)
+{
+  stream_write_record(streamID, MEMTYPE_DOUBLE, (const void *) data, nmiss);
+}
+
+void streamWriteRecordF(int streamID, const float *data, int nmiss)
+{
+  stream_write_record(streamID, MEMTYPE_FLOAT, (const void *) data, nmiss);
+}
+
+
 void streamCopyRecord(int streamID2, int streamID1)
 {
   int status = 0;
   int filetype = CDI_UNDEFID, filetype1, filetype2;
-  int byteorder1, byteorder2;
   stream_t *streamptr1;
   stream_t *streamptr2;
 
@@ -440,10 +454,8 @@ void streamCopyRecord(int streamID2, int streamID1)
 
   filetype1 = streamptr1->filetype;
   filetype2 = streamptr2->filetype;
-  byteorder1 = streamptr1->byteorder;
-  byteorder2 = streamptr2->byteorder;
 
-  if ( filetype1  == filetype2 ) filetype = filetype2;
+  if ( filetype1 == filetype2 ) filetype = filetype2;
 
   switch (filetype)
     {
@@ -523,7 +535,7 @@ void cdiCreateRecords(int streamID, int tsID)
 	{
 	  varID = streamptr->tsteps[0].records[recID].varID;
 	  if ( varID != -1 ) /* varID = -1 for write mode !!! */
-	    if ( vlistInqVarTime(vlistID, varID) == TIME_CONSTANT )
+	    if ( vlistInqVarTsteptype(vlistID, varID) == TSTEP_CONSTANT )
 	      continue;
 	  nrecords++;
 	}
@@ -555,7 +567,7 @@ void cdiCreateRecords(int streamID, int tsID)
 	{
 	  varID = streamptr->tsteps[0].records[recID].varID;
 	  if ( varID != -1 ) /* varID = -1 for write mode !!! */
-	    if ( vlistInqVarTime(vlistID, varID) == TIME_VARIABLE )
+	    if ( vlistInqVarTsteptype(vlistID, varID) != TSTEP_CONSTANT )
 	      {
 		streamptr->tsteps[tsID].records[recID].position = CDI_UNDEFID;
 		streamptr->tsteps[tsID].records[recID].size     = 0;
