@@ -3,12 +3,21 @@
 #endif
 
 #include <string.h>
+#ifdef USE_MPI
+#include <mpi.h>
+#endif
 
 #include "dmemory.h"
 
 #include "cdi.h"
+#include "taxis.h"
 #include "stream_int.h"
 #include "calendar.h"
+#include "pio_util.h"
+#include "namespace.h"
+#include "resource_handle.h"
+#include "pio_rpc.h"
+#include "pio_util.h"
 
 extern int cdiDefaultCalendar;
 
@@ -24,13 +33,33 @@ char *Timeunits[] = {
   "days",
   "months",
   "years",
-  "quarter",
+  "quarters",
   "3hours",
   "6hours",
   "12hours",
 };
 
 
+static int    taxisCompareP    ( void * taxisptr1, void * taxisptr2 );
+static void   taxisDestroyP    ( void * taxisptr );
+static void   taxisPrintP      ( void * taxisptr, FILE * fp );
+#ifdef USE_MPI
+static int    taxisGetPackSize ( void * taxisptr, MPI_Comm comm );
+static void   taxisPack        ( void * taxisptr, void *buf, int size,
+				 int *position, MPI_Comm comm );
+static int    taxisTxCode      ( void );
+#endif
+
+resOps taxisOps = { taxisCompareP, taxisDestroyP, taxisPrintP
+#ifdef USE_MPI
+                    , taxisGetPackSize, taxisPack, taxisTxCode
+#endif
+};
+
+
+static int  TAXIS_Debug = 0;   /* If set to 1, debugging */
+
+
 char *tunitNamePtr(int unitID)
 {
   char *name;
@@ -44,6 +73,7 @@ char *tunitNamePtr(int unitID)
   return (name);
 }
 
+#if 0
 static
 void taxis_defaults(void)
 {
@@ -52,9 +82,9 @@ void taxis_defaults(void)
   timeunit = getenv("TIMEUNIT");
   if ( timeunit )
     {
-      if ( strcmp(timeunit, "minute") == 0 )
+      if ( strcmp(timeunit, "minutes") == 0 )
 	DefaultTimeUnit = TUNIT_MINUTE;
-      else if ( strcmp(timeunit, "hour") == 0 )
+      else if ( strcmp(timeunit, "hours") == 0 )
 	DefaultTimeUnit = TUNIT_HOUR;
       else if ( strcmp(timeunit, "3hours") == 0 )
 	DefaultTimeUnit = TUNIT_3HOURS;
@@ -62,264 +92,68 @@ void taxis_defaults(void)
 	DefaultTimeUnit = TUNIT_6HOURS;
       else if ( strcmp(timeunit, "12hours") == 0 )
 	DefaultTimeUnit = TUNIT_12HOURS;
-      else if ( strcmp(timeunit, "day") == 0 )
+      else if ( strcmp(timeunit, "days") == 0 )
 	DefaultTimeUnit = TUNIT_DAY;
-      else if ( strcmp(timeunit, "month") == 0 )
+      else if ( strcmp(timeunit, "months") == 0 )
 	DefaultTimeUnit = TUNIT_MONTH;
-      else if ( strcmp(timeunit, "year") == 0 )
+      else if ( strcmp(timeunit, "years") == 0 )
 	DefaultTimeUnit = TUNIT_YEAR;
       else
 	Warning("Unsupported TIMEUNIT %s!", timeunit);
     }
 }
-
-
-static int  TAXIS_Debug = 0;   /* If set to 1, debugging */
-
-static int _taxis_min = MIN_TAXES;
-static int _taxis_max = MAX_TAXES;
-
-static void taxis_initialize(void);
-
-static int _taxis_init = FALSE;
-
-#if  defined  (HAVE_LIBPTHREAD)
-#  include <pthread.h>
-
-static pthread_once_t  _taxis_init_thread = PTHREAD_ONCE_INIT;
-static pthread_mutex_t _taxis_mutex;
-
-#  define TAXIS_LOCK()         pthread_mutex_lock(&_taxis_mutex)
-#  define TAXIS_UNLOCK()       pthread_mutex_unlock(&_taxis_mutex)
-#  define TAXIS_INIT()	      \
-   if ( _taxis_init == FALSE ) pthread_once(&_taxis_init_thread, taxis_initialize)
-
-#else
-
-#  define TAXIS_LOCK()
-#  define TAXIS_UNLOCK()
-#  define TAXIS_INIT()	      \
-   if ( _taxis_init == FALSE ) taxis_initialize()
-
 #endif
 
-
-typedef struct _taxisPtrToIdx {
-  int      idx;
-  int      next;
-  taxis_t *ptr;
-} taxisPtrToIdx;
-
-
-static taxisPtrToIdx *_taxisList  = NULL;
-static int            _taxisAvail = -1;
-
 static
-void taxis_list_new(void)
+void taxisDefaultValue ( taxis_t *taxisptr )
 {
-  assert(_taxisList == NULL);
-
-  _taxisList = (taxisPtrToIdx *) malloc(_taxis_min*sizeof(taxisPtrToIdx));
+  taxisptr->self        = CDI_UNDEFID;
+  taxisptr->used        = FALSE;
+  taxisptr->type        = DefaultTimeType;
+  taxisptr->vdate       = 0;
+  taxisptr->vtime       = 0;
+  taxisptr->rdate       = CDI_UNDEFID;
+  taxisptr->rtime       = 0;
+  taxisptr->calendar    = cdiDefaultCalendar;
+  taxisptr->unit        = DefaultTimeUnit;
+  taxisptr->numavg      = 0;
+  taxisptr->has_bounds  = FALSE;
+  taxisptr->vdate_lb    = 0;
+  taxisptr->vtime_lb    = 0;
+  taxisptr->vdate_ub    = 0;
+  taxisptr->vtime_ub    = 0;
+  taxisptr->name        = NULL;
+  taxisptr->longname    = NULL;
 }
 
 static
-void taxis_list_delete(void)
-{
-  if ( _taxisList ) free(_taxisList);
-}
-
-static
-void taxis_init_pointer(void)
-{
-  int  i;
-  
-  for ( i = 0; i < _taxis_min; ++i )
-    {
-      _taxisList[i].idx  = i;
-      _taxisList[i].next = i + 1;
-      _taxisList[i].ptr  = NULL;
-    }
-
-  _taxisList[_taxis_min-1].next = -1;
-
-  _taxisAvail = 0;
-}
-
-static
-void taxis_list_extend(void)
-{
-  int ntaxis;
-  int i;
-
-  assert(_taxisList != NULL);
-
-  ntaxis = _taxis_min + MIN_TAXES;
-
-  if ( ntaxis <= _taxis_max)
-    {
-      _taxisList = (taxisPtrToIdx *) realloc(_taxisList, ntaxis*sizeof(taxisPtrToIdx));
-  
-      for ( i = _taxis_min; i < ntaxis; ++i )
-	{
-	  _taxisList[i].idx  = i;
-	  _taxisList[i].next = i + 1;
-	  _taxisList[i].ptr  = NULL;
-	}
-
-      _taxisAvail = _taxis_min;
-      _taxisList[_taxis_min-1].next = _taxis_min;
-      _taxis_min = ntaxis;
-      _taxisList[_taxis_min-1].next = -1;
-    }
-  else
-    Warning("Too many open taxes (limit is %d)!", _taxis_max);
-}
-
-static
-taxis_t *taxis_to_pointer(int idx)
-{
-  taxis_t *taxisptr = NULL;
-
-  TAXIS_INIT();
-
-  if ( idx >= 0 && idx < _taxis_min )
-    {
-      TAXIS_LOCK();
-
-      taxisptr = _taxisList[idx].ptr;
-
-      TAXIS_UNLOCK();
-    }
-  else
-    Error("taxis index %d undefined!", idx);
-
-  return (taxisptr);
-}
-
-
-/* Create an index from a pointer */
-static
-int taxis_from_pointer(taxis_t *ptr)
-{
-  int      idx = -1;
-
-  if ( ptr )
-    {
-      TAXIS_LOCK();
-
-      if ( _taxisAvail < 0 ) taxis_list_extend();
-
-      if ( _taxisAvail >= 0 )
-	{
-	  taxisPtrToIdx *newptr;
-
-	  newptr       = &_taxisList[_taxisAvail];
-	  _taxisAvail  = newptr->next;
-	  newptr->next = -1;
-	  idx	       = newptr->idx;
-	  newptr->ptr  = ptr;
-      
-	  if ( TAXIS_Debug )
-	    Message("Pointer %p has idx %d from taxis list", ptr, idx);
-	}
-
-      TAXIS_UNLOCK();
-    }
-  else
-    Error("Internal problem (pointer %p undefined)", ptr);
-
-  return (idx);
-}
-
-static
-void taxis_init_ptr(taxis_t *taxisptr)
-{
-  taxisptr->self       = -1;
-
-  taxisptr->used       = FALSE;
-  taxisptr->type       = DefaultTimeType;
-  taxisptr->vdate      = 0;
-  taxisptr->vtime      = 0;
-  taxisptr->rdate      = CDI_UNDEFID;
-  taxisptr->rtime      = 0;
-  taxisptr->calendar   = cdiDefaultCalendar;
-  taxisptr->unit       = DefaultTimeUnit;
-  taxisptr->numavg     = 0;
-  taxisptr->has_bounds = FALSE;
-  taxisptr->vdate_lb   = 0;
-  taxisptr->vtime_lb   = 0;
-  taxisptr->vdate_ub   = 0;
-  taxisptr->vtime_ub   = 0;
-}
-
-static
-void taxis_init_entry(taxis_t *taxisptr)
-{
-  taxis_init_ptr(taxisptr);
-
-  taxisptr->self       = taxis_from_pointer(taxisptr);
-}
-
-static
-taxis_t *taxis_new_entry(void)
+taxis_t *taxisNewEntry(void)
 {
   taxis_t *taxisptr;
 
-  taxisptr = (taxis_t *) malloc(sizeof(taxis_t));
+  taxisptr = (taxis_t *) xmalloc ( sizeof ( taxis_t ));
 
-  if ( taxisptr ) taxis_init_entry(taxisptr);
+  taxisDefaultValue ( taxisptr );
+  taxisptr->self = reshPut (( void * ) taxisptr, &taxisOps );
 
   return (taxisptr);
 }
 
 static
-void taxis_delete_entry(taxis_t *taxisptr)
-{
-  int idx;
-
-  idx = taxisptr->self;
-
-  TAXIS_LOCK();
-
-  free(taxisptr);
-
-  _taxisList[idx].next = _taxisAvail;
-  _taxisList[idx].ptr  = 0;
-  _taxisAvail          = idx;
-
-  TAXIS_UNLOCK();
-
-  if ( TAXIS_Debug )
-    Message("Removed idx %d from taxis list", idx);
-}
-
-static
-void taxis_initialize(void)
+void taxisInit (void)
 {
+  static int taxisInitialized = 0;
   char *env;
 
-#if  defined  (HAVE_LIBPTHREAD)
-  /* initialize global API mutex lock */
-  pthread_mutex_init(&_taxis_mutex, NULL);
-#endif
+  if ( taxisInitialized ) return;
+
+  taxisInitialized = 1; 
 
   env = getenv("TAXIS_DEBUG");
   if ( env ) TAXIS_Debug = atoi(env);
-
-  taxis_list_new();
-  atexit(taxis_list_delete);
-
-  TAXIS_LOCK();
-
-  taxis_init_pointer();
-  
-  TAXIS_UNLOCK();
-
-  _taxis_init = TRUE;
-
-  taxis_defaults();
 }
 
+#if 0
 static
 void taxis_copy(taxis_t *taxisptr2, taxis_t *taxisptr1)
 {
@@ -329,6 +163,7 @@ void taxis_copy(taxis_t *taxisptr2, taxis_t *taxisptr1)
   memcpy(taxisptr2, taxisptr1, sizeof(taxis_t));
   taxisptr2->self = taxisID2;
 }
+#endif
 
 static
 void taxis_check_ptr(const char *caller, taxis_t *taxisptr)
@@ -377,10 +212,9 @@ int taxisCreate(int taxistype)
   if ( CDI_Debug )
     Message("taxistype: %d", taxistype);
 
-  TAXIS_INIT();
+  taxisInit ();
 
-  taxisptr = taxis_new_entry();
-  if ( ! taxisptr ) Error("No memory");
+  taxisptr = taxisNewEntry();
 
   taxisID = taxisptr->self;
   taxisptr->type = taxistype;
@@ -391,6 +225,20 @@ int taxisCreate(int taxistype)
   return (taxisID);
 }
 
+static
+void taxisDestroyKernel( taxis_t * taxisptr )
+{
+  int id;
+
+  taxis_check_ptr(__func__, taxisptr);
+
+  id = taxisptr->self;
+
+  free ( taxisptr );
+
+  reshRemove ( id, &taxisOps );
+}
+
 /*
 @Function  taxisDestroy
 @Title     Destroy a Time axis
@@ -405,11 +253,18 @@ void taxisDestroy(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
+  if ( taxisptr->name     ) free(taxisptr->name);
+  if ( taxisptr->longname ) free(taxisptr->longname);
 
-  taxis_delete_entry(taxisptr);
+  taxisDestroyKernel ( taxisptr );
+}
+
+
+void taxisDestroyP( void * taxisptr )
+{
+  taxisDestroyKernel (( taxis_t * ) taxisptr );
 }
 
 
@@ -419,9 +274,9 @@ int taxisDuplicate(int taxisID1)
   taxis_t *taxisptr1;
   taxis_t *taxisptr2;
 
-  taxisptr1 = taxis_to_pointer(taxisID1);
+  taxisptr1 = ( taxis_t * ) reshGetVal ( taxisID1, &taxisOps );
 
-  taxisptr2 = taxis_new_entry();
+  taxisptr2 = taxisNewEntry();
   if ( ! taxisptr2 ) Error("No memory");
 
   taxisID2 = taxisptr2->self;
@@ -430,6 +285,8 @@ int taxisDuplicate(int taxisID1)
     Message("taxisID2: %d", taxisID2);
 
   ptaxisCopy(taxisptr2, taxisptr1);
+  if ( taxisptr1->name     ) taxisptr2->name = strdup(taxisptr1->name);
+  if ( taxisptr1->longname ) taxisptr2->longname = strdup(taxisptr1->longname);
 
   // taxisptr2->has_bounds = FALSE;
 
@@ -441,7 +298,13 @@ void taxisDefType(int taxisID, int type)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  if ( reshGetStatus ( taxisID, &taxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed." );
+      return;
+    }
+
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -466,7 +329,7 @@ void taxisDefVdate(int taxisID, int vdate)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -491,7 +354,7 @@ void taxisDefVtime(int taxisID, int vtime)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -516,7 +379,13 @@ void taxisDefRdate(int taxisID, int rdate)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  if ( reshGetStatus ( taxisID, &taxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -541,7 +410,13 @@ void taxisDefRtime(int taxisID, int rtime)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  if ( reshGetStatus ( taxisID, &taxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -568,7 +443,13 @@ void taxisDefCalendar(int taxisID, int calendar)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  if ( reshGetStatus ( taxisID, &taxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -580,7 +461,13 @@ void taxisDefTunit(int taxisID, int unit)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  if ( reshGetStatus ( taxisID, &taxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -592,14 +479,19 @@ void taxisDefNumavg(int taxisID, int numavg)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  if ( reshGetStatus ( taxisID, &taxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
   taxisptr->numavg = numavg;
 }
 
-
 /*
 The type of the time axis, one of the set of predefined CDI time types.
 The valid CDI time types are TAXIS_ABSOLUTE and TAXIS_RELATIVE.
@@ -608,7 +500,7 @@ int taxisInqType(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -620,7 +512,7 @@ int taxisHasBounds(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -632,7 +524,13 @@ void taxisDeleteBounds(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  if ( reshGetStatus ( taxisID, &taxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -645,13 +543,13 @@ void taxisCopyTimestep(int taxisID2, int taxisID1)
   taxis_t *taxisptr1;
   taxis_t *taxisptr2;
 
-  taxisptr1 = taxis_to_pointer(taxisID1);
-  taxisptr2 = taxis_to_pointer(taxisID2);
+  taxisptr1 = ( taxis_t * ) reshGetVal ( taxisID1, &taxisOps );
+  taxisptr2 = ( taxis_t * ) reshGetVal ( taxisID2, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr1);
   taxis_check_ptr(__func__, taxisptr2);
 
-  TAXIS_LOCK();
+  reshLock ();
 
   taxisptr2->rdate = taxisptr1->rdate;
   taxisptr2->rtime = taxisptr1->rtime;
@@ -667,7 +565,7 @@ void taxisCopyTimestep(int taxisID2, int taxisID1)
       taxisptr2->vtime_ub = taxisptr1->vtime_ub;
     }
 
-  TAXIS_UNLOCK();
+  reshUnlock ();
 }
 
 /*
@@ -690,7 +588,7 @@ int taxisInqVdate(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -702,7 +600,7 @@ void taxisInqVdateBounds(int taxisID, int *vdate_lb, int *vdate_ub)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -715,7 +613,13 @@ void taxisDefVdateBounds(int taxisID, int vdate_lb, int vdate_ub)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  if ( reshGetStatus ( taxisID, &taxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -745,7 +649,7 @@ int taxisInqVtime(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -756,8 +660,8 @@ int taxisInqVtime(int taxisID)
 void taxisInqVtimeBounds(int taxisID, int *vtime_lb, int *vtime_ub)
 {
   taxis_t *taxisptr;
-
-  taxisptr = taxis_to_pointer(taxisID);
+  
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -770,7 +674,13 @@ void taxisDefVtimeBounds(int taxisID, int vtime_lb, int vtime_ub)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  if ( reshGetStatus ( taxisID, &taxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -780,7 +690,6 @@ void taxisDefVtimeBounds(int taxisID, int vtime_lb, int vtime_ub)
   taxisptr->has_bounds = TRUE;
 }
 
-
 /*
 @Function  taxisInqRdate
 @Title     Get the reference date
@@ -801,7 +710,7 @@ int taxisInqRdate(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -814,7 +723,6 @@ int taxisInqRdate(int taxisID)
   return (taxisptr->rdate);
 }
 
-
 /*
 @Function  taxisInqRtime
 @Title     Get the reference time
@@ -835,7 +743,7 @@ int taxisInqRtime(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -848,7 +756,6 @@ int taxisInqRtime(int taxisID)
   return (taxisptr->rtime);
 }
 
-
 /*
 @Function  taxisInqCalendar
 @Title     Get the calendar
@@ -863,7 +770,7 @@ The function @func{taxisInqCalendar} returns the calendar of a Time axis.
 @Result
 @func{taxisInqCalendar} returns the type of the calendar,
 one of the set of predefined CDI calendar types.
-The valid CDI calendar types are @func{CALENDAR_STANDARD}, @func{CALENDAR_PROLEPRIC}, 
+The valid CDI calendar types are @func{CALENDAR_STANDARD}, @func{CALENDAR_PROLEPTIC}, 
 @func{CALENDAR_360DAYS}, @func{CALENDAR_365DAYS} and @func{CALENDAR_366DAYS}.
 
 @EndFunction
@@ -872,7 +779,7 @@ int taxisInqCalendar(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -884,7 +791,7 @@ int taxisInqTunit(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -896,7 +803,7 @@ int taxisInqNumavg(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -908,7 +815,7 @@ taxis_t *taxisPtr(int taxisID)
 {
   taxis_t *taxisptr;
 
-  taxisptr = taxis_to_pointer(taxisID);
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
   taxis_check_ptr(__func__, taxisptr);
 
@@ -1011,7 +918,6 @@ void timeval2vtime(double timevalue, taxis_t *taxis, int *vdate, int *vtime)
   int timeunit;
   int calendar;
   int julday, secofday, days, secs;
-  static int lwarn = TRUE;
 
   *vdate = 0;
   *vtime = 0;
@@ -1022,7 +928,7 @@ void timeval2vtime(double timevalue, taxis_t *taxis, int *vdate, int *vtime)
   rdate  = (*taxis).rdate;
   rtime  = (*taxis).rtime;
 
-  if ( rdate == 0 && rtime == 0 && DBL_IS_EQUAL(timevalue, 0) ) return;
+  if ( rdate == 0 && rtime == 0 && DBL_IS_EQUAL(timevalue, 0.) ) return;
 
   cdiDecodeDate(rdate, &year, &month, &day);
   cdiDecodeTime(rtime, &hour, &minute, &second);
@@ -1035,61 +941,31 @@ void timeval2vtime(double timevalue, taxis_t *taxis, int *vdate, int *vtime)
 
   if ( timeunit == TUNIT_MONTH || timeunit == TUNIT_YEAR )
     {
-      static int lold = FALSE;
+      int nmon, dpm;
+      double fmon;
 
-      if ( timeunit == TUNIT_YEAR )
-	{
-	  timeunit = TUNIT_MONTH;
-	  timevalue *= 12;
-	}
+      if ( timeunit == TUNIT_YEAR ) timevalue *= 12;
 
-      if ( lold )
-	{
-	  int nmon;
+      nmon = (int) timevalue;
+      fmon = timevalue - nmon;
 
-	  if ( (NINT(timevalue*10))%10 && lwarn )
-	    {
-	      Warning("Possible wrong calculation of date/time!\n"
-		      "Non integer time offset unsupported for time unit MONTH and YEAR!");
-	      lwarn = FALSE;
-	    }
+      month += nmon;
 
-	  nmon = (int) (timevalue+0.001);
+      while ( month > 12 ) { month -= 12; year++; }
+      while ( month <  1 ) { month += 12; year--; }
 
-	  month += nmon;
-	  
-	  while ( month > 12 ) { month -= 12; year++; }
-	  while ( month <  1 ) { month += 12; year--; }
-	}
-      else
-	{
-	  int nmon, dpm;
-	  double fmon;
-	  
-	  nmon = (int) timevalue;
-	  fmon = timevalue - nmon;
-	  
-	  month += nmon;
-	  
-	  while ( month > 12 ) { month -= 12; year++; }
-	  while ( month <  1 ) { month += 12; year--; }
-
-	  dpm = days_per_month(calendar, year, month);
-	  timeunit = TUNIT_DAY;
-	  timevalue = fmon*dpm;
-	}
+      dpm = days_per_month(calendar, year, month);
+      timeunit = TUNIT_DAY;
+      timevalue = fmon*dpm;
     }
 
-  if ( timeunit != TUNIT_MONTH && timeunit != TUNIT_YEAR )
-    {
-      encode_caldaysec(calendar, year, month, day, hour, minute, second, &julday, &secofday);
+  encode_caldaysec(calendar, year, month, day, hour, minute, second, &julday, &secofday);
 
-      cdiDecodeTimevalue(timeunit, timevalue, &days, &secs);
+  cdiDecodeTimevalue(timeunit, timevalue, &days, &secs);
 
-      julday_add(days, secs, &julday, &secofday);
+  julday_add(days, secs, &julday, &secofday);
 
-      decode_caldaysec(calendar, julday, secofday, &year, &month, &day, &hour, &minute, &second);
-    }
+  decode_caldaysec(calendar, julday, secofday, &year, &month, &day, &hour, &minute, &second);
 
   *vdate = cdiEncodeDate(year, month, day);
   *vtime = cdiEncodeTime(hour, minute, second);
@@ -1155,7 +1031,7 @@ double vtime2timeval(int vdate, int vtime, taxis_t *taxis)
 
       value += (days+secs/86400.)/dpm;
 
-      if ( timeunit == TUNIT_YEAR ) value = NINT(value/12);
+      if ( timeunit == TUNIT_YEAR ) value = value/12;
     }
   else
     {
@@ -1186,7 +1062,7 @@ void conv_timeval(double timevalue, int *rvdate, int *rvtime)
     daysec = (int) (-(timevalue - vdate)*86400 + 0.01);
   else
     daysec = (int) ( (timevalue - vdate)*86400 + 0.01);
-  
+
   hour   =  daysec / 3600;
   minute = (daysec - hour*3600)/60;
   second =  daysec - hour*3600 - minute*60;
@@ -1324,16 +1200,16 @@ double cdiEncodeTimeval(int date, int time, taxis_t *taxis)
 
 void ptaxisInit(taxis_t *taxisptr)
 {
-  taxis_init_ptr(taxisptr);
+  taxisDefaultValue ( taxisptr );
 }
 
 
 void ptaxisCopy(taxis_t *dest, taxis_t *source)
 {
-  TAXIS_LOCK();
+  reshLock ();
 
   /* memcpy(dest, source, sizeof(taxis_t)); */
-  dest->used       = source->used;  
+  dest->used       = source->used;
   dest->type       = source->type;
   dest->vdate      = source->vdate;
   dest->vtime      = source->vtime;
@@ -1348,5 +1224,180 @@ void ptaxisCopy(taxis_t *dest, taxis_t *source)
   dest->vdate_ub   = source->vdate_ub;
   dest->vtime_ub   = source->vtime_ub;
 
-  TAXIS_UNLOCK();
+  reshUnlock ();
 }
+
+
+void taxisPrintKernel ( taxis_t * taxisptr, FILE * fp )
+{
+  int vdate_lb, vdate_ub;
+  int vtime_lb, vtime_ub;
+
+  taxis_check_ptr ( __func__, taxisptr );
+
+  taxisInqVdateBounds ( taxisptr->self, &vdate_lb, &vdate_ub);
+  taxisInqVtimeBounds ( taxisptr->self, &vtime_lb, &vtime_ub);
+
+  fprintf ( fp, "#\n");
+  fprintf ( fp, "# taxisID %d\n", taxisptr->self);
+  fprintf ( fp, "#\n");
+  fprintf ( fp, "self       = %d\n", taxisptr->self );
+  fprintf ( fp, "used       = %d\n", taxisptr->used );
+  fprintf ( fp, "type       = %d\n", taxisptr->type );
+  fprintf ( fp, "vdate      = %d\n", taxisptr->vdate );
+  fprintf ( fp, "vtime      = %d\n", taxisptr->vtime );
+  fprintf ( fp, "rdate      = %d\n", taxisptr->rdate );
+  fprintf ( fp, "rtime      = %d\n", taxisptr->rtime );
+  fprintf ( fp, "calendar   = %d\n", taxisptr->calendar );
+  fprintf ( fp, "unit       = %d\n", taxisptr->unit );
+  fprintf ( fp, "numavg     = %d\n", taxisptr->numavg );
+  fprintf ( fp, "has_bounds = %d\n", taxisptr->has_bounds );
+  fprintf ( fp, "vdate_lb   = %d\n", vdate_lb );
+  fprintf ( fp, "vtime_lb   = %d\n", vtime_lb );
+  fprintf ( fp, "vdate_ub   = %d\n", vdate_ub );
+  fprintf ( fp, "vtime_ub   = %d\n", vtime_ub );
+  fprintf ( fp, "\n");
+}
+
+void taxisPrint ( int taxisID )
+{
+  taxis_t * taxisptr;
+
+  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
+  taxisPrintKernel ( taxisptr, stdout );
+}
+
+void taxisPrintP ( void * taxisptr, FILE * fp )
+{
+  taxisPrintKernel (( taxis_t * ) taxisptr, fp );
+}
+
+int taxisCompareP ( void *  taxisptr1, void * taxisptr2 )
+{
+  taxis_t * t1, * t2;
+
+  t1 = ( taxis_t * ) taxisptr1;
+  t2 = ( taxis_t * ) taxisptr2;
+
+  xassert ( t1 );
+  xassert ( t2 );
+
+  return ! ( t1->used       == t2->used       &&
+	     t1->type       == t2->type       &&
+	     t1->vdate      == t2->vdate      &&
+	     t1->vtime      == t2->vtime      &&
+	     t1->rdate      == t2->rdate      &&
+	     t1->rtime      == t2->rtime      &&
+	     t1->calendar   == t2->calendar   &&
+	     t1->unit       == t2->unit       &&
+	     t1->numavg     == t2->numavg     &&
+	     t1->has_bounds == t2->has_bounds &&
+	     t1->vdate_lb   == t2->vdate_lb   &&
+	     t1->vtime_lb   == t2->vtime_lb   &&
+	     t1->vdate_ub   == t2->vdate_ub   &&
+	     t1->vtime_ub   == t2->vtime_ub );
+}
+
+
+#ifdef USE_MPI
+static int
+taxisTxCode ( void )
+{
+  return TAXIS;
+}
+
+enum { taxisNint = 15 };
+
+static
+int taxisGetPackSize ( void * voidP, MPI_Comm comm )
+{
+  int packBufferSize = 0, size;
+
+  xmpi ( MPI_Pack_size ( taxisNint, MPI_INT, comm, &size ));
+  packBufferSize += size;
+  xmpi ( MPI_Pack_size ( 1, MPI_DOUBLE, comm, &size ));
+  packBufferSize += size;
+
+  return packBufferSize;
+}
+
+
+void taxisUnpack ( char * unpackBuffer, int unpackBufferSize,
+		   int * unpackBufferPos, int nspTarget, MPI_Comm comm )
+{
+  taxis_t * taxisP;
+  int intBuffer[taxisNint];
+  double d;
+
+  xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+		      intBuffer, taxisNint, MPI_INT, comm ));
+  xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+		      &d, 1, MPI_DOUBLE, comm ));
+
+  xassert ( xchecksum ( DATATYPE_INT, taxisNint, intBuffer ) == d );
+
+  taxisInit ();
+
+  taxisP = taxisNewEntry();
+  if ( ! taxisP ) Error("No memory");
+
+  xassert ( namespaceAdaptKey ( intBuffer[0], nspTarget ) == taxisP->self );
+
+  taxisP->used       = intBuffer[1];
+  taxisP->type       = intBuffer[2];
+  taxisP->vdate      = intBuffer[3];
+  taxisP->vtime      = intBuffer[4];
+  taxisP->rdate      = intBuffer[5];
+  taxisP->rtime      = intBuffer[6];
+  taxisP->calendar   = intBuffer[7];
+  taxisP->unit       = intBuffer[8];
+  taxisP->numavg     = intBuffer[9];
+  taxisP->has_bounds = intBuffer[10];
+  taxisP->vdate_lb   = intBuffer[11];
+  taxisP->vtime_lb   = intBuffer[12];
+  taxisP->vdate_ub   = intBuffer[13];
+  taxisP->vtime_ub   = intBuffer[14];
+}
+
+
+static
+void taxisPack ( void * voidP, void * packBuffer, int packBufferSize,
+		 int * packBufferPos, MPI_Comm comm )
+{
+  taxis_t   * taxisP = ( taxis_t * ) voidP;
+  int intBuffer[taxisNint];
+  double d;
+
+  intBuffer[0]  = taxisP->self; 
+  intBuffer[1]  = taxisP->used;      
+  intBuffer[2]  = taxisP->type;      
+  intBuffer[3]  = taxisP->vdate;     
+  intBuffer[4]  = taxisP->vtime;     
+  intBuffer[5]  = taxisP->rdate;     
+  intBuffer[6]  = taxisP->rtime;     
+  intBuffer[7]  = taxisP->calendar;  
+  intBuffer[8]  = taxisP->unit;      
+  intBuffer[9]  = taxisP->numavg;    
+  intBuffer[10] = taxisP->has_bounds;
+  intBuffer[11] = taxisP->vdate_lb;  
+  intBuffer[12] = taxisP->vtime_lb;  
+  intBuffer[13] = taxisP->vdate_ub;  
+  intBuffer[14] = taxisP->vtime_ub;  
+  
+  xmpi ( MPI_Pack ( intBuffer, taxisNint, MPI_INT, 
+		    packBuffer, packBufferSize, packBufferPos, comm ));
+  d = xchecksum ( DATATYPE_INT, taxisNint, intBuffer );
+  xmpi ( MPI_Pack ( &d, 1, MPI_DOUBLE,
+		    packBuffer, packBufferSize, packBufferPos, comm ));
+}
+
+#endif
+/*
+ * Local Variables:
+ * c-file-style: "Java"
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * show-trailing-whitespace: t
+ * require-trailing-newline: t
+ * End:
+ */
