@@ -10,6 +10,11 @@
 #include "dmemory.h"
 #include "varscan.h"
 #include "vlist.h"
+#include "grid.h"
+
+
+extern void zaxisGetIndexList ( int, int * );
+
 
 #undef  UNDEFID
 #define UNDEFID -1
@@ -55,6 +60,7 @@ typedef struct
   char       *name;
   char       *longname;
   char       *units;
+  ensinfo_t  *ensdata;
 }
 vartable_t;
 
@@ -89,9 +95,9 @@ void paramInitEntry(int varID, int param)
   vartable[varID].name           = NULL;
   vartable[varID].longname       = NULL;
   vartable[varID].units          = NULL;
+  vartable[varID].ensdata        = NULL;
 }
 
-
 static
 int varGetEntry(int param, int zaxistype, int ltype)
 {
@@ -108,7 +114,7 @@ int varGetEntry(int param, int zaxistype, int ltype)
   return (UNDEFID);
 }
 
-
+static
 void varFree(void)
 {
   int varID;
@@ -121,6 +127,7 @@ void varFree(void)
       if ( vartable[varID].name )     free(vartable[varID].name);
       if ( vartable[varID].longname ) free(vartable[varID].longname);
       if ( vartable[varID].units )    free(vartable[varID].units);
+      if ( vartable[varID].ensdata )  free(vartable[varID].ensdata);
     }
 
   if ( vartable )
@@ -137,6 +144,7 @@ void varFree(void)
   Vctsize = 0;
 }
 
+static
 int levelNewEntry(int varID, int level1, int level2)
 {
   int levelID = 0;
@@ -206,7 +214,8 @@ int levelNewEntry(int varID, int level1, int level2)
 
 #define  UNDEF_PARAM  -4711
 
-int paramNewEntry (int param)
+static
+int paramNewEntry(int param)
 {
   int varID = 0;
 
@@ -282,7 +291,7 @@ void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
       vartable[varID].zaxistype = zaxistype;
       vartable[varID].ltype     = ltype;
       vartable[varID].lbounds   = lbounds;
-      if ( tsteptype > 0 ) vartable[varID].tsteptype = tsteptype;
+      if ( tsteptype != UNDEFID ) vartable[varID].tsteptype = tsteptype;
       if ( numavg ) vartable[varID].timave = 1;
 
       if ( name )     if ( name[0] )     vartable[varID].name     = strdup(name);
@@ -319,7 +328,7 @@ void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
   *plevelID = levelID;
 }
 
-
+static
 int dblcmp(const void *s1, const void *s2)
 {
   int cmp = 0;
@@ -330,7 +339,7 @@ int dblcmp(const void *s1, const void *s2)
   return (cmp);
 }
 
-
+static
 int cmpLevelTable(const void *s1, const void *s2)
 {
   int cmp = 0;
@@ -393,7 +402,7 @@ void cdiGenVars(int streamID)
   int timave, timaccu;
   int lbounds;
   int comptype;
-  char name[256], longname[256], units[256];
+  char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
   double *dlevels = NULL;
   double *dlevels1 = NULL;
   double *dlevels2 = NULL;
@@ -527,11 +536,10 @@ void cdiGenVars(int streamID)
       free(dlevels);
 
       varID = streamNewVar(streamID, gridID, zaxisID);
-      varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARIABLE);
+      varID = vlistDefVar(vlistID, gridID, zaxisID, tsteptype);
 
       vlistDefVarParam(vlistID, varID, param);
       vlistDefVarDatatype(vlistID, varID, prec);
-      vlistDefVarTsteptype(vlistID, varID, tsteptype);
       vlistDefVarTimave(vlistID, varID, timave);
       vlistDefVarTimaccu(vlistID, varID, timaccu);
       vlistDefVarCompType(vlistID, varID, comptype);
@@ -542,6 +550,10 @@ void cdiGenVars(int streamID)
       if ( vartable[varid].longname ) vlistDefVarLongname(vlistID, varID, vartable[varid].longname);
       if ( vartable[varid].units )    vlistDefVarUnits(vlistID, varID, vartable[varid].units);
 
+      if ( vartable[varid].ensdata )  vlistDefVarEnsemble(vlistID, varID, vartable[varid].ensdata->ens_index,
+	                                                  vartable[varid].ensdata->ens_count,
+							  vartable[varid].ensdata->forecast_init_type);
+
       if ( cdiDefaultTableID != UNDEFID )
 	{
 	  int pdis, pcat, pnum;
@@ -621,6 +633,7 @@ void varDefVCT(size_t vctsize, double *vctptr)
     }
 }
 
+
 void varDefZAxisReference(int nlev, int nvgrid, char *uuid)
 {
   numberOfVerticalLevels = nlev;
@@ -628,6 +641,7 @@ void varDefZAxisReference(int nlev, int nvgrid, char *uuid)
   strncpy(uuidVGrid, uuid, 16);
 }
 
+
 int varDefGrid(int vlistID, grid_t grid, int mode)
 {
   /*
@@ -640,6 +654,7 @@ int varDefGrid(int vlistID, grid_t grid, int mode)
   int gridID = UNDEFID;
   int index;
   vlist_t *vlistptr;
+  int * gridIndexList, i;
 
   vlistptr = vlist_to_pointer(vlistID);
 
@@ -663,14 +678,21 @@ int varDefGrid(int vlistID, grid_t grid, int mode)
   if ( ! griddefined )
     {
       ngrids = gridSize();
-      for ( gridID = 0; gridID < ngrids; gridID++ )
-	{
-	  if ( gridCompare(gridID, grid) == 0 )
-	    {
-	      gridglobdefined = TRUE;
-	      break;
-	    }
-	}
+      if ( ngrids > 0 )
+        {
+          gridIndexList = malloc(ngrids*sizeof(int));
+          gridGetIndexList ( ngrids, gridIndexList );
+          for ( i = 0; i < ngrids; i++ )
+            {
+              gridID = gridIndexList[i];
+              if ( gridCompare(gridID, grid) == 0 )
+                {
+                  gridglobdefined = TRUE;
+                  break;
+                }
+            }
+          if ( gridIndexList ) free ( gridIndexList );
+        }
 
       ngrids = vlistptr->ngrids;
       if ( mode == 1 )
@@ -709,8 +731,8 @@ int zaxisCompare(int zaxisID, int zaxistype, int nlevels, int lbounds, double *l
       if ( nlevels == zaxisInqSize(zaxisID) && zlbounds == lbounds )
 	{
 	  const double *dlevels;
-	  char zlongname[256];
-	  char zunits[256];
+	  char zlongname[CDI_MAX_NAME];
+	  char zunits[CDI_MAX_NAME];
 
 	  dlevels = zaxisInqLevelsPtr(zaxisID);
 	  for ( levelID = 0; levelID < nlevels; levelID++ )
@@ -755,6 +777,7 @@ int varDefZaxis(int vlistID, int zaxistype, int nlevels, double *levels, int lbo
   int index;
   int zaxisglobdefined = 0;
   vlist_t *vlistptr;
+  int i;
 
   vlistptr = vlist_to_pointer(vlistID);
 
@@ -776,12 +799,22 @@ int varDefZaxis(int vlistID, int zaxistype, int nlevels, double *levels, int lbo
   if ( ! zaxisdefined )
     {
       nzaxis = zaxisSize();
-      for ( zaxisID = 0; zaxisID < nzaxis; zaxisID++ )
-	if ( zaxisCompare(zaxisID, zaxistype, nlevels, lbounds, levels, longname, units, ltype) == 0 )
-	  {
-	    zaxisglobdefined = 1;
-	    break;
-	  }
+      if ( nzaxis > 0 )
+        {
+          int *zaxisIndexList;
+          zaxisIndexList = (int *) malloc ( nzaxis * sizeof ( int ));
+          zaxisGetIndexList ( nzaxis, zaxisIndexList );
+          for ( i = 0; i < nzaxis; i++ )
+            {
+              zaxisID = zaxisIndexList[i];
+              if ( zaxisCompare(zaxisID, zaxistype, nlevels, lbounds, levels, longname, units, ltype) == 0 )
+                {
+                  zaxisglobdefined = 1;
+                  break;
+                }
+            }
+          if ( zaxisIndexList ) free ( zaxisIndexList );
+        }
 
       nzaxis = vlistptr->nzaxis;
       if ( mode == 1 )
@@ -885,6 +918,18 @@ void varDefTable(int varID, int tableID)
 {
   vartable[varID].tableID = tableID;
 }
+
+
+void varDefEnsembleInfo(int varID, int ens_idx, int ens_count, int forecast_type)
+{
+  if ( vartable[varID].ensdata == NULL )
+      vartable[varID].ensdata = (ensinfo_t *) malloc( sizeof( ensinfo_t ) );
+
+  vartable[varID].ensdata->ens_index = ens_idx;
+  vartable[varID].ensdata->ens_count = ens_count;
+  vartable[varID].ensdata->forecast_init_type = forecast_type;
+}
+
 /*
  * Local Variables:
  * c-file-style: "Java"
