@@ -10,7 +10,11 @@
 
 #include "cdi.h"
 #include "stream_int.h"
-
+#include "pio_util.h"
+#include "resource_handle.h"
+#include "pio_rpc.h"
+#include "varscan.h"
+#include "namespace.h"
 
 #define  LevelUp    1
 #define  LevelDown  2
@@ -48,10 +52,10 @@ static int CDI_MaxZaxistype = sizeof(ZaxistypeEntry) / sizeof(ZaxistypeEntry[0])
 
 typedef struct {
   unsigned char positive;
-  char     name[256];
-  char     longname[256];
-  char     stdname[256];
-  char     units[256];
+  char     name[CDI_MAX_NAME];
+  char     longname[CDI_MAX_NAME];
+  char     stdname[CDI_MAX_NAME];
+  char     units[CDI_MAX_NAME];
   double  *vals;
   double  *lbounds;
   double  *ubounds;
@@ -69,136 +73,28 @@ typedef struct {
 }
 zaxis_t;
 
-static int  ZAXIS_Debug = 0;   /* If set to 1, debugging */
-
-static int _zaxis_max = MAX_ZAXES;
-
-static void zaxis_initialize(void);
-
-static int _zaxis_init = FALSE;
-
-#if  defined  (HAVE_LIBPTHREAD)
-#  include <pthread.h>
-
-static pthread_once_t _zaxis_init_thread = PTHREAD_ONCE_INIT;
-static pthread_mutex_t _zaxis_mutex;
-
-#  define ZAXIS_LOCK()         pthread_mutex_lock(&_zaxis_mutex);
-#  define ZAXIS_UNLOCK()       pthread_mutex_unlock(&_zaxis_mutex);
-#  define ZAXIS_INIT()	      \
-   if ( _zaxis_init == FALSE ) pthread_once(&_zaxis_init_thread, zaxis_initialize);
-
-#else
-
-#  define ZAXIS_LOCK()
-#  define ZAXIS_UNLOCK()
-#  define ZAXIS_INIT()	      \
-   if ( _zaxis_init == FALSE ) zaxis_initialize();
-
+static int    zaxisCompareP    ( void * zaxisptr1, void * zaxisptr2 );
+static void   zaxisDestroyP    ( void * zaxisptr );
+static void   zaxisPrintP      ( void * zaxisptr, FILE * fp );
+#ifdef USE_MPI
+static int    zaxisGetPackSize ( void * zaxisptr, MPI_Comm comm );
+static void   zaxisPack        ( void * zaxisptr, void * buffer, int size,
+                                 int *pos, MPI_Comm comm);
+static int    zaxisTxCode      ( void );
 #endif
 
+resOps zaxisOps = { zaxisCompareP, zaxisDestroyP, zaxisPrintP
+#ifdef USE_MPI
+                    , zaxisGetPackSize, zaxisPack, zaxisTxCode
+#endif
+};
 
-typedef struct _zaxisPtrToIdx {
-  int idx;
-  zaxis_t *ptr;
-  struct _zaxisPtrToIdx *next;
-} zaxisPtrToIdx;
-
-
-static zaxisPtrToIdx *_zaxisList  = NULL;
-static zaxisPtrToIdx *_zaxisAvail = NULL;
-
+static int  ZAXIS_Debug = 0;   /* If set to 1, debugging */
 
 static
-void zaxis_list_new(void)
+void zaxisDefaultValue ( zaxis_t *zaxisptr )
 {
-  assert(_zaxisList == NULL);
-
-  _zaxisList = (zaxisPtrToIdx *) malloc(_zaxis_max*sizeof(zaxisPtrToIdx));
-}
-
-static
-void zaxis_list_delete(void)
-{
-  if ( _zaxisList ) free(_zaxisList);
-}
-
-static
-void zaxis_init_pointer(void)
-{
-  int  i;
-
-  for ( i = 0; i < _zaxis_max; i++ )
-    {
-      _zaxisList[i].next = _zaxisList + i + 1;
-      _zaxisList[i].idx  = i;
-      _zaxisList[i].ptr  = 0;
-    }
-
-  _zaxisList[_zaxis_max-1].next = 0;
-
-  _zaxisAvail = _zaxisList;
-}
-
-
-zaxis_t *zaxis_to_pointer(int idx)
-{
-  zaxis_t *zaxisptr = NULL;
-
-  ZAXIS_INIT();
-
-  if ( idx >= 0 && idx < _zaxis_max )
-    {
-      ZAXIS_LOCK();
-
-      zaxisptr = _zaxisList[idx].ptr;
-
-      ZAXIS_UNLOCK();
-    }
-  else
-    Error("zaxis index %d undefined!", idx);
-
-  return (zaxisptr);
-}
-
-/* Create an index from a pointer */
-static
-int zaxis_from_pointer(zaxis_t *ptr)
-{
-  int      idx = -1;
-  zaxisPtrToIdx *newptr;
-
-  if ( ptr )
-    {
-      ZAXIS_LOCK();
-
-      if ( _zaxisAvail )
-	{
-	  newptr       = _zaxisAvail;
-	  _zaxisAvail  = _zaxisAvail->next;
-	  newptr->next = 0;
-	  idx	       = newptr->idx;
-	  newptr->ptr  = ptr;
-
-	  if ( ZAXIS_Debug )
-	    Message("Pointer %p has idx %d from zaxis list", ptr, idx);
-	}
-      else
-	Warning("Too many open zaxis (limit is %d)!", _zaxis_max);
-
-      ZAXIS_UNLOCK();
-    }
-  else
-    Error("Internal problem (pointer %p undefined)", ptr);
-
-  return (idx);
-}
-
-static
-void zaxis_init_entry(zaxis_t *zaxisptr)
-{
-  zaxisptr->self        = zaxis_from_pointer(zaxisptr);
-
+  zaxisptr->self        = CDI_UNDEFID;
   zaxisptr->name[0]     = 0;
   zaxisptr->longname[0] = 0;
   zaxisptr->stdname[0]  = 0;
@@ -219,62 +115,33 @@ void zaxis_init_entry(zaxis_t *zaxisptr)
   zaxisptr->uuid[0]     = 0;
 }
 
+
 static
-zaxis_t *zaxis_new_entry(void)
+zaxis_t *zaxisNewEntry(void)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = (zaxis_t *) malloc(sizeof(zaxis_t));
+  zaxisptr = (zaxis_t *) xmalloc(sizeof(zaxis_t));
 
-  if ( zaxisptr ) zaxis_init_entry(zaxisptr);
+  zaxisDefaultValue ( zaxisptr );
+
+  zaxisptr->self = reshPut (( void * ) zaxisptr, &zaxisOps );
 
   return (zaxisptr);
 }
 
 static
-void zaxis_delete_entry(zaxis_t *zaxisptr)
-{
-  int idx;
-
-  idx = zaxisptr->self;
-
-  ZAXIS_LOCK();
-
-  free(zaxisptr);
-
-  _zaxisList[idx].next = _zaxisAvail;
-  _zaxisList[idx].ptr  = 0;
-  _zaxisAvail          = &_zaxisList[idx];
-
-  ZAXIS_UNLOCK();
-
-  if ( ZAXIS_Debug )
-    Message("Removed idx %d from zaxis list", idx);
-}
-
-static
-void zaxis_initialize(void)
+void zaxisInit(void)
 {
+  static int zaxisInitialized = 0;
   char *env;
 
-#if  defined  (HAVE_LIBPTHREAD)
-  /* initialize global API mutex lock */
-  pthread_mutex_init(&_zaxis_mutex, NULL);
-#endif
+  if ( zaxisInitialized ) return;
+
+  zaxisInitialized = 1;
 
   env = getenv("ZAXIS_DEBUG");
   if ( env ) ZAXIS_Debug = atoi(env);
-
-  zaxis_list_new();
-  atexit(zaxis_list_delete);
-
-  ZAXIS_LOCK();
-
-  zaxis_init_pointer();
-
-  ZAXIS_UNLOCK();
-
-  _zaxis_init = TRUE;
 }
 
 static
@@ -298,19 +165,7 @@ void zaxisCheckPtr(const char *caller, int zaxisID, zaxis_t *zaxisptr)
 
 int zaxisSize(void)
 {
-  int zaxissize = 0;
-  int i;
-
-  ZAXIS_INIT();
-
-  ZAXIS_LOCK();
-
-  for ( i = 0; i < _zaxis_max; i++ )
-    if ( _zaxisList[i].ptr ) zaxissize++;
-
-  ZAXIS_UNLOCK();
-
-  return (zaxissize);
+  return reshCountType ( &zaxisOps );
 }
 
 
@@ -359,10 +214,9 @@ int zaxisCreate(int zaxistype, int size)
   if ( CDI_Debug )
     Message("zaxistype: %d size: %d ", zaxistype, size);
 
-  ZAXIS_INIT();
+  zaxisInit ();
 
-  zaxisptr = zaxis_new_entry();
-  if ( ! zaxisptr ) Error("No memory");
+  zaxisptr = zaxisNewEntry();
 
   zaxisID = zaxisptr->self;
 
@@ -391,6 +245,26 @@ int zaxisCreate(int zaxistype, int size)
   return (zaxisID);
 }
 
+
+void zaxisDestroyKernel( zaxis_t * zaxisptr )
+{
+  int id;
+
+  xassert ( zaxisptr );
+
+  id = zaxisptr->self;
+
+  if ( zaxisptr->vals )    free ( zaxisptr->vals );
+  if ( zaxisptr->lbounds ) free ( zaxisptr->lbounds );
+  if ( zaxisptr->ubounds ) free ( zaxisptr->ubounds );
+  if ( zaxisptr->weights ) free ( zaxisptr->weights );
+  if ( zaxisptr->vct )     free ( zaxisptr->vct );
+
+  free ( zaxisptr );
+
+  reshRemove ( id, &zaxisOps );
+}
+
 /*
 @Function  zaxisDestroy
 @Title     Destroy a vertical Z-axis
@@ -405,13 +279,16 @@ void zaxisDestroy(int zaxisID)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxisDestroyKernel ( zaxisptr );
+}
 
-  if ( zaxisptr->vals ) free(zaxisptr->vals);
 
-  zaxis_delete_entry(zaxisptr);
+static
+void zaxisDestroyP ( void * zaxisptr )
+{
+  zaxisDestroyKernel (( zaxis_t * ) zaxisptr );
 }
 
 
@@ -451,7 +328,13 @@ void zaxisDefName(int zaxisID, const char *name)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -477,7 +360,13 @@ void zaxisDefLongname(int zaxisID, const char *longname)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -503,7 +392,13 @@ void zaxisDefUnits(int zaxisID, const char *units)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -534,7 +429,7 @@ void zaxisInqName(int zaxisID, char *name)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -564,7 +459,7 @@ void zaxisInqLongname(int zaxisID, char *longname)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -594,7 +489,7 @@ void zaxisInqUnits(int zaxisID, char *units)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -606,7 +501,7 @@ void zaxisInqStdname(int zaxisID, char *stdname)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -618,7 +513,13 @@ void zaxisDefPrec(int zaxisID, int prec)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -630,7 +531,7 @@ int zaxisInqPrec(int zaxisID)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -642,7 +543,7 @@ int zaxisInqPositive(int zaxisID)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -654,7 +555,13 @@ void zaxisDefLtype(int zaxisID, int ltype)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -666,7 +573,7 @@ int zaxisInqLtype(int zaxisID)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -694,7 +601,13 @@ void zaxisDefLevels(int zaxisID, const double *levels)
   double *vals;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -725,7 +638,13 @@ void zaxisDefLevel(int zaxisID, int levelID, double level)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -751,7 +670,13 @@ void zaxisDefReference(int zaxisID, const int reference)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+    if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -776,7 +701,13 @@ void zaxisDefUUID(int zaxisID, const char *uuid)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+    if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -804,7 +735,7 @@ char *zaxisInqUUID(int zaxisID, char *uuid)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -833,7 +764,7 @@ int zaxisInqReference(int zaxisID)
   int reference = -1;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -863,7 +794,7 @@ double zaxisInqLevel(int zaxisID, int levelID)
   double level = 0;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -878,7 +809,7 @@ double zaxisInqLbound(int zaxisID, int index)
   double level = 0;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -895,7 +826,7 @@ double zaxisInqUbound(int zaxisID, int index)
   double level = 0;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -911,7 +842,7 @@ const double *zaxisInqLevelsPtr(int zaxisID)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -941,7 +872,7 @@ void zaxisInqLevels(int zaxisID, double *levels)
   int i;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -957,7 +888,7 @@ int zaxisInqLbounds(int zaxisID, double *lbounds)
   int i;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -966,8 +897,8 @@ int zaxisInqLbounds(int zaxisID, double *lbounds)
       size = zaxisptr->size;
 
       if ( lbounds )
-	for ( i = 0; i < size; i++ )
-	  lbounds[i] =  zaxisptr->lbounds[i];
+        for ( i = 0; i < size; i++ )
+          lbounds[i] =  zaxisptr->lbounds[i];
     }
 
   return (size);
@@ -980,7 +911,7 @@ int zaxisInqUbounds(int zaxisID, double *ubounds)
   int i;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -989,8 +920,8 @@ int zaxisInqUbounds(int zaxisID, double *ubounds)
       size = zaxisptr->size;
 
       if ( ubounds )
-	for ( i = 0; i < size; i++ )
-	  ubounds[i] =  zaxisptr->ubounds[i];
+        for ( i = 0; i < size; i++ )
+          ubounds[i] =  zaxisptr->ubounds[i];
     }
 
   return (size);
@@ -1003,7 +934,7 @@ int zaxisInqWeights(int zaxisID, double *weights)
   int i;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1012,8 +943,8 @@ int zaxisInqWeights(int zaxisID, double *weights)
       size = zaxisptr->size;
 
       if ( weights )
-	for ( i = 0; i < size; i++ )
-	  weights[i] =  zaxisptr->weights[i];
+        for ( i = 0; i < size; i++ )
+          weights[i] =  zaxisptr->weights[i];
     }
 
   return (size);
@@ -1027,7 +958,7 @@ int zaxisInqLevelID(int zaxisID, double level)
   int i;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1064,7 +995,7 @@ int zaxisInqType(int zaxisID)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1092,7 +1023,7 @@ int zaxisInqSize(int zaxisID)
   int size = 1;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1107,44 +1038,44 @@ void cdiCheckZaxis(int zaxisID)
   int size, i, found;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
-  
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
+
   zaxis_check_ptr(zaxisID, zaxisptr);
 
   if ( zaxisInqType(zaxisID) == ZAXIS_GENERIC )
     {
       size = zaxisptr->size;
       if ( size > 1 )
-	{
-	  /* check direction */
-	  if ( ! zaxisptr->direction )
-	    {
-	      found = 0;
-	      for ( i = 1; i < size; i++ )
-		if ( zaxisptr->vals[i] > zaxisptr->vals[i-1] )
-		  found++;
-	      if ( found == size-1 )
-		{
-		  zaxisptr->direction = LevelUp;
-		}
-	      else
-		{
-		  found = 0;
-		  for ( i = 1; i < size; i++ )
-		    if ( zaxisptr->vals[i] < zaxisptr->vals[i-1] )
-		      found++;
-		  if ( found == size-1 )
-		    {
-		      zaxisptr->direction = LevelDown;
-		    }
-		}
-	    }
-	  /* check consistent */
-	  if ( !zaxisptr->direction )
-	    {
-	      Warning("Direction undefined for zaxisID %d", zaxisID);
-	    }
-	}
+        {
+          /* check direction */
+          if ( ! zaxisptr->direction )
+            {
+              found = 0;
+              for ( i = 1; i < size; i++ )
+                if ( zaxisptr->vals[i] > zaxisptr->vals[i-1] )
+                  found++;
+              if ( found == size-1 )
+                {
+                  zaxisptr->direction = LevelUp;
+                }
+              else
+                {
+                  found = 0;
+                  for ( i = 1; i < size; i++ )
+                    if ( zaxisptr->vals[i] < zaxisptr->vals[i-1] )
+                      found++;
+                  if ( found == size-1 )
+                    {
+                      zaxisptr->direction = LevelDown;
+                    }
+                }
+            }
+          /* check consistent */
+          if ( !zaxisptr->direction )
+            {
+              Warning("Direction undefined for zaxisID %d", zaxisID);
+            }
+        }
     }
 }
 
@@ -1153,7 +1084,13 @@ void zaxisDefVct(int zaxisID, int size, const double *vct)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1173,7 +1110,7 @@ void zaxisInqVct(int zaxisID, double *vct)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1185,7 +1122,7 @@ int zaxisInqVctSize(int zaxisID)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1197,7 +1134,7 @@ const double *zaxisInqVctPtr(int zaxisID)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1210,12 +1147,18 @@ void zaxisDefLbounds(int zaxisID, const double *lbounds)
   size_t size;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
   size = zaxisptr->size;
-  
+
   if ( CDI_Debug )
     if ( zaxisptr->lbounds != NULL )
       Warning("Lower bounds already defined for zaxisID = %d", zaxisID);
@@ -1232,7 +1175,13 @@ void zaxisDefUbounds(int zaxisID, const double *ubounds)
   size_t size;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1254,7 +1203,13 @@ void zaxisDefWeights(int zaxisID, const double *weights)
   size_t size;
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  if ( reshGetStatus ( zaxisID, &zaxisOps ) == CLOSED )
+    {
+      xwarning("%s", "Operation not executed.");
+      return;
+    }
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1275,10 +1230,10 @@ void zaxisChangeType(int zaxisID, int zaxistype)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
-  
+
   zaxisptr->type = zaxistype;
 }
 
@@ -1287,7 +1242,7 @@ void zaxisResize(int zaxisID, int size)
 {
   zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1305,7 +1260,7 @@ int zaxisDuplicate(int zaxisID)
   int size;
   zaxis_t *zaxisptr, *zaxisptrnew;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
 
   zaxis_check_ptr(zaxisID, zaxisptr);
 
@@ -1313,7 +1268,7 @@ int zaxisDuplicate(int zaxisID)
   zaxissize = zaxisInqSize(zaxisID);
 
   zaxisIDnew = zaxisCreate(zaxistype, zaxissize);
-  zaxisptrnew = zaxis_to_pointer(zaxisIDnew);
+  zaxisptrnew = ( zaxis_t * ) reshGetVal ( zaxisIDnew, &zaxisOps );
 
   zaxis_copy(zaxisptrnew, zaxisptr);
 
@@ -1350,33 +1305,32 @@ int zaxisDuplicate(int zaxisID)
       size = zaxisptr->vctsize;
 
       if ( size )
-	{
-	  zaxisptrnew->vctsize = size;
-	  zaxisptrnew->vct = (double *) malloc(size*sizeof(double));
-	  memcpy(zaxisptrnew->vct, zaxisptr->vct, size*sizeof(double));
-	}
+        {
+          zaxisptrnew->vctsize = size;
+          zaxisptrnew->vct = (double *) malloc(size*sizeof(double));
+          memcpy(zaxisptrnew->vct, zaxisptr->vct, size*sizeof(double));
+        }
     }
 
   return (zaxisIDnew);
 }
 
 
-void zaxisPrint(int zaxisID)
+void zaxisPrintKernel ( zaxis_t * zaxisptr, FILE * fp )
 {
-  FILE *fp = stdout;
+  int zaxisID;
   int type;
   char uuid[17];
   int nlevels, levelID;
   int nbyte0, nbyte;
   double level;
-  zaxis_t *zaxisptr;
 
-  zaxisptr = zaxis_to_pointer(zaxisID);
+  xassert ( zaxisptr );
 
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxisID = zaxisptr->self;
 
-  type    = zaxisInqType(zaxisID);
-  nlevels = zaxisInqSize(zaxisID);
+  type    = zaxisptr->type;
+  nlevels = zaxisptr->size;
 
   nbyte0 = 0;
   fprintf(fp, "#\n");
@@ -1429,39 +1383,40 @@ void zaxisPrint(int zaxisID)
       int vctsize;
       const double *vct;
 
-      vctsize = zaxisInqVctSize(zaxisID);
-      vct     = zaxisInqVctPtr(zaxisID);
+      vctsize = zaxisptr->vctsize;
+      vct     = zaxisptr->vct;
       fprintf(fp, "vctsize   = %d\n", vctsize);
       if ( vctsize )
-	{
-	  nbyte0 = fprintf(fp, "vct       = ");
-	  nbyte = nbyte0;
-	  for ( i = 0; i < vctsize; i++ )
-	    {
-	      if ( nbyte > 70 || i == vctsize/2 )
-		{
-		  fprintf(fp, "\n%*s", nbyte0, "");
-		  nbyte = nbyte0;
-		}
-	      nbyte += fprintf(fp, "%.9g ", vct[i]);
-	    }
-	  fprintf(fp, "\n");
-	  /*
-	  nbyte0 = fprintf(fp, "vct_b     = ");
-	  nbyte  = nbyte0;
-	  for ( i = 0; i < vctsize/2; i++ )
-	    {
-	      if ( nbyte > 70 )
-		{
+        {
+          nbyte0 = fprintf(fp, "vct       = ");
+          nbyte = nbyte0;
+          for ( i = 0; i < vctsize; i++ )
+            {
+              if ( nbyte > 70 || i == vctsize/2 )
+                {
                   fprintf(fp, "\n%*s", nbyte0, "");
-		  nbyte = nbyte0;
-		}
-	      nbyte += fprintf(fp, "%.9g ", vct[vctsize/2+i]);
-	    }
-	  fprintf(fp, "\n");
-	  */
-	}
+                  nbyte = nbyte0;
+                }
+              nbyte += fprintf(fp, "%.9g ", vct[i]);
+            }
+          fprintf(fp, "\n");
+          /*
+          nbyte0 = fprintf(fp, "vct_b     = ");
+          nbyte  = nbyte0;
+          for ( i = 0; i < vctsize/2; i++ )
+            {
+              if ( nbyte > 70 )
+                {
+                  fprintf(fp, "\n%*s", nbyte0, "");
+                  nbyte = nbyte0;
+                }
+              nbyte += fprintf(fp, "%.9g ", vct[vctsize/2+i]);
+            }
+          fprintf(fp, "\n");
+          */
+        }
     }
+
   if ( type == ZAXIS_REFERENCE )
     {
       const unsigned char *d;
@@ -1472,6 +1427,434 @@ void zaxisPrint(int zaxisID)
               d[8], d[9], d[10], d[11], d[12], d[13], d[14], d[15]);
     }
 }
+
+
+void zaxisPrint ( int zaxisID )
+{
+  zaxis_t *zaxisptr;
+
+  zaxisptr = ( zaxis_t * ) reshGetVal ( zaxisID, &zaxisOps );
+
+  zaxisPrintKernel ( zaxisptr, stdout );
+}
+
+
+static
+void zaxisPrintP ( void * voidptr, FILE * fp )
+{
+  zaxis_t *zaxisptr = ( zaxis_t * ) voidptr;
+
+  xassert ( zaxisptr );
+
+  zaxisPrintKernel(zaxisptr, fp);
+}
+
+
+static
+int  zaxisCompareP ( void * zaxisptr1, void * zaxisptr2 )
+{
+  zaxis_t * z1 = ( zaxis_t * ) zaxisptr1 ;
+  zaxis_t * z2 = ( zaxis_t * ) zaxisptr2 ;
+  static int differ = -1;
+  static int equal  =  0;
+  int i;
+
+  xassert ( z1 );
+  xassert ( z2 );
+
+  if ( z1->type      != z2->type )      return differ;
+  if ( z1->ltype     != z2->ltype )     return differ;
+  if ( z1->direction != z2->direction ) return differ;
+  if ( z1->prec      != z2->prec )      return differ;
+  if ( z1->size      != z2->size )      return differ;
+  if ( z1->vctsize   != z2->vctsize )   return differ;
+
+  if ( z1->vals )
+    {
+      xassert ( z1->size );
+
+      if ( !z2->vals ) return differ;
+
+      for ( i = 0; i < z1->size; i++ )
+        if ( z1->vals[i] != z2->vals[i] ) return differ;
+    }
+  else if ( z2->vals )
+    return differ;
+
+  if ( z1->lbounds )
+    {
+      xassert ( z1->size );
+
+      if ( !z2->lbounds ) return differ;
+
+      for ( i = 0; i < z1->size; i++ )
+        if ( z1->lbounds[i] != z2->lbounds[i] ) return differ;
+    }
+  else if ( z2->lbounds )
+    return differ;
+
+  if ( z1->ubounds )
+    {
+      xassert ( z1->size );
+
+      if ( !z2->ubounds ) return differ;
+
+      for ( i = 0; i < z1->size; i++ )
+        if ( z1->ubounds[i] != z2->ubounds[i] ) return differ;
+    }
+  else if ( z2->ubounds )
+    return differ;
+
+  if ( z1->weights )
+    {
+      xassert ( z1->size );
+
+      if ( !z2->weights ) return differ;
+
+      for ( i = 0; i < z1->size; i++ )
+        if ( z1->weights[i] != z2->weights[i] ) return differ;
+    }
+  else if ( z2->weights )
+    return differ;
+
+ if ( z1->vct )
+    {
+      xassert ( z1->vctsize );
+
+      if ( !z2->vct ) return differ;
+
+      for ( i = 0; i < z1->vctsize; i++ )
+        if ( z1->vct[i] != z2->vct[i] ) return differ;
+    }
+  else if ( z2->vct )
+    return differ;
+
+  if ( memcmp ( &z1->name    , &z2->name    , CDI_MAX_NAME ))
+    return differ;
+  if ( memcmp ( &z1->longname, &z2->longname, CDI_MAX_NAME ))
+    return differ;
+  if ( memcmp ( &z1->stdname , &z2->stdname , CDI_MAX_NAME ))
+    return differ;
+  if ( memcmp ( &z1->units   , &z2->units   , CDI_MAX_NAME ))
+    return differ;
+
+  if ( z1->positive != z2->positive ) return differ;
+
+
+  return equal;
+}
+
+
+#ifdef USE_MPI
+static int
+zaxisTxCode ( void )
+{
+  return ZAXIS;
+}
+
+enum { zaxisNint     = 7,
+       zaxisNstrings = 4,
+       vals     = 1 << 0,
+       lbounds  = 1 << 1,
+       ubounds  = 1 << 2,
+       weights  = 1 << 3,
+       vct      = 1 << 4
+};
+
+
+static
+int zaxisGetMemberMask ( zaxis_t * zaxisP )
+{
+  int memberMask = 0;
+
+  if ( zaxisP->vals )      memberMask |= vals;
+  if ( zaxisP->lbounds )   memberMask |= lbounds;
+  if ( zaxisP->ubounds )   memberMask |= ubounds;
+  if ( zaxisP->weights )   memberMask |= weights;
+  if ( zaxisP->vct )       memberMask |= vct;
+
+  return memberMask;
+}
+
+static
+int zaxisGetPackSize ( void * voidP, MPI_Comm comm )
+{
+  zaxis_t * zaxisP = ( zaxis_t * ) voidP;
+  int packBufferSize = 0, size;
+
+  xmpi ( MPI_Pack_size ( zaxisNint + 1, MPI_INT, comm, &size ));
+  packBufferSize += size;
+  xmpi ( MPI_Pack_size ( 1, MPI_DOUBLE, comm, &size ));
+  packBufferSize += size;
+
+  if ( zaxisP->vals )
+    {
+      xassert ( zaxisP->size );
+
+      xmpi ( MPI_Pack_size ( zaxisP->size, MPI_DOUBLE, comm, &size ));
+      packBufferSize += size;
+      xmpi ( MPI_Pack_size ( 1, MPI_DOUBLE, comm, &size ));
+      packBufferSize += size;
+    }
+
+  if ( zaxisP->lbounds )
+    {
+      xassert ( zaxisP->size );
+
+      xmpi ( MPI_Pack_size ( zaxisP->size, MPI_DOUBLE, comm, &size ));
+      packBufferSize += size;
+      xmpi ( MPI_Pack_size ( 1, MPI_DOUBLE, comm, &size ));
+      packBufferSize += size;
+    }
+
+  if ( zaxisP->ubounds )
+    {
+      xassert ( zaxisP->size );
+
+      xmpi ( MPI_Pack_size ( zaxisP->size, MPI_DOUBLE, comm, &size ));
+      packBufferSize += size;
+      xmpi ( MPI_Pack_size ( 1, MPI_DOUBLE, comm, &size ));
+      packBufferSize += size;
+    }
+
+  if ( zaxisP->weights )
+    {
+      xassert ( zaxisP->size );
+
+      xmpi ( MPI_Pack_size ( zaxisP->size, MPI_DOUBLE, comm, &size ));
+      packBufferSize += size;
+      xmpi ( MPI_Pack_size ( 1, MPI_DOUBLE, comm, &size ));
+      packBufferSize += size;
+    }
+
+  if ( zaxisP->vct )
+    {
+      xassert ( zaxisP->vctsize );
+
+      xmpi ( MPI_Pack_size ( zaxisP->vctsize, MPI_DOUBLE, comm, &size ));
+      packBufferSize += size;
+      xmpi ( MPI_Pack_size ( 1, MPI_DOUBLE, comm, &size ));
+      packBufferSize += size;
+    }
+
+  xmpi ( MPI_Pack_size ( zaxisNstrings * CDI_MAX_NAME , MPI_CHAR, comm, &size ));
+  packBufferSize += size;
+  xmpi ( MPI_Pack_size ( 1, MPI_DOUBLE, comm, &size ));
+  packBufferSize += size;
+  xmpi ( MPI_Pack_size ( 1, MPI_UNSIGNED_CHAR, comm, &size ));
+  packBufferSize += size;
+
+  return packBufferSize;
+}
+
+
+void zaxisUnpack ( char * unpackBuffer, int unpackBufferSize,
+                   int * unpackBufferPos, int nspTarget, MPI_Comm comm )
+{
+  zaxis_t * zaxisP;
+  int intBuffer[zaxisNint + 1], memberMask;
+  double d;
+  char charBuffer[zaxisNstrings * CDI_MAX_NAME];
+
+  xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                      intBuffer, zaxisNint + 1, MPI_INT, comm ));
+  xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                      &d, 1, MPI_DOUBLE, comm ));
+
+  xassert ( xchecksum ( DATATYPE_INT, zaxisNint + 1, intBuffer ) == d );
+
+  zaxisInit ();
+
+  zaxisP = zaxisNewEntry();
+  if ( ! zaxisP ) Error("No memory");
+
+  xassert ( namespaceAdaptKey ( intBuffer[0], nspTarget ) == zaxisP->self );
+
+  zaxisP->prec      = intBuffer[1];
+  zaxisP->type      = intBuffer[2];
+  zaxisP->ltype     = intBuffer[3];
+  zaxisP->size      = intBuffer[4];
+  zaxisP->direction = intBuffer[5];
+  zaxisP->vctsize   = intBuffer[6];
+  memberMask        = intBuffer[7];
+
+  if ( memberMask & vals )
+    {
+      xassert ( zaxisP->size );
+
+      zaxisP->vals = xmalloc ( zaxisP->size * sizeof ( double ));
+      xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          zaxisP->vals, zaxisP->size, MPI_DOUBLE, comm ));
+      xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          &d, 1, MPI_DOUBLE, comm ));
+      xassert ( xchecksum ( DATATYPE_FLT, zaxisP->size, zaxisP->vals ) == d );
+    }
+
+  if ( memberMask & lbounds )
+    {
+      xassert ( zaxisP->size );
+
+      zaxisP->lbounds = xmalloc ( zaxisP->size * sizeof ( double ));
+      xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          zaxisP->lbounds, zaxisP->size, MPI_DOUBLE, comm ));
+      xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          &d, 1, MPI_DOUBLE, comm ));
+      xassert ( xchecksum ( DATATYPE_FLT, zaxisP->size, zaxisP->lbounds ) == d );
+    }
+
+  if ( memberMask & ubounds )
+    {
+      xassert ( zaxisP->size );
+
+      zaxisP->ubounds = xmalloc ( zaxisP->size * sizeof ( double ));
+      xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          zaxisP->ubounds, zaxisP->size, MPI_DOUBLE, comm ));
+      xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          &d, 1, MPI_DOUBLE, comm ));
+      xassert ( xchecksum ( DATATYPE_FLT, zaxisP->size, zaxisP->ubounds ) == d );
+    }
+
+  if ( memberMask & weights )
+    {
+      xassert ( zaxisP->size );
+
+      zaxisP->weights = xmalloc ( zaxisP->size * sizeof ( double ));
+      xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          zaxisP->weights, zaxisP->size, MPI_DOUBLE, comm ));
+      xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          &d, 1, MPI_DOUBLE, comm ));
+      xassert ( xchecksum ( DATATYPE_FLT, zaxisP->size, zaxisP->weights ) == d );
+    }
+
+  if ( memberMask & vct )
+    {
+      xassert ( zaxisP->vctsize );
+
+      zaxisP->vct = xmalloc ( zaxisP->vctsize * sizeof ( double ));
+      xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          zaxisP->vct, zaxisP->vctsize, MPI_DOUBLE, comm ));
+      xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          &d, 1, MPI_DOUBLE, comm ));
+      xassert ( xchecksum ( DATATYPE_FLT, zaxisP->vctsize, zaxisP->vct ) == d );
+    }
+
+  xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                      charBuffer, zaxisNstrings * CDI_MAX_NAME, MPI_CHAR, comm ));
+  xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                      &d, 1, MPI_DOUBLE, comm ));
+
+  xassert ( d == xchecksum ( DATATYPE_TXT, zaxisNstrings * CDI_MAX_NAME, charBuffer ));
+
+  memcpy ( zaxisP->name,     &charBuffer[CDI_MAX_NAME * 0], CDI_MAX_NAME );
+  memcpy ( zaxisP->longname, &charBuffer[CDI_MAX_NAME * 1], CDI_MAX_NAME );
+  memcpy ( zaxisP->stdname,  &charBuffer[CDI_MAX_NAME * 2], CDI_MAX_NAME );
+  memcpy ( zaxisP->units,    &charBuffer[CDI_MAX_NAME * 3], CDI_MAX_NAME );
+
+  xmpi ( MPI_Unpack ( unpackBuffer, unpackBufferSize, unpackBufferPos,
+                      &zaxisP->positive, 1, MPI_UNSIGNED_CHAR, comm ));
+}
+
+static
+void zaxisPack ( void * voidP, void * packBuffer, int packBufferSize,
+                 int * packBufferPos, MPI_Comm comm )
+{
+  zaxis_t   * zaxisP = ( zaxis_t * ) voidP;
+  int intBuffer[zaxisNint + 1];
+  double d;
+  char charBuffer[zaxisNstrings * CDI_MAX_NAME];
+
+  intBuffer[0]  = zaxisP->self;
+  intBuffer[1]  = zaxisP->prec;
+  intBuffer[2]  = zaxisP->type;
+  intBuffer[3]  = zaxisP->ltype;
+  intBuffer[4]  = zaxisP->size;
+  intBuffer[5]  = zaxisP->direction;
+  intBuffer[6]  = zaxisP->vctsize;
+  intBuffer[7]  = zaxisGetMemberMask ( zaxisP );
+
+  xmpi ( MPI_Pack ( intBuffer, zaxisNint + 1, MPI_INT,
+                    packBuffer, packBufferSize, packBufferPos, comm ));
+  d = xchecksum ( DATATYPE_INT, zaxisNint + 1, intBuffer );
+  xmpi ( MPI_Pack ( &d, 1, MPI_DOUBLE,
+                    packBuffer, packBufferSize, packBufferPos, comm ));
+
+  if ( zaxisP->vals )
+    {
+
+      xassert ( zaxisP->size );
+
+      xmpi ( MPI_Pack ( zaxisP->vals, zaxisP->size, MPI_DOUBLE,
+                        packBuffer, packBufferSize, packBufferPos, comm ));
+      d = xchecksum ( DATATYPE_FLT, zaxisP->size, zaxisP->vals );
+      xmpi ( MPI_Pack ( &d, 1, MPI_DOUBLE,
+                        packBuffer, packBufferSize, packBufferPos, comm ));
+    }
+
+  if ( zaxisP->lbounds )
+    {
+      xassert ( zaxisP->size );
+
+      xmpi ( MPI_Pack ( zaxisP->lbounds, zaxisP->size, MPI_DOUBLE,
+                        packBuffer, packBufferSize, packBufferPos, comm ));
+      d = xchecksum ( DATATYPE_FLT, zaxisP->size, zaxisP->lbounds );
+      xmpi ( MPI_Pack ( &d, 1, MPI_DOUBLE,
+                        packBuffer, packBufferSize, packBufferPos, comm ));
+    }
+
+  if ( zaxisP->ubounds )
+    {
+      xassert ( zaxisP->size );
+
+      xmpi ( MPI_Pack ( zaxisP->ubounds, zaxisP->size, MPI_DOUBLE,
+                        packBuffer, packBufferSize, packBufferPos, comm ));
+      d = xchecksum ( DATATYPE_FLT, zaxisP->size, zaxisP->ubounds );
+      xmpi ( MPI_Pack ( &d, 1, MPI_DOUBLE,
+                        packBuffer, packBufferSize, packBufferPos, comm ));
+    }
+
+  if ( zaxisP->weights )
+    {
+      xassert ( zaxisP->size );
+
+      xmpi ( MPI_Pack ( zaxisP->weights, zaxisP->size, MPI_DOUBLE,
+                        packBuffer, packBufferSize, packBufferPos, comm ));
+      d = xchecksum ( DATATYPE_FLT, zaxisP->size, zaxisP->weights );
+      xmpi ( MPI_Pack ( &d, 1, MPI_DOUBLE,
+                        packBuffer, packBufferSize, packBufferPos, comm ));
+    }
+
+  if ( zaxisP->vct )
+    {
+      xassert ( zaxisP->vctsize );
+
+      xmpi ( MPI_Pack ( zaxisP->vct, zaxisP->vctsize, MPI_DOUBLE,
+                        packBuffer, packBufferSize, packBufferPos, comm ));
+      d = xchecksum ( DATATYPE_FLT, zaxisP->vctsize, zaxisP->vct );
+      xmpi ( MPI_Pack ( &d, 1, MPI_DOUBLE,
+                        packBuffer, packBufferSize, packBufferPos, comm ));
+    }
+
+  memcpy ( &charBuffer[CDI_MAX_NAME * 0], zaxisP->name,     CDI_MAX_NAME );
+  memcpy ( &charBuffer[CDI_MAX_NAME * 1], zaxisP->longname, CDI_MAX_NAME );
+  memcpy ( &charBuffer[CDI_MAX_NAME * 2], zaxisP->stdname,  CDI_MAX_NAME );
+  memcpy ( &charBuffer[CDI_MAX_NAME * 3], zaxisP->units,    CDI_MAX_NAME );
+
+  xmpi ( MPI_Pack ( charBuffer, zaxisNstrings * CDI_MAX_NAME, MPI_CHAR,
+                    packBuffer, packBufferSize, packBufferPos, comm ));
+  d = xchecksum  ( DATATYPE_TXT, zaxisNstrings * CDI_MAX_NAME, charBuffer );
+  xmpi ( MPI_Pack ( &d, 1, MPI_DOUBLE,
+                    packBuffer, packBufferSize, packBufferPos, comm ));
+
+  xmpi ( MPI_Pack ( &zaxisP->positive, 1, MPI_UNSIGNED_CHAR,
+                    packBuffer, packBufferSize, packBufferPos, comm ));
+}
+
+#endif
+
+void zaxisGetIndexList ( int nzaxis, int * zaxisResHs )
+{
+  reshGetResHListOfType ( nzaxis, zaxisResHs, &zaxisOps );
+}
 /*
  * Local Variables:
  * c-file-style: "Java"
