@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,6 @@
 #include "pstream.h"
 #include "util.h"
 
-void    vlistDefVarTime(int vlistID, int varID, int timeID);
 
 void *Cat(void *argument)
 {
@@ -61,7 +60,7 @@ void *Cat(void *argument)
 
       if ( indf == 0 )
 	{
-	  if ( fileExist(cdoStreamName(nfiles)) )
+	  if ( fileExists(cdoStreamName(nfiles)) )
 	    {
 	      streamID2 = streamOpenAppend(cdoStreamName(nfiles));
 
@@ -90,7 +89,7 @@ void *Cat(void *argument)
 	      if ( ntsteps == 1 )
 		{
 		  for ( varID = 0; varID < nvars; ++varID )
-		    if ( vlistInqVarTime(vlistID1, varID) == TIME_VARIABLE ) break;
+		    if ( vlistInqVarTsteptype(vlistID1, varID) != TSTEP_CONSTANT ) break;
 		  
 		  if ( varID == nvars ) ntsteps = 0;
 		}
@@ -98,7 +97,7 @@ void *Cat(void *argument)
 	      if ( ntsteps == 0 && nfiles > 1 )
 		{		  
 		  for ( varID = 0; varID < nvars; ++varID )
-		    vlistDefVarTime(vlistID2, varID, TIME_VARIABLE);
+		    vlistDefVarTsteptype(vlistID2, varID, TSTEP_INSTANT);
 		}
 
 	      streamDefVlist(streamID2, vlistID2);
