@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -79,6 +79,7 @@ void *Ensstat3(void *argument)
   double *uThresh = NULL, *lThresh = NULL;    // thresholds for histograms
   double **roc = NULL;                 // receiver operating characteristics table
   double val;
+  int ival;
   field_t *field;
   int fileID;
   const char *ofilename;
@@ -116,9 +117,9 @@ void *Ensstat3(void *argument)
   ofilename = cdoStreamName(nfiles);
 
   if ( !cdoSilentMode && !cdoOverwriteMode )
-    if ( fileExist(ofilename) )
+    if ( fileExists(ofilename) )
       if ( !userFileOverwrite(ofilename) )
-	cdoAbort("Outputfile %s already exist!", ofilename);
+	cdoAbort("Outputfile %s already exists!", ofilename);
 
   ef = (ens_file_t *) malloc(nfiles*sizeof(ens_file_t));
 
@@ -158,6 +159,7 @@ void *Ensstat3(void *argument)
 
   vlistID1 = ef[0].vlistID;
   vlistID2 = vlistCreate();
+  nvars = vlistNvars(vlistID1);
   varID2 = (int *) malloc( nvars*sizeof(int));
 
   levs = (double*) calloc ( nfiles, sizeof(double) );
@@ -167,7 +169,7 @@ void *Ensstat3(void *argument)
   zaxisDefLevels(zaxisID2,levs);
   zaxisDefName(zaxisID2, "histogram_binID");
 
-  time_mode = datafunc == TIME? TIME_VARIABLE : TIME_CONSTANT;
+  time_mode = datafunc == TIME? TSTEP_INSTANT : TSTEP_CONSTANT;
 
   for ( varID=0; varID<nvars; varID++) {
 
@@ -256,13 +258,20 @@ void *Ensstat3(void *argument)
 	  streamID = ef[fileID].streamID;
 	  nrecs = streamInqTimestep(streamID, tsID);
 	  if ( nrecs != nrecs0 )
-	    cdoAbort("Number of records changed from %d to %d", nrecs0, nrecs);
+	    {
+	      if ( nrecs == 0 )
+		cdoAbort("Inconsistent ensemble file, too few time steps in %s!", cdoStreamName(fileID));
+	      else
+		cdoAbort("Inconsistent ensemble file, number of records at time step %d of %s and %s differ!",
+			   tsID+1, cdoStreamName(0), cdoStreamName(fileID));
+	    }
 	}
 
-      if ( operfunc == func_rank && ( datafunc == TIME || tsID == 0 ) ) {
-	taxisCopyTimestep(taxisID2, taxisID1);
-	if ( nrecs0 > 0 ) streamDefTimestep(streamID2, tsID);
-      }
+      if ( operfunc == func_rank && ( datafunc == TIME || tsID == 0 ) )
+	{
+	  taxisCopyTimestep(taxisID2, taxisID1);
+	  if ( nrecs0 > 0 ) streamDefTimestep(streamID2, tsID);
+	}
 
       //      fprintf(stderr,"TIMESTEP %i varID %i rec %i\n",tsID,varID,recID);
       
@@ -336,7 +345,7 @@ void *Ensstat3(void *argument)
 		      /* RECEIVER OPERATING CHARACTERISTICS */
 		      /* ********************************** */
 		      dat = &field[ompthID].ptr[1];
-		      val = field[ompthID].ptr[0] > 0.5? 1 : 0;
+		      ival = field[ompthID].ptr[0] > 0.5? 1 : 0;
 
 		      for ( binID=0; binID<nbins; binID++ )
 			hist[binID] = 0;
@@ -353,7 +362,7 @@ void *Ensstat3(void *argument)
 		      if ( chksum != nens )  exit(1);
 
 		      cum = 0;
-		      if ( val == 1. ) 
+		      if ( ival == 1 ) 
 			{
 			  // all true positives in first bin
 			  ctg_tab[0][TP] += nens;
@@ -368,7 +377,7 @@ void *Ensstat3(void *argument)
 			  ctg_tab[binID][TP] += nens-cum;
 			  ctg_tab[binID][FN] += cum;
 			}
-		      else if ( val == 0. ) 
+		      else if ( ival == 0 ) 
 			{
 			  // all false positives in first bin
 			  ctg_tab[0][FP] += nens;
