@@ -212,9 +212,9 @@ void *Ensval(void *argument)
     fprintf(stderr,"StreamID %i: %s\n",stream,ofilename);
 
     if ( !cdoSilentMode && !cdoOverwriteMode )
-      if ( fileExist(ofilename) )
+      if ( fileExists(ofilename) )
 	if ( !userFileOverwrite(ofilename) )
-	    cdoAbort("Outputfile %s already exist!", ofilename);
+	    cdoAbort("Outputfile %s already exists!", ofilename);
 
     streamID2[stream] = streamOpenWrite(ofilename, cdoFiletype());    
 
@@ -250,7 +250,7 @@ void *Ensval(void *argument)
 	  streamID = ef[fileID].streamID;
 	  nrecs = streamInqTimestep(streamID, tsID);
 	  if ( nrecs != nrecs0 )
-	    cdoAbort("Number of records changed from %d to %d at time Step", nrecs0, nrecs, tsID);
+	    cdoAbort("Number of records at time step %d of %s and %s differ!", tsID+1, cdoStreamName(0), cdoStreamName(fileID));
 	}
       
       for ( stream = 0; stream < nostreams; stream++ ) {
@@ -412,7 +412,7 @@ void *Ensval(void *argument)
 	    // Last Bin
 	    p=1.; g=0.;
 	    o = 1. - heavysideN/gridsize; 
-	    if ( o != 1. ) {
+	    if ( IS_EQUAL(o,1.) ) {
 	      g = alpha[nens] / (1-o);
 	      
 	      crps_reli    += g * (o-p) * (o-p);
