@@ -2,7 +2,7 @@
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate model Data.
  
- Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+ Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
  See COPYING file for copying and redistribution conditions.
  
  This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,6 @@
 #include "cdo_int.h"
 #include "pstream.h"
 
-void    vlistDefVarTime(int vlistID, int varID, int timeID);
 
 // NO MISSING VALUE SUPPORT ADDED SO FAR
 
@@ -35,7 +34,7 @@ void *Eofcoeff(void * argument)
 {
   char eof_name[8], oname[1024], filesuffix[32];
   double *w;
-  double missval1=-999, missval2;
+  double missval1 = -999, missval2;
   double *xvals, *yvals;  
   field_t ***eof;  
   field_t in;  
@@ -73,7 +72,7 @@ void *Eofcoeff(void * argument)
     gridsize = vlistGridsizeMax(vlistID1);  
   else 
     {
-      gridsize=-1;
+      gridsize = -1;
       cdoAbort ("Gridsize of input files does not match");
     }
       
@@ -159,7 +158,7 @@ void *Eofcoeff(void * argument)
   
   vlistDefTaxis(vlistID3, taxisID3);
   for (varID =0; varID<nvars; varID++)
-    vlistDefVarTime(vlistID3, varID, TIME_VARIABLE);
+    vlistDefVarTsteptype(vlistID3, varID, TSTEP_INSTANT);
   
   // open streams for eofcoeff output
   streamIDs = (int *) malloc (neof*sizeof(int)); 
