@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -36,9 +36,16 @@
 
 static
 void print_location_LL(int operfunc, int vlistID, int varID, int levelID, int gridID, double sglval, double *fieldptr,
-		       int code, int year, int month, int day, int hour, int minute, int second)
+		       int vdate, int vtime)
 {
   static int showHeader = TRUE;
+  int year, month, day, hour, minute, second;
+  int code;
+
+  code = vlistInqVarCode(vlistID, varID);
+
+  cdiDecodeDate(vdate, &year, &month, &day);
+  cdiDecodeTime(vtime, &hour, &minute, &second);
 
   if ( gridInqType(gridID) == GRID_GAUSSIAN ||
        gridInqType(gridID) == GRID_LONLAT )
@@ -84,8 +91,6 @@ void *Fldstat(void *argument)
   int streamID1, streamID2;
   int vlistID1, vlistID2;
   int gridID2, lastgrid = -1;
-  int wstatus = FALSE;
-  int code = 0, oldcode = 0;
   int index, ngrids;
   int recID, nrecs;
   int tsID, varID, levelID;
@@ -170,16 +175,13 @@ void *Fldstat(void *argument)
       streamDefTimestep(streamID2, tsID);
 
       /* Precompute date + time for later representation in verbose mode */
-      int vdate, vtime;
-      int year, month, day, hour, minute, second;
+      int vdate = 0, vtime = 0;
       if ( cdoVerbose )
         {
           if ( operfunc == func_min || operfunc == func_max )
             {
               vdate = taxisInqVdate(taxisID1);
               vtime = taxisInqVtime(taxisID1);
-              cdiDecodeDate(vdate, &year, &month, &day);
-              cdiDecodeTime(vtime, &hour, &minute, &second);
             }
         }
 
@@ -190,32 +192,32 @@ void *Fldstat(void *argument)
 
 	  field.grid = vlistInqVarGrid(vlistID1, varID);
 	  field.size = gridInqSize(field.grid);
+
 	  if ( needWeights && field.grid != lastgrid )
 	    {
 	      lastgrid = field.grid;
-	      wstatus = gridWeights(field.grid, field.weight);
+	      field.weight[0] = 1;
+	      if ( field.size > 1 )
+		{
+		  int wstatus = gridWeights(field.grid, field.weight);
+		  if ( wstatus != 0 && tsID == 0 && levelID == 0 )
+		    {
+		      char varname[CDI_MAX_NAME];
+		      vlistInqVarName(vlistID1, varID, varname);
+		      cdoWarning("Using constant grid cell area weights for variable %s!", varname);
+		    }
+		}
 	    }
-	  code = vlistInqVarCode(vlistID1, varID);
-	  if ( wstatus != 0 && tsID == 0 && code != oldcode )
-	    cdoWarning("Using constant grid cell area weights for code %d!", oldcode=code);
 
 	  field.missval = vlistInqVarMissval(vlistID1, varID);
 
-	  /* RQ */
 	  if ( operfunc == func_pctl )
 	    sglval = fldpctl(field, pn);
 	  else  
 	    sglval = fldfun(field, operfunc);
-	  /* QR */
 
-	  if ( cdoVerbose )
-            {
-              if ( operfunc == func_min || operfunc == func_max )
-                {
-		  print_location_LL(operfunc, vlistID1, varID, levelID, field.grid, sglval, field.ptr,
-				    code, year, month, day, hour, minute, second);
-                }
-            }
+	  if ( cdoVerbose && (operfunc == func_min || operfunc == func_max) )
+	    print_location_LL(operfunc, vlistID1, varID, levelID, field.grid, sglval, field.ptr, vdate, vtime);
 
 	  if ( DBL_IS_EQUAL(sglval, field.missval) )
 	    nmiss = 1;
@@ -232,6 +234,8 @@ void *Fldstat(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
+  vlistDestroy(vlistID2);
+
   if ( field.ptr )    free(field.ptr);
   if ( field.weight ) free(field.weight);
 
