@@ -85,13 +85,22 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid)
   int gridID;
   int gridID2 = -1;
   int gridtype = -1;
-  int xsize = 0, ysize = 0;
+  int *xsize, *ysize;
+  int *xoff, *yoff;
   int xsize2, ysize2;
+  int idx;
   int nx, ny, ix, iy, i, j, ij, offset;
   double **xvals, **yvals;
   double *xvals2, *yvals2;
   xyinfo_t *xyinfo;
 
+  gridID   = vlistGrid(ef[0].vlistID, igrid);
+  gridtype = gridInqType(gridID);
+  if ( gridtype == GRID_GENERIC && gridInqXsize(gridID) == 0 && gridInqYsize(gridID) == 0 )
+    return (gridID2);
+
+  xsize = (int *) malloc(nfiles*sizeof(int));
+  ysize = (int *) malloc(nfiles*sizeof(int));
   xyinfo = (xyinfo_t *) malloc(nfiles*sizeof(xyinfo_t));
   xvals = (double **) malloc(nfiles*sizeof(double));
   yvals = (double **) malloc(nfiles*sizeof(double));
@@ -104,13 +113,16 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid)
 	    (gridtype == GRID_GENERIC && gridInqXsize(gridID) > 0 && gridInqYsize(gridID) > 0)) )
 	cdoAbort("Unsupported grid type: %s!", gridNamePtr(gridtype));
 
+      xsize[fileID] = gridInqXsize(gridID);
+      ysize[fileID] = gridInqYsize(gridID);
+      /*
       if ( xsize == 0 ) xsize = gridInqXsize(gridID);
       if ( ysize == 0 ) ysize = gridInqYsize(gridID);
       if ( xsize != gridInqXsize(gridID) ) cdoAbort("xsize differ!");
       if ( ysize != gridInqYsize(gridID) ) cdoAbort("ysize differ!");
-
-      xvals[fileID] = (double *) malloc(xsize*sizeof(double));
-      yvals[fileID] = (double *) malloc(ysize*sizeof(double));
+      */
+      xvals[fileID] = (double *) malloc(xsize[fileID]*sizeof(double));
+      yvals[fileID] = (double *) malloc(ysize[fileID]*sizeof(double));
       gridInqXvals(gridID, xvals[fileID]);
       gridInqYvals(gridID, yvals[fileID]);
 
@@ -120,9 +132,9 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid)
       xyinfo[fileID].y  = yvals[fileID][0];
       xyinfo[fileID].id = fileID;
 
-      if ( fileID == 0 && ysize > 1 )
+      if ( fileID == 0 && ysize[fileID] > 1 )
 	{
-	  if ( yvals[fileID][0] > yvals[fileID][ysize-1] ) lsouthnorth = FALSE;
+	  if ( yvals[fileID][0] > yvals[fileID][ysize[fileID]-1] ) lsouthnorth = FALSE;
 	}
     }
   
@@ -152,36 +164,54 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid)
       else break;
     }
   ny = nfiles/nx;
-  // printf("nx %d  ny %d\n", nx, ny);
-  xsize2 = nx*xsize;
-  ysize2 = ny*ysize;
+  if ( cdoVerbose ) cdoPrint("nx %d  ny %d", nx, ny);
+
+  xsize2 = 0;
+  for ( i = 0; i < nx; ++i ) xsize2 += xsize[xyinfo[i].id];
+  ysize2 = 0;
+  for ( j = 0; j < ny; ++j ) ysize2 += ysize[xyinfo[j*nx].id];
+  if ( cdoVerbose ) cdoPrint("xsize2 %d  ysize2 %d", xsize2, ysize2);
 
   xvals2 = (double *) malloc(xsize2*sizeof(double));
   yvals2 = (double *) malloc(ysize2*sizeof(double));
 
+  xoff = (int *) malloc((nx+1)*sizeof(int));
+  yoff = (int *) malloc((ny+1)*sizeof(int));
+
+  xoff[0] = 0;
   for ( i = 0; i < nx; ++i )
-    memcpy(xvals2+i*xsize, xvals[xyinfo[i].id], xsize*sizeof(double));
+    {
+      idx = xyinfo[i].id;
+      memcpy(xvals2+xoff[i], xvals[idx], xsize[idx]*sizeof(double));
+      xoff[i+1] = xoff[i] + xsize[idx];
+    }
 
+  yoff[0] = 0;
   for ( j = 0; j < ny; ++j )
-    memcpy(yvals2+j*ysize, yvals[xyinfo[j*nx].id], ysize*sizeof(double));
+    {
+      idx = xyinfo[j*nx].id;
+      memcpy(yvals2+yoff[j], yvals[idx], ysize[idx]*sizeof(double));
+      yoff[j+1] = yoff[j] + ysize[idx];
+    }
 
-  if ( igrid == 0 )
+  if ( gridindex != NULL )
     {
       for ( fileID = 0; fileID < nfiles; fileID++ )
 	{
+	  idx = xyinfo[fileID].id;
 	  iy = fileID/nx;
 	  ix = fileID - iy*nx;
 
-          offset = iy*xsize2*ysize + ix*xsize;
+          offset = yoff[iy]*xsize2 + xoff[ix];
 	  /*
 	  printf("fileID %d %d, iy %d, ix %d, offset %d\n",
 		 fileID, xyinfo[fileID].id, iy, ix, offset);
 	  */
 	  ij = 0;
-	  for ( j = 0; j < ysize; ++j )
-	    for ( i = 0; i < xsize; ++i )
+	  for ( j = 0; j < ysize[idx]; ++j )
+	    for ( i = 0; i < xsize[idx]; ++i )
 	      {
-		gridindex[xyinfo[fileID].id][ij++] = offset+j*xsize2+i;
+		gridindex[idx][ij++] = offset+j*xsize2+i;
 	      }
 	}
     }
@@ -192,6 +222,10 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid)
   gridDefXvals(gridID2, xvals2);
   gridDefYvals(gridID2, yvals2);
 
+  free(xoff);
+  free(yoff);
+  free(xsize);
+  free(ysize);
   free(xvals2);
   free(yvals2);
 
@@ -233,9 +267,10 @@ void *Gather(void *argument)
   int cmpfunc;
   int varID, recID;
   int gridsize = 0;
+  int gridsizemax = 0;
   int gridsize2;
-  int gridID2;
   int *gridIDs = NULL;
+  int *vars = NULL;
   int nrecs, nrecs0;
   int ngrids;
   int levelID;
@@ -258,9 +293,9 @@ void *Gather(void *argument)
   ofilename = cdoStreamName(nfiles);
 
   if ( !cdoSilentMode && !cdoOverwriteMode )
-    if ( fileExist(ofilename) )
+    if ( fileExists(ofilename) )
       if ( !userFileOverwrite(ofilename) )
-	cdoAbort("Outputfile %s already exist!", ofilename);
+	cdoAbort("Outputfile %s already exists!", ofilename);
 
   ef = (ens_file_t *) malloc(nfiles*sizeof(ens_file_t));
 
@@ -274,48 +309,78 @@ void *Gather(void *argument)
       ef[fileID].vlistID  = vlistID;
     }
 
-  /* check that the contents is always the same */
   nvars = vlistNvars(ef[0].vlistID);
+  vars  = (int *) malloc(nvars*sizeof(int));
+  for ( varID = 0; varID < nvars; varID++ ) vars[varID] = FALSE;
+
+  /* check that the contents is always the same */
   if ( nvars == 1 ) 
-    cmpfunc = CMP_NAME | CMP_GRIDSIZE | CMP_NLEVEL;
+    cmpfunc = CMP_NAME | CMP_NLEVEL;
   else
-    cmpfunc = CMP_NAME | CMP_GRIDSIZE | CMP_NLEVEL;
+    cmpfunc = CMP_NAME | CMP_NLEVEL;
 
   for ( fileID = 1; fileID < nfiles; fileID++ )
     vlistCompare(ef[0].vlistID, ef[fileID].vlistID, cmpfunc);
 
   vlistID1 = ef[0].vlistID;
-  gridsize = vlistGridsizeMax(vlistID1);
+  gridsizemax = vlistGridsizeMax(vlistID1);
+  for ( fileID = 1; fileID < nfiles; fileID++ )
+    {
+      gridsize = vlistGridsizeMax(ef[fileID].vlistID);
+      if ( gridsize > gridsizemax ) gridsizemax = gridsize;
+    }
+  gridsize = gridsizemax;
 
   for ( fileID = 0; fileID < nfiles; fileID++ )
-    ef[fileID].array = (double *) malloc(gridsize*sizeof(double));
+    ef[fileID].array = (double *) malloc(gridsizemax*sizeof(double));
 
   ngrids = vlistNgrids(ef[0].vlistID);
   gridIDs = (int *) malloc(ngrids*sizeof(int));
-  gridindex = (int **) malloc(nfiles*sizeof(int*));
+  gridindex = (int **) malloc(nfiles*sizeof(int *));
   for ( fileID = 0; fileID < nfiles; fileID++ )
-    gridindex[fileID] = (int *) malloc(gridsize*sizeof(int));
+    gridindex[fileID] = (int *) malloc(gridsizemax*sizeof(int));
 
+  int ginit = FALSE;
   for ( i = 0; i < ngrids; ++i )
-    gridIDs[i] = genGrid(nfiles, ef, gridindex, i);
-
+    {
+      if ( ginit == FALSE )
+	{
+	  gridIDs[i] = genGrid(nfiles, ef, gridindex, i);
+	  if ( gridIDs[i] != -1 ) ginit = TRUE;
+	}
+      else
+	gridIDs[i] = genGrid(nfiles, ef, NULL, i);
+    }
 
   vlistID2 = vlistDuplicate(vlistID1);
   taxisID1 = vlistInqTaxis(vlistID1);
   taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  gridID2 = gridIDs[0];
-  gridsize2 = gridInqSize(gridID2);
-  // printf("gridsize2 %d\n", gridsize2);
-  for ( i = 1; i < ngrids; ++i )
-    {
-      if ( gridsize2 != gridInqSize(gridIDs[i]) )
-	cdoAbort("gridsize differ!");
-    }
+  gridsize2 = 0;
   for ( i = 0; i < ngrids; ++i )
     {
-      vlistChangeGridIndex(vlistID2, i, gridIDs[i]);
+      if ( gridIDs[i] != -1 ) 
+	{
+	  if ( gridsize2 == 0 ) gridsize2 = gridInqSize(gridIDs[i]);
+	  if ( gridsize2 != gridInqSize(gridIDs[i]) ) cdoAbort("gridsize differ!");
+	  vlistChangeGridIndex(vlistID2, i, gridIDs[i]);
+	}
+    }
+
+  for ( varID = 0; varID < nvars; varID++ )
+    {
+      int gridID = vlistInqVarGrid(ef[0].vlistID, varID);
+
+      for ( i = 0; i < ngrids; ++i )
+	{
+	  if ( gridIDs[i] != -1 ) 
+	    {
+	      if ( gridID == vlistGrid(ef[0].vlistID, i) )
+	      vars[varID] = TRUE;
+	      break;
+	    }
+	}
     }
 
   streamID2 = streamOpenWrite(ofilename, cdoFiletype());
@@ -333,7 +398,7 @@ void *Gather(void *argument)
 	  streamID = ef[fileID].streamID;
 	  nrecs = streamInqTimestep(streamID, tsID);
 	  if ( nrecs != nrecs0 )
-	    cdoAbort("Number of records changed from %d to %d", nrecs0, nrecs);
+	    cdoAbort("Number of records at time step %d of %s and %s differ!", tsID+1, cdoStreamName(0), cdoStreamName(fileID));
 	}
 
       taxisCopyTimestep(taxisID2, taxisID1);
@@ -342,29 +407,42 @@ void *Gather(void *argument)
       
       for ( recID = 0; recID < nrecs0; recID++ )
 	{
-	  missval = vlistInqVarMissval(vlistID1, 0);
+	  streamID = ef[0].streamID;
+	  streamInqRecord(streamID, &varID, &levelID);
+
+	  missval = vlistInqVarMissval(vlistID1, varID);
 	  for ( i = 0; i < gridsize2; i++ ) array2[i] = missval;
 
 #if defined (_OPENMP)
-#pragma omp parallel for default(shared) private(fileID, streamID, nmiss, i) \
-                                     lastprivate(varID, levelID)
+#pragma omp parallel for default(shared) private(fileID, streamID, nmiss, i)
 #endif
 	  for ( fileID = 0; fileID < nfiles; fileID++ )
 	    {
+	      int varIDx, levelIDx;
 	      streamID = ef[fileID].streamID;
-	      streamInqRecord(streamID, &varID, &levelID);
+	      if ( fileID > 0 ) streamInqRecord(streamID, &varIDx, &levelIDx);
 	      streamReadRecord(streamID, ef[fileID].array, &nmiss);
 
-	      for ( i = 0; i < gridsize; ++i )
-		array2[gridindex[fileID][i]] = ef[fileID].array[i];
+	      if ( vars[varID] )
+		{
+		  gridsize = gridInqSize(vlistInqVarGrid(ef[fileID].vlistID, varID));
+		  for ( i = 0; i < gridsize; ++i )
+		    array2[gridindex[fileID][i]] = ef[fileID].array[i];
+		}
 	    }
 
-	  nmiss = 0;
-	  for ( i = 0; i < gridsize2; i++ )
-	    if ( DBL_IS_EQUAL(array2[i], missval) ) nmiss++;
-
 	  streamDefRecord(streamID2, varID, levelID);
-	  streamWriteRecord(streamID2, array2, nmiss);
+
+	  if ( vars[varID] )
+	    {
+	      nmiss = 0;
+	      for ( i = 0; i < gridsize2; i++ )
+		if ( DBL_IS_EQUAL(array2[i], missval) ) nmiss++;
+	      
+	      streamWriteRecord(streamID2, array2, nmiss);
+	    }
+	  else
+	    streamWriteRecord(streamID2, ef[0].array, 0);
 	}
 
       tsID++;
@@ -386,6 +464,7 @@ void *Gather(void *argument)
   if ( array2 ) free(array2);
 
   free(gridIDs);
+  if ( vars   ) free(vars);
 
   cdoFinish();
 
