@@ -35,7 +35,7 @@ int get_tunits(const char *unit, int *incperiod, int *incunit, int *tunit);
 
 void *Inttime(void *argument)
 {
-  int streamID1, streamID2;
+  int streamID1, streamID2 = -1;
   int nrecs, nvars, nlevel;
   int i, nrecords;
   int tsID, tsIDo, recID, varID, levelID;
@@ -133,17 +133,9 @@ void *Inttime(void *argument)
 
   taxisID1 = vlistInqTaxis(vlistID1);
   taxisID2 = taxisDuplicate(taxisID1);
-  if ( taxisHasBounds(taxisID2) )
-    {
-      cdoWarning("Time bounds unsupported by this operator, removed!");
-      taxisDeleteBounds(taxisID2);
-    }
+  if ( taxisHasBounds(taxisID2) ) taxisDeleteBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
-
-  streamDefVlist(streamID2, vlistID2);
-
   calendar = taxisInqCalendar(taxisID1);
 
   juldate = juldate_encode(calendar, vdate, vtime);
@@ -222,6 +214,13 @@ void *Inttime(void *argument)
 		  cdoPrint("%s %s  %f  %d", vdatestr, vtimestr, juldate_to_seconds(juldate), calendar);
 		}
 
+	      if ( streamID2 == -1 )
+		{
+		  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+
+		  streamDefVlist(streamID2, vlistID2);
+		}
+
 	      taxisDefVdate(taxisID2, vdate);
 	      taxisDefVtime(taxisID2, vtime);
 	      streamDefTimestep(streamID2, tsIDo++);
@@ -323,9 +322,11 @@ void *Inttime(void *argument)
 
   if ( array )  free(array);
 
-  streamClose(streamID2);
+  if ( streamID2 != -1 ) streamClose(streamID2);
   streamClose(streamID1);
 
+  if ( tsIDo == 0 ) cdoWarning("date/time out of time axis, no time step interpolated!");
+
   cdoFinish();
 
   return (0);
