@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -354,6 +354,34 @@ void get_remap_env(void)
     }
 }
 
+static
+void scale_gridbox_area(long gridsize, const double *restrict array1, long gridsize2, double *restrict array2, const double *restrict grid2_area)
+{
+  static int lgridboxinfo = TRUE;
+  long i;
+  double array1sum = 0;
+  double array2sum = 0;
+
+  for ( i = 0; i < gridsize; i++ )
+    array1sum += array1[i];
+
+  for ( i = 0; i < gridsize2; i++ )
+    array2sum += grid2_area[i];
+
+  for ( i = 0; i < gridsize2; i++ )
+    array2[i] = grid2_area[i]/array2sum*array1sum;
+
+  if ( lgridboxinfo )
+    {
+      cdoPrint("gridbox_area replaced and scaled to %g", array1sum);
+      lgridboxinfo = FALSE;
+    }
+}
+
+
+int timer_remap, timer_remap_init, timer_remap_sort;
+int timer_remap_bil, timer_remap_nn, timer_remap_con, timer_remap_con_l1, timer_remap_con_l2;
+
 
 void *Remap(void *argument)
 {
@@ -378,7 +406,6 @@ void *Remap(void *argument)
   int submap_type = SUBMAP_TYPE_NONE;
   int need_gradiants = FALSE;
   int non_global;
-  int lgridboxinfo = TRUE;
   int grid1sizemax;
   short *remapgrids = NULL;
   char varname[CDI_MAX_NAME];
@@ -390,6 +417,18 @@ void *Remap(void *argument)
   char *remap_file = NULL;
   int lwrite_remap;
 
+  if ( cdoTimer )
+    {
+      timer_remap        = timer_new("remap");
+      timer_remap_init   = timer_new("remap init");
+      timer_remap_sort   = timer_new("remap sort");
+      timer_remap_bil    = timer_new("remap bil");
+      timer_remap_nn     = timer_new("remap nn");
+      timer_remap_con    = timer_new("remap con");
+      timer_remap_con_l1 = timer_new("remap con loop1");
+      timer_remap_con_l2 = timer_new("remap con loop2");
+    }
+
   cdoInitialize(argument);
 
   cdoOperatorAdd("remapcon",    REMAPCON,    0, NULL);
@@ -483,9 +522,7 @@ void *Remap(void *argument)
     }
 
   for ( index = 0; index < ngrids; index++ )
-    {
-      if ( remapgrids[index] == TRUE ) break;
-    }
+    if ( remapgrids[index] == TRUE ) break;
 
   if ( index == ngrids )
     cdoAbort("No remappable grid found!");
@@ -595,7 +632,7 @@ void *Remap(void *argument)
 
       envstr = getenv("NORMALIZE_OPT");
 
-      if ( envstr )
+      if ( envstr && *envstr )
         {
 	  if      ( memcmp(envstr, "frac", 4) == 0 )
 	    norm_opt = NORM_OPT_FRACAREA;
@@ -805,8 +842,15 @@ void *Remap(void *argument)
 		  remaps[r].vars.norm_opt = norm_opt;
 		  remaps[r].vars.pinit = FALSE;
 		  
+		  if ( (map_type == MAP_TYPE_BILINEAR || map_type == MAP_TYPE_BICUBIC) &&
+		       (gridInqType(gridID1) == GRID_GME || gridInqType(gridID1) == GRID_UNSTRUCTURED) )
+		    cdoAbort("Bilinear/bicubic interpolation doesn't support unstructured source grids!");
+
 		  /* initialize grid information for both grids */
+		  if ( cdoTimer ) timer_start(timer_remap_init);
 		  remapGridInit(map_type, remap_extrapolate, gridID1, gridID2, &remaps[r].grid);
+		  if ( cdoTimer ) timer_stop(timer_remap_init);
+
 		  remaps[r].grid.store_link_fast = remap_store_link_fast;
 		}
 
@@ -831,7 +875,9 @@ void *Remap(void *argument)
 	      memset(remaps[r].grid.grid2_frac, 0, remaps[r].grid.grid2_size*sizeof(double));
 
 	      /* initialize some remapping variables */
+	      if ( cdoTimer ) timer_start(timer_remap_init);
 	      remapVarsInit(map_type, &remaps[r].grid, &remaps[r].vars);
+	      if ( cdoTimer ) timer_stop(timer_remap_init);
 
 	      if      ( map_type == MAP_TYPE_CONSERV  ) remap_conserv(&remaps[r].grid, &remaps[r].vars);
 	      else if ( map_type == MAP_TYPE_BILINEAR ) remap_bilin(&remaps[r].grid, &remaps[r].vars);
@@ -842,6 +888,7 @@ void *Remap(void *argument)
 	      if ( remaps[r].vars.num_links != remaps[r].vars.max_links )
 		resize_remap_vars(&remaps[r].vars, remaps[r].vars.num_links-remaps[r].vars.max_links);
 
+	      if ( cdoTimer ) timer_start(timer_remap_sort);
 	      if ( sort_mode == MERGE_SORT )
 		{ /* 
 		  ** use a combination of the old sort_add and a split and merge approach.
@@ -858,6 +905,7 @@ void *Remap(void *argument)
 			   remaps[r].vars.grid2_add, remaps[r].vars.grid1_add,
 			   remaps[r].vars.wts);
 		}
+	      if ( cdoTimer ) timer_stop(timer_remap_sort);
 	      	      
 	      if ( lwrite_remap ) goto WRITE_REMAP;
 
@@ -935,7 +983,7 @@ void *Remap(void *argument)
 	    double array2sum = 0;
    
 	    for ( i = 0; i < gridsize; i++ )
-	      printf("1 %d %g %g %g %g\n", i, array1[i], remaps[r].grid.grid1_frac[i],remaps[r].grid.grid1_area[i],remaps[r].grid.grid1_frac[i]);
+	      printf("1 %d %g %g %g %g\n", i, array1[i], remaps[r].grid.grid1_frac[i], remaps[r].grid.grid1_area[i],remaps[r].grid.grid1_frac[i]);
 	    for ( i = 0; i < gridsize; i++ )
 	      array1sum += remaps[r].grid.grid1_area[i];
 
@@ -951,23 +999,7 @@ void *Remap(void *argument)
 	  if ( operfunc == REMAPCON || operfunc == REMAPCON2 )
 	    if ( strcmp(varname, "gridbox_area") == 0 )
 	      {
-		double array1sum = 0;
-		double array2sum = 0;
-
-		for ( i = 0; i < gridsize; i++ )
-		  array1sum += array1[i];
-
-		for ( i = 0; i < gridsize2; i++ )
-		  array2sum += remaps[r].grid.grid2_area[i];
-
-		for ( i = 0; i < gridsize2; i++ )
-		  array2[i] = remaps[r].grid.grid2_area[i]/array2sum*array1sum;
-
-		if ( lgridboxinfo )
-		  {
-		    cdoPrint("%s replaced and scaled to %g", varname, array1sum);
-		    lgridboxinfo = FALSE;
-		  }
+		scale_gridbox_area(gridsize, array1, gridsize2, array2, remaps[r].grid.grid2_area);
 	      }
 
 	  /* calculate some statistics */
@@ -977,7 +1009,7 @@ void *Remap(void *argument)
 	  if ( gridInqType(gridID2) == GRID_GME )
 	    {
 	      int ni, nd;
-	      ni = gridInqGMEni(gridID2);
+ 	      ni = gridInqGMEni(gridID2);
 	      nd = gridInqGMEnd(gridID2);
 	      j = remaps[r].grid.grid2_size;
 
