@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -183,7 +183,7 @@ void *Replace(void *argument)
   gridsize = vlistGridsizeMax(vlistID1);
   array = (double *) malloc(gridsize*sizeof(double));
 
-  nts2 = streamNtsteps(streamID2);
+  nts2 = vlistNtsteps(vlistID2);
 
   tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
@@ -225,7 +225,6 @@ void *Replace(void *argument)
 	    if ( varlist1[idx] == varID )
 	      {
 		levelID2 = varlevel[idx][levelID];
-		printf("levelID, levelID2 %d %d\n", levelID, levelID2);
 		if ( levelID2 != -1 )
 		  {
 		    gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
