@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -690,6 +690,16 @@ int genindexgrid(int gridID1, int *lat1, int *lat2, int *lon11, int *lon12, int
       cdoWarning("First latitude index out of range, set to 1!");
       *lat1 = 1;
     }
+  if ( *lat2 > nlat1 )
+    {
+      cdoWarning("First latitude index out of range, set to %d!", nlat1);
+      *lat1 = nlat1;
+    }
+  if ( *lat2 < 1 )
+    {
+      cdoWarning("First latitude index out of range, set to 1!");
+      *lat2 = 1;
+    }
   if ( *lat2 > nlat1 )
     {
       cdoWarning("Last latitude index out of range, set to %d!", nlat1);
@@ -808,7 +818,7 @@ void *Selbox(void *argument)
   int gridID1 = -1, gridID2;
   int index, ngrids, gridtype = -1;
   int nmiss;
-  int *vars;
+  int *vars = NULL;
   int i;
   int nwpv; // number of words per value; real:1  complex:2
   double missval;
@@ -853,7 +863,7 @@ void *Selbox(void *argument)
 
       if ( gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_CURVILINEAR ||
 	   (operatorID == SELINDEXBOX && (gridtype == GRID_GENERIC || gridtype == GRID_SINUSOIDAL) && 
-	    gridInqXsize(gridID1) > 0 && gridInqYsize(gridID1) > 0 ) ||
+	    gridInqXsize(gridID1) > 0 && gridInqYsize(gridID1) > 0) ||
 	   (operatorID == SELLONLATBOX && gridtype == GRID_UNSTRUCTURED) )
 	{
 	  if ( operatorID == SELLONLATBOX )
@@ -877,6 +887,9 @@ void *Selbox(void *argument)
 	    if ( gridID1 == vlistInqVarGrid(vlistID1, varID) )
 	      vars[varID] = TRUE;
 	}
+      else if ( gridtype == GRID_GENERIC && gridInqXsize(gridID1) == 0 && gridInqYsize(gridID1) == 0 )
+	{
+	}
       else
 	{
 	  cdoPrint("Unsupported grid type: %s", gridNamePtr(gridtype));
@@ -919,28 +932,30 @@ void *Selbox(void *argument)
       for ( recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
+	  streamReadRecord(streamID1, array1, &nmiss);
+
+	  streamDefRecord(streamID2, varID, levelID);
 
 	  if ( vars[varID] )
 	    {
-	      streamReadRecord(streamID1, array1, &nmiss);
-
-	      gridID1 = vlistInqVarGrid(vlistID1, varID);
-	      for ( index = 0; index < ngrids; index++ )
-		if ( gridID1 == sbox[index].gridID1 ) break;
-
-	      if ( index == ngrids ) cdoAbort("Internal problem, grid not found!");
-
 	      if ( vlistInqVarDatatype(vlistID1, varID) == DATATYPE_CPX32 || 
 		   vlistInqVarDatatype(vlistID1, varID) == DATATYPE_CPX64 )
 		nwpv = 2;
 	      else
 		nwpv = 1;
+	      
+	      gridID1 = vlistInqVarGrid(vlistID1, varID);
+
+	      for ( index = 0; index < ngrids; index++ )
+		if ( gridID1 == sbox[index].gridID1 ) break;
+
+	      if ( index == ngrids ) cdoAbort("Internal problem, grid not found!");
 
 	      gridsize2 = gridInqSize(sbox[index].gridID2);
 
 	      if ( operatorID == SELLONLATBOX  && gridtype == GRID_UNSTRUCTURED )
 		window_cell(nwpv, array1, gridID1, array2, gridsize2, sbox[index].cellidx);
-	      else
+ 	      else
 		window(nwpv, array1, gridID1, array2, sbox[index].lat1, sbox[index].lat2, sbox[index].lon11, 
 		       sbox[index].lon12, sbox[index].lon21, sbox[index].lon22);
 
@@ -952,9 +967,12 @@ void *Selbox(void *argument)
 		    if ( DBL_IS_EQUAL(array2[i], missval) ) nmiss++;
 		}
 
-	      streamDefRecord(streamID2, varID, levelID);
 	      streamWriteRecord(streamID2, array2, nmiss);
 	    }
+	  else
+	    {
+	      streamWriteRecord(streamID2, array1, nmiss);
+	    }
 	}
       tsID++;
     }
@@ -962,6 +980,8 @@ void *Selbox(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
+  vlistDestroy(vlistID2);
+
   if ( vars   ) free(vars);
   if ( array2 ) free(array2);
   if ( array1 ) free(array1);
