@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,8 +30,6 @@
 #include "list.h"
 #include "namelist.h"
 
-void    vlistDefVarTime(int vlistID, int varID, int timeID);
-
 
 #define  PML_INT         1
 #define  PML_FLT         2
@@ -286,7 +284,6 @@ int pmlRead(pml_t *pml, int argc, char **argv)
   pml_entry_t *pentry[MAX_PML_ENTRY];
   int params[MAX_PML_ENTRY];
   int num_par[MAX_PML_ENTRY];
-  int len_par[MAX_PML_ENTRY];
   int nparams = 0;
   int i, istart;
   char *epos;
@@ -301,7 +298,6 @@ int pmlRead(pml_t *pml, int argc, char **argv)
   for ( i = 0; i < argc; ++i )
     {
       len = strlen(argv[i]);
-      len_par[i] = (int)len;
       bufsize += len+1;
     }
 
@@ -311,7 +307,6 @@ int pmlRead(pml_t *pml, int argc, char **argv)
   istart = 0;
   while ( istart < argc )
     {
-
       epos = strchr(argv[istart], '=');
       if ( epos == NULL )
 	{
@@ -343,6 +338,7 @@ int pmlRead(pml_t *pml, int argc, char **argv)
       istart++;
       for ( i = istart; i < argc; ++i )
 	{
+	  if ( *argv[i] == 0 ) { i++; break;}
 	  epos = strchr(argv[i], '=');
 	  if ( epos != NULL ) break;
 
@@ -410,7 +406,7 @@ void par_check_int_flag(int npar, int *parlist, int *flaglist, const char *txt)
 
   for ( i = 0; i < npar; ++i )
     if ( flaglist[i] == FALSE )
-      cdoWarning("%s %d not found!", txt, parlist[i]);
+      cdoWarning("%s >%d< not found!", txt, parlist[i]);
 }
 
 
@@ -420,7 +416,7 @@ void par_check_flt_flag(int npar, double *parlist, int *flaglist, const char *tx
 
   for ( i = 0; i < npar; ++i )
     if ( flaglist[i] == FALSE )
-      cdoWarning("%s %g not found!", txt, parlist[i]);
+      cdoWarning("%s >%g< not found!", txt, parlist[i]);
 }
 
 
@@ -430,7 +426,7 @@ void par_check_word_flag(int npar, char **parlist, int *flaglist, const char *tx
 
   for ( i = 0; i < npar; ++i )
     if ( flaglist[i] == FALSE )
-      cdoWarning("%s %s not found!", txt, parlist[i]);
+      cdoWarning("%s >%s< not found!", txt, parlist[i]);
 }
 
 
@@ -446,6 +442,8 @@ void *Select(void *argument)
   int recID, varID, levelID;
   int iparam;
   int nsel;
+  int vdate, vtime;
+  int copytimestep;
   char paramstr[32];
   char varname[CDI_MAX_NAME];
   char stdname[CDI_MAX_NAME];
@@ -460,17 +458,28 @@ void *Select(void *argument)
   double *array = NULL;
   int taxisID1, taxisID2 = CDI_UNDEFID;
   int ntsteps;
+  int ltimsel = FALSE;
   int npar;
   int *vars = NULL;
   pml_t *pml;
-  PML_DEF_INT(code,    1024, "Code number");
-  PML_DEF_INT(ltype,   1024, "Level type");
-  PML_DEF_FLT(level,   1024, "Level");
-  PML_DEF_WORD(name,   1024, "Variable name");
-  PML_DEF_WORD(param,  1024, "Parameter");
+  PML_DEF_INT(timestep, 4096, "Timestep");
+  PML_DEF_INT(year,     1024, "Year");
+  PML_DEF_INT(month,      32, "Month");
+  PML_DEF_INT(day,        32, "Day");
+  PML_DEF_INT(code,     1024, "Code number");
+  PML_DEF_INT(ltype,    1024, "Level type");
+  PML_DEF_INT(levidx,   1024, "Level index");
+  PML_DEF_FLT(level,    1024, "Level");
+  PML_DEF_WORD(name,    1024, "Variable name");
+  PML_DEF_WORD(param,   1024, "Parameter");
 
+  PML_INIT_INT(timestep);
+  PML_INIT_INT(year);
+  PML_INIT_INT(month);
+  PML_INIT_INT(day);
   PML_INIT_INT(code);
   PML_INIT_INT(ltype);
+  PML_INIT_INT(levidx);
   PML_INIT_FLT(level);
   PML_INIT_WORD(name);
   PML_INIT_WORD(param);
@@ -488,15 +497,20 @@ void *Select(void *argument)
 
   nsel     = operatorArgc();
   argnames = operatorArgv();
-  /*
+
   if ( cdoVerbose )
     for ( i = 0; i < nsel; i++ )
       printf("name %d = %s\n", i+1, argnames[i]);
-  */
+
   pml = pmlNew("SELECT");
 
+  PML_ADD_INT(pml, timestep);
+  PML_ADD_INT(pml, year);
+  PML_ADD_INT(pml, month);
+  PML_ADD_INT(pml, day);
   PML_ADD_INT(pml, code);
   PML_ADD_INT(pml, ltype);
+  PML_ADD_INT(pml, levidx);
   PML_ADD_FLT(pml, level);
   PML_ADD_WORD(pml, name);
   PML_ADD_WORD(pml, param);
@@ -505,8 +519,13 @@ void *Select(void *argument)
 
   if ( cdoVerbose ) pmlPrint(pml);
 
+  PML_NUM(pml, timestep);
+  PML_NUM(pml, year);
+  PML_NUM(pml, month);
+  PML_NUM(pml, day);
   PML_NUM(pml, code);
   PML_NUM(pml, ltype);
+  PML_NUM(pml, levidx);
   PML_NUM(pml, level);
   PML_NUM(pml, name);
   PML_NUM(pml, param);
@@ -562,21 +581,47 @@ void *Select(void *argument)
 	      param = paramstr;
 
 	      zaxisID = vlistInqVarZaxis(vlistID1, varID);
+	      nlevs   = zaxisInqSize(zaxisID);
 	      ltype   = zaxis2ltype(zaxisID);
 
+
 	      vars[varID] = FALSE;
 	      
 	      if ( npar_ltype )
 		{
-		  if ( npar_code  && PAR_CHECK_INT(ltype) && PAR_CHECK_INT(code) )   vars[varID] = TRUE;
-		  if ( npar_name  && PAR_CHECK_INT(ltype) && PAR_CHECK_WORD(name) )  vars[varID] = TRUE;
-		  if ( npar_param && PAR_CHECK_INT(ltype) && PAR_CHECK_WORD(param) ) vars[varID] = TRUE;
+		  if ( !vars[varID] && npar_code  && PAR_CHECK_INT(ltype) && PAR_CHECK_INT(code) )   vars[varID] = TRUE;
+		  if ( !vars[varID] && npar_name  && PAR_CHECK_INT(ltype) && PAR_CHECK_WORD(name) )  vars[varID] = TRUE;
+		  if ( !vars[varID] && npar_param && PAR_CHECK_INT(ltype) && PAR_CHECK_WORD(param) ) vars[varID] = TRUE;
+		  if ( !vars[varID] && !npar_code && !npar_name && !npar_param )
+		    {
+		      if ( PAR_CHECK_INT(ltype) ) vars[varID] = TRUE;
+		      else
+			{
+			  for ( levID = 0; levID < nlevs; levID++ )
+			    {
+			      levidx = levID + 1;
+			      level = zaxisInqLevel(zaxisID, levID);
+			      if ( !vars[varID] && npar_levidx && PAR_CHECK_INT(ltype) && PAR_CHECK_INT(levidx) )  vars[varID] = TRUE;
+			      if ( !vars[varID] && npar_level  && PAR_CHECK_INT(ltype) && PAR_CHECK_FLT(level)  )  vars[varID] = TRUE;
+			    }
+			}
+		    }
 		}
 	      else
 		{
-		  if ( npar_code  && PAR_CHECK_INT(code) )   vars[varID] = TRUE;
-		  if ( npar_name  && PAR_CHECK_WORD(name) )  vars[varID] = TRUE;
-		  if ( npar_param && PAR_CHECK_WORD(param) ) vars[varID] = TRUE;
+		  if ( !vars[varID] && npar_code  && PAR_CHECK_INT(code) )   vars[varID] = TRUE;
+		  if ( !vars[varID] && npar_name  && PAR_CHECK_WORD(name) )  vars[varID] = TRUE;
+		  if ( !vars[varID] && npar_param && PAR_CHECK_WORD(param) ) vars[varID] = TRUE;
+		  if ( !vars[varID] && !npar_code && !npar_name && !npar_param )
+		    {
+		      for ( levID = 0; levID < nlevs; levID++ )
+			{
+			  levidx = levID + 1;
+			  level = zaxisInqLevel(zaxisID, levID);
+			  if ( !vars[varID] && npar_levidx && PAR_CHECK_INT(levidx) )  vars[varID] = TRUE;
+			  if ( !vars[varID] && npar_level  && PAR_CHECK_FLT(level)  )  vars[varID] = TRUE;
+			}
+		    }
 		}
 	    }
 
@@ -589,6 +634,7 @@ void *Select(void *argument)
 
 		  for ( levID = 0; levID < nlevs; levID++ )
 		    {
+		      levidx = levID + 1;
 		      level = zaxisInqLevel(zaxisID, levID);
 		      
 		      if ( nlevs == 1 && IS_EQUAL(level, 0) )
@@ -597,7 +643,12 @@ void *Select(void *argument)
 			}
 		      else
 			{
-			  if ( npar_level )
+			  if ( npar_levidx )
+			    {
+			      if ( PAR_CHECK_INT(levidx) )
+				vlistDefFlag(vlistID1, varID, levID, result);
+			    }
+			  else if ( npar_level )
 			    {
 			      if ( PAR_CHECK_FLT(level) )
 				vlistDefFlag(vlistID1, varID, levID, result);
@@ -613,10 +664,13 @@ void *Select(void *argument)
 
 	  PAR_CHECK_INT_FLAG(code);
 	  PAR_CHECK_INT_FLAG(ltype);
+	  PAR_CHECK_INT_FLAG(levidx);
 	  PAR_CHECK_FLT_FLAG(level);
 	  PAR_CHECK_WORD_FLAG(name);
 	  PAR_CHECK_WORD_FLAG(param);
 
+	  if ( npar_timestep || npar_year || npar_month || npar_day ) ltimsel = TRUE;
+
 	  npar = 0;
 	  for ( varID = 0; varID < nvars; varID++ )
 	    {
@@ -630,8 +684,24 @@ void *Select(void *argument)
 	    }
 
 	  if ( npar == 0 )
-	    cdoAbort("No variable selected!");
+	    {
+	      if ( ltimsel == TRUE )
+		{
+		  for ( varID = 0; varID < nvars; varID++ )
+		    {
+		      vars[varID] = TRUE;
+		      zaxisID = vlistInqVarZaxis(vlistID1, varID);
+		      nlevs   = zaxisInqSize(zaxisID);
 
+		      for ( levID = 0; levID < nlevs; levID++ )
+			vlistDefFlag(vlistID1, varID, levID, TRUE);
+		    }
+		}
+	      else
+		{
+		  cdoAbort("No variable selected!");
+		}
+	    }
 
 	  // if ( cdoVerbose ) vlistPrint(vlistID1);
 
@@ -650,6 +720,10 @@ void *Select(void *argument)
 	  vlistCopyFlag(vlistID2, vlistID1);
 
 	  // if ( cdoVerbose ) vlistPrint(vlistID2);
+	  nvars = vlistNvars(vlistID2);
+	  for ( varID = 0; varID < nvars; ++varID )
+	    if ( vlistInqVarTsteptype(vlistID2, varID) != TSTEP_CONSTANT ) break;
+	  if ( varID == nvars ) vlistDefNtsteps(vlistID2, 0);
 
 	  taxisID2 = taxisDuplicate(taxisID1);
 	  vlistDefTaxis(vlistID2, taxisID2);
@@ -659,7 +733,7 @@ void *Select(void *argument)
 	  if ( ntsteps == 1 )
 	    {
 	      for ( varID = 0; varID < nvars; ++varID )
-		if ( vlistInqVarTime(vlistID1, varID) == TIME_VARIABLE ) break;
+		if ( vlistInqVarTsteptype(vlistID1, varID) != TSTEP_CONSTANT ) break;
 	      
 	      if ( varID == nvars ) ntsteps = 0;
 	    }
@@ -667,7 +741,7 @@ void *Select(void *argument)
 	  if ( ntsteps == 0 && nfiles > 1 )
 	    {	      
 	      for ( varID = 0; varID < nvars; ++varID )
-		vlistDefVarTime(vlistID2, varID, TIME_VARIABLE);
+		vlistDefVarTsteptype(vlistID2, varID, TSTEP_INSTANT);
 	    }
 
 	  streamID2 = streamOpenWrite(cdoStreamName(nfiles), cdoFiletype());
@@ -686,43 +760,79 @@ void *Select(void *argument)
 	  vlistCompare(vlistID0, vlistID1, CMP_ALL);
 	}
 
+
       tsID1 = 0;
       while ( (nrecs = streamInqTimestep(streamID1, tsID1)) )
 	{
-	  taxisCopyTimestep(taxisID2, taxisID1);
-
-	  streamDefTimestep(streamID2, tsID2);
-     
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  if ( ltimsel == TRUE )
 	    {
-	      streamInqRecord(streamID1, &varID, &levelID);
-	      if ( vlistInqFlag(vlistID0, varID, levelID) == TRUE )
-		{
-		  varID2   = vlistFindVar(vlistID2, varID);
-		  levelID2 = vlistFindLevel(vlistID2, varID, levelID);
+	      copytimestep = FALSE;
+	      timestep = tsID1 + 1;
 
-		  streamDefRecord(streamID2, varID2, levelID2);
-		  if ( lcopy )
+	      if ( npar_timestep && PAR_CHECK_INT(timestep) ) copytimestep = TRUE;
+
+	      if ( !copytimestep && npar_timestep == 0 )
+		{
+		  int lyear = 0, lmonth = 0, lday = 0;
+		  vdate = taxisInqVdate(taxisID1);
+		  vtime = taxisInqVtime(taxisID1);
+		  cdiDecodeDate(vdate, &year, &month, &day);
+		  if ( npar_year  == 0 || (npar_year   && PAR_CHECK_INT(year))  ) lyear  = TRUE;
+		  if ( npar_month == 0 || (npar_month  && PAR_CHECK_INT(month)) ) lmonth = TRUE;
+		  if ( npar_day   == 0 || (npar_day    && PAR_CHECK_INT(day))   ) lday   = TRUE;
+
+		  if ( lyear && lmonth && lday ) copytimestep = TRUE;
+		}
+
+	      if ( operatorID == DELETE ) copytimestep = !copytimestep;
+	    }
+	  else
+	    {
+	      copytimestep = TRUE;
+	    }
+
+	  if ( copytimestep == TRUE )
+	    {
+	      taxisCopyTimestep(taxisID2, taxisID1);
+
+	      streamDefTimestep(streamID2, tsID2);
+     
+	      for ( recID = 0; recID < nrecs; recID++ )
+		{
+		  streamInqRecord(streamID1, &varID, &levelID);
+		  if ( vlistInqFlag(vlistID0, varID, levelID) == TRUE )
 		    {
-		      streamCopyRecord(streamID2, streamID1);
-		    }
-		  else
-		    {
-		      streamReadRecord(streamID1, array, &nmiss);
-		      streamWriteRecord(streamID2, array, nmiss);
+		      varID2   = vlistFindVar(vlistID2, varID);
+		      levelID2 = vlistFindLevel(vlistID2, varID, levelID);
+		      
+		      streamDefRecord(streamID2, varID2, levelID2);
+		      if ( lcopy )
+			{
+			  streamCopyRecord(streamID2, streamID1);
+			}
+		      else
+			{
+			  streamReadRecord(streamID1, array, &nmiss);
+			  streamWriteRecord(streamID2, array, nmiss);
+			}
 		    }
 		}
-     	    }
+	      tsID2++;
+	    }
 
 	  tsID1++;
-	  tsID2++;
 	}
       
       streamClose(streamID1);
     }
 
+  PAR_CHECK_INT_FLAG(timestep);
+  PAR_CHECK_INT_FLAG(year);
+  PAR_CHECK_INT_FLAG(month);
+  PAR_CHECK_INT_FLAG(day);
+
   streamClose(streamID2);
- 
+
   vlistDestroy(vlistID0);
   vlistDestroy(vlistID2);
 
