@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -383,12 +383,16 @@ void *Selvar(void *argument)
 	}
     }
   
-  if ( npar == 0 )
-    cdoAbort("No variables selected!");
+  if ( npar == 0 ) cdoAbort("No variables selected!");
 
   vlistID2 = vlistCreate();
   vlistCopyFlag(vlistID2, vlistID1);
 
+  nvars = vlistNvars(vlistID2);
+  for ( varID = 0; varID < nvars; ++varID )
+    if ( vlistInqVarTsteptype(vlistID2, varID) != TSTEP_CONSTANT ) break;
+  if ( varID == nvars ) vlistDefNtsteps(vlistID2, 0);
+
   taxisID1 = vlistInqTaxis(vlistID1);
   taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
