@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,14 +25,17 @@
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
+#include "util.h"
 
 #include "printinfo.h"
 
+#define MAXCHARS 82
 
 void *Sinfo(void *argument)
 {
-  int SINFO, SINFOP, SINFON, SINFOC;
+  enum {func_generic, func_param, func_name, func_code};
   int operatorID;
+  int operfunc, lensemble;
   int indf;
   int varID;
   int gridsize = 0;
@@ -42,7 +45,7 @@ void *Sinfo(void *argument)
   int nrecs, nvars, nzaxis, ntsteps;
   int levelID, levelsize;
   int tsID, ntimeout;
-  int timeID, taxisID;
+  int tsteptype, taxisID;
   int nbyte, nbyte0;
   int index;
   char varname[CDI_MAX_NAME];
@@ -59,13 +62,20 @@ void *Sinfo(void *argument)
 
   cdoInitialize(argument);
 
-  SINFO  = cdoOperatorAdd("sinfo",  0, 0, NULL);
-  SINFOP = cdoOperatorAdd("sinfop", 0, 0, NULL);
-  SINFON = cdoOperatorAdd("sinfon", 0, 0, NULL);
-  SINFOC = cdoOperatorAdd("sinfoc", 0, 0, NULL);
+  cdoOperatorAdd("sinfo",   func_generic, 0, NULL);
+  cdoOperatorAdd("sinfop",  func_param,   0, NULL);
+  cdoOperatorAdd("sinfon",  func_name,    0, NULL);
+  cdoOperatorAdd("sinfoc",  func_code,    0, NULL);
+  cdoOperatorAdd("seinfo",  func_generic, 1, NULL);
+  cdoOperatorAdd("seinfop", func_param,   1, NULL);
+  cdoOperatorAdd("seinfon", func_name,    1, NULL);
+  cdoOperatorAdd("seinfoc", func_code,    1, NULL);
 
   operatorID = cdoOperatorID();
 
+  operfunc  = cdoOperatorF1(operatorID);
+  lensemble = cdoOperatorF2(operatorID);
+
   for ( indf = 0; indf < cdoStreamCnt(); indf++ )
     {
       streamID = streamOpenRead(cdoStreamName(indf));
@@ -75,15 +85,30 @@ void *Sinfo(void *argument)
       printf("   File format: ");
       printFiletype(streamID, vlistID);
 
-      if ( operatorID == SINFON )
-	fprintf(stdout,
-		"%6d : Institut Source   Name        Time Typ  Grid Size Num  Levels Num\n",  -(indf+1));
-      else if ( operatorID == SINFOC )
-	fprintf(stdout,
-		"%6d : Institut Source  Table Code   Time Typ  Grid Size Num  Levels Num\n",  -(indf+1));
+      if ( lensemble )
+	{
+	  if ( operfunc == func_name )
+	    fprintf(stdout,
+		    "%6d : Institut Source   Ttype    Einfo Levels Num  Gridsize Num Dtype : Parameter name\n",  -(indf+1));
+	  else if ( operfunc == func_code )
+	    fprintf(stdout,
+		    "%6d : Institut Source   Ttype    Einfo Levels Num  Gridsize Num Dtype : Table Code\n",  -(indf+1));
+	  else
+	    fprintf(stdout,
+		    "%6d : Institut Source   Ttype    Einfo Levels Num  Gridsize Num Dtype : Parameter ID\n",  -(indf+1));
+	}
       else
-	fprintf(stdout,
-		"%6d : Institut Source   Param       Time Typ  Grid Size Num  Levels Num\n",  -(indf+1));
+	{
+	  if ( operfunc == func_name )
+	    fprintf(stdout,
+		    "%6d : Institut Source   Ttype    Levels Num  Gridsize Num Dtype : Parameter name\n",  -(indf+1));
+	  else if ( operfunc == func_code )
+	    fprintf(stdout,
+		    "%6d : Institut Source   Ttype    Levels Num  Gridsize Num Dtype : Table Code\n",  -(indf+1));
+	  else
+	    fprintf(stdout,
+		    "%6d : Institut Source   Ttype    Levels Num  Gridsize Num Dtype : Parameter ID\n",  -(indf+1));
+	}
 
       nvars = vlistNvars(vlistID);
 
@@ -95,79 +120,86 @@ void *Sinfo(void *argument)
 	  gridID  = vlistInqVarGrid(vlistID, varID);
 	  zaxisID = vlistInqVarZaxis(vlistID, varID);
 
-	  cdiParamToString(param, paramstr, sizeof(paramstr));
-
-	  if ( operatorID == SINFON ) vlistInqVarName(vlistID, varID, varname);
-
-	  gridsize = gridInqSize(gridID);
-
 	  fprintf(stdout, "%6d : ", varID + 1);
 
+	  /* institute info */
 	  instptr = institutInqNamePtr(vlistInqVarInstitut(vlistID, varID));
 	  if ( instptr )
-	    fprintf(stdout, "%-9s", instptr);
+	    fprintf(stdout, "%-8s ", instptr);
 	  else
 	    fprintf(stdout, "unknown  ");
 
+	  /* source info */
 	  modelptr = modelInqNamePtr(vlistInqVarModel(vlistID, varID));
 	  if ( modelptr )
-	    fprintf(stdout, "%-9s", modelptr);
+	    fprintf(stdout, "%-8s ", modelptr);
 	  else
 	    fprintf(stdout, "unknown  ");
 
-	  if ( operatorID == SINFON )
-	    fprintf(stdout, "%-11s ", varname);
-	  else if ( operatorID == SINFOC )
-	    fprintf(stdout, "%4d %4d   ", tabnum, code);
-	  else
-	    fprintf(stdout, "%-11s ", paramstr);
+	  /* tsteptype */
+	  tsteptype = vlistInqVarTsteptype(vlistID, varID);
+	  if      ( tsteptype == TSTEP_CONSTANT ) fprintf(stdout, "%-8s ", "constant");
+	  else if ( tsteptype == TSTEP_INSTANT  ) fprintf(stdout, "%-8s ", "instant");
+	  else if ( tsteptype == TSTEP_INSTANT2 ) fprintf(stdout, "%-8s ", "instant");
+	  else if ( tsteptype == TSTEP_INSTANT3 ) fprintf(stdout, "%-8s ", "instant");
+	  else if ( tsteptype == TSTEP_MIN      ) fprintf(stdout, "%-8s ", "min");
+	  else if ( tsteptype == TSTEP_MAX      ) fprintf(stdout, "%-8s ", "max");
+	  else if ( tsteptype == TSTEP_ACCUM    ) fprintf(stdout, "%-8s ", "accum");
+	  else                                    fprintf(stdout, "%-8s ", "unknown");
 
-	  timeID = vlistInqVarTime(vlistID, varID);
+	  /* ensemble information */
+	  if ( lensemble )
+	    {
+	      int ensID, ensCount, forecast_type;
+	      if ( vlistInqVarEnsemble(vlistID, varID, &ensID, &ensCount, &forecast_type) )
+		fprintf(stdout, "%2d/%-2d ", ensID, ensCount);
+	      else
+		fprintf(stdout, "--/-- ");
+	    }
 
-	  if ( timeID == TIME_CONSTANT )
-	    fprintf(stdout, "con ");
-	  else
-	    fprintf(stdout, "var ");
+	  /* layer info */
+	  levelsize = zaxisInqSize(zaxisID);
+	  fprintf(stdout, "%6d ", levelsize);
+	  fprintf(stdout, "%3d ", vlistZaxisIndex(vlistID, zaxisID) + 1);
 
+	  /* grid info */
+	  gridsize = gridInqSize(gridID);
+	  fprintf(stdout, "%9d ", gridsize);
+	  fprintf(stdout, "%3d ", vlistGridIndex(vlistID, gridID) + 1);
+
+	  /* datatype */
 	  datatype = vlistInqVarDatatype(vlistID, varID);
-
-	  if      ( datatype == DATATYPE_PACK   ) strcpy(pstr, "P0");
-	  else if ( datatype > 0 && datatype <= 32  ) sprintf(pstr, "P%d", datatype);
-	  else if ( datatype == DATATYPE_CPX32  ) strcpy(pstr, "C32");
-	  else if ( datatype == DATATYPE_CPX64  ) strcpy(pstr, "C64");
-	  else if ( datatype == DATATYPE_FLT32  ) strcpy(pstr, "F32");
-	  else if ( datatype == DATATYPE_FLT64  ) strcpy(pstr, "F64");
-	  else if ( datatype == DATATYPE_INT8   ) strcpy(pstr, "I8");
-	  else if ( datatype == DATATYPE_INT16  ) strcpy(pstr, "I16");
-	  else if ( datatype == DATATYPE_INT32  ) strcpy(pstr, "I32");
-	  else if ( datatype == DATATYPE_UINT8  ) strcpy(pstr, "U8");
-	  else if ( datatype == DATATYPE_UINT16 ) strcpy(pstr, "U16");
-	  else if ( datatype == DATATYPE_UINT32 ) strcpy(pstr, "U32");
-	  else                                    strcpy(pstr, "-1");
+	  datatype2str(datatype, pstr);
 
 	  fprintf(stdout, " %-3s", pstr);
 
 	  if ( vlistInqVarCompType(vlistID, varID) == COMPRESS_NONE )
-	    fprintf(stdout, " ");
+	    fprintf(stdout, "  ");
 	  else
-	    fprintf(stdout, "z");
+	    fprintf(stdout, "z ");
 
-	  fprintf(stdout, "%9d", gridsize);
+	  /* parameter info */
+	  fprintf(stdout, ": ");
 
-	  fprintf(stdout, " %3d ", vlistGridIndex(vlistID, gridID) + 1);
+	  cdiParamToString(param, paramstr, sizeof(paramstr));
 
-	  levelsize = zaxisInqSize(zaxisID);
-	  fprintf(stdout, " %6d", levelsize);
-	  fprintf(stdout, " %3d", vlistZaxisIndex(vlistID, zaxisID) + 1);
+	  if ( operfunc == func_name ) vlistInqVarName(vlistID, varID, varname);
+
+	  if ( operfunc == func_name )
+	    fprintf(stdout, "%-11s", varname);
+	  else if ( operfunc == func_code )
+	    fprintf(stdout, "%4d %4d", tabnum, code);
+	  else
+	    fprintf(stdout, "%-11s", paramstr);
 
 	  fprintf(stdout, "\n");
 	}
 
-      fprintf(stdout, "   Horizontal grids :\n");
+      fprintf(stdout, "   Grid coordinates :\n");
       printGridInfo(vlistID);
 
       nzaxis = vlistNzaxis(vlistID);
-      fprintf(stdout, "   Vertical grids :\n");
+      fprintf(stdout, "   Vertical coordinates :\n");
       for ( index = 0; index < nzaxis; index++)
 	{
 	  zaxisID   = vlistZaxis(vlistID, index);
@@ -186,7 +218,7 @@ void *Sinfo(void *argument)
 	  nbyte = nbyte0;
 	  for ( levelID = 0; levelID < levelsize; levelID++ )
 	    {
-	      if ( nbyte > 80 )
+	      if ( nbyte > MAXCHARS )
 		{
 		  fprintf(stdout, "\n");
 		  fprintf(stdout, "%*s", nbyte0, "");
@@ -203,7 +235,7 @@ void *Sinfo(void *argument)
 	      nbyte0 = fprintf(stdout, "%33s : ", "bounds");
 	      for ( levelID = 0; levelID < levelsize; levelID++ )
 		{
-		  if ( nbyte > 80 )
+		  if ( nbyte > MAXCHARS )
 		    {
 		      fprintf(stdout, "\n");
 		      fprintf(stdout, "%*s", nbyte0, "");
@@ -223,13 +255,13 @@ void *Sinfo(void *argument)
       if ( ntsteps != 0 )
 	{
 	  if ( ntsteps == CDI_UNDEFID )
-	    fprintf(stdout, "   Time axis :  unlimited steps\n");
+	    fprintf(stdout, "   Time coordinate :  unlimited steps\n");
 	  else
-	    fprintf(stdout, "   Time axis :  %d step%s\n", ntsteps, ntsteps == 1 ? "" : "s");
+	    fprintf(stdout, "   Time coordinate :  %d step%s\n", ntsteps, ntsteps == 1 ? "" : "s");
 
 	  if ( taxisID != CDI_UNDEFID )
 	    {
-	      int calendar, unit;
+	      int calendar, tunits;
 
 	      if ( taxisInqType(taxisID) == TAXIS_RELATIVE )
 		{
@@ -241,26 +273,26 @@ void *Sinfo(void *argument)
 
 		  fprintf(stdout, "     RefTime = %s %s", vdatestr, vtimestr);
 		      
-		  unit = taxisInqTunit(taxisID);
-		  if ( unit != CDI_UNDEFID )
+		  tunits = taxisInqTunit(taxisID);
+		  if ( tunits != CDI_UNDEFID )
 		    {
-		      if ( unit == TUNIT_YEAR )
+		      if ( tunits == TUNIT_YEAR )
 			fprintf(stdout, "  Units = years");
-		      else if ( unit == TUNIT_MONTH )
+		      else if ( tunits == TUNIT_MONTH )
 			fprintf(stdout, "  Units = months");
-		      else if ( unit == TUNIT_DAY )
+		      else if ( tunits == TUNIT_DAY )
 			fprintf(stdout, "  Units = days");
-		      else if ( unit == TUNIT_12HOURS )
+		      else if ( tunits == TUNIT_12HOURS )
 			fprintf(stdout, "  Units = 12hours");
-		      else if ( unit == TUNIT_6HOURS )
+		      else if ( tunits == TUNIT_6HOURS )
 			fprintf(stdout, "  Units = 6hours");
-		      else if ( unit == TUNIT_3HOURS )
+		      else if ( tunits == TUNIT_3HOURS )
 			fprintf(stdout, "  Units = 3hours");
-		      else if ( unit == TUNIT_HOUR )
+		      else if ( tunits == TUNIT_HOUR )
 			fprintf(stdout, "  Units = hours");
-		      else if ( unit == TUNIT_MINUTE )
+		      else if ( tunits == TUNIT_MINUTE )
 			fprintf(stdout, "  Units = minutes");
-		      else if ( unit == TUNIT_SECOND )
+		      else if ( tunits == TUNIT_SECOND )
 			fprintf(stdout, "  Units = seconds");
 		      else
 			fprintf(stdout, "  Units = unknown");
