@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -190,7 +190,7 @@ void *Sorttimestamp(void *argument)
 		  char vdatestr[32], vtimestr[32];
 		  date2str(vdate[xtsID], vdatestr, sizeof(vdatestr));
 		  time2str(vtime[xtsID], vtimestr, sizeof(vtimestr));
-		  cdoPrint("Timestep %4d %s %s already exist, skipped!", xtsID, vdatestr, vtimestr);
+		  cdoPrint("Timestep %4d %s %s already exists, skipped!", xtsID, vdatestr, vtimestr);
 		}
 	      continue;
 	    }
