@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -34,11 +34,19 @@
 #include "pstream.h"
 
 
+static
+void gen_filename(char *filename, int swap_obase, const char *obase, const char *suffix)
+{
+  if ( swap_obase ) strcat(filename, obase);
+  if ( suffix[0] ) strcat(filename, suffix);
+}
+
+
 void *Split(void *argument)
 {
   int SPLITCODE, SPLITPARAM, SPLITNAME, SPLITLEVEL, SPLITGRID, SPLITZAXIS, SPLITTABNUM;
   int operatorID;
-  int nchars;
+  int nchars = 0;
   int streamID1;
   int varID;
   int code, tabnum, param;
@@ -57,6 +65,7 @@ void *Split(void *argument)
   int lcopy = FALSE;
   int gridsize;
   int nmiss;
+  int swap_obase = FALSE;
   double *array = NULL;
 
   cdoInitialize(argument);
@@ -71,6 +80,11 @@ void *Split(void *argument)
 
   operatorID = cdoOperatorID();
 
+  if ( operatorArgc() == 1 )
+    if ( memcmp("swap", operatorArgv()[0], 4) == 0 ) swap_obase = TRUE;
+
+  if ( operatorArgc() > 1 ) cdoAbort("Too many arguments!");
+
   if ( UNCHANGED_RECORD ) lcopy = TRUE;
 
   streamID1 = streamOpenRead(cdoStreamName(0));
@@ -81,8 +95,11 @@ void *Split(void *argument)
   nrecs  = vlistNrecs(vlistID1);
   nzaxis = vlistNzaxis(vlistID1);
 
-  strcpy(filename, cdoStreamName(1));
-  nchars = strlen(filename);
+  if ( swap_obase == 0 )
+    {
+      strcpy(filename, cdoStreamName(1));
+      nchars = strlen(filename);
+    }
 
   filesuffix[0] = 0;
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoDefaultFileType, vlistID1);
@@ -126,6 +143,7 @@ void *Split(void *argument)
 		    }
 		}
 	    }
+
 	  vlistID2 = vlistCreate();
 	  vlistCopyFlag(vlistID2, vlistID1);
 	  vlistIDs[index] = vlistID2;
@@ -133,20 +151,17 @@ void *Split(void *argument)
 	  if ( codes[index] > 9999 )
 	    {
 	      sprintf(filename+nchars, "%05d", codes[index]);
-	      if ( filesuffix[0] )
-		sprintf(filename+nchars+5, "%s", filesuffix);
+	      gen_filename(filename, swap_obase, cdoStreamName(1), filesuffix);
 	    }
 	  else if ( codes[index] > 999 )
 	    {
 	      sprintf(filename+nchars, "%04d", codes[index]);
-	      if ( filesuffix[0] )
-		sprintf(filename+nchars+4, "%s", filesuffix);
+	      gen_filename(filename, swap_obase, cdoStreamName(1), filesuffix);
 	    }
 	  else
 	    {
 	      sprintf(filename+nchars, "%03d", codes[index]);
-	      if ( filesuffix[0] )
-		sprintf(filename+nchars+3, "%s", filesuffix);
+	      gen_filename(filename, swap_obase, cdoStreamName(1), filesuffix);
 	    }
 
 	  streamIDs[index] = streamOpenWrite(filename, cdoFiletype());
@@ -204,8 +219,8 @@ void *Split(void *argument)
 
 	  filename[nchars] = '\0';
 	  strcat(filename, paramstr);
-	  if ( filesuffix[0] )
-	    strcat(filename, filesuffix);
+	  gen_filename(filename, swap_obase, cdoStreamName(1), filesuffix);
+
 	  streamIDs[index] = streamOpenWrite(filename, cdoFiletype());
 
 	  streamDefVlist(streamIDs[index], vlistIDs[index]);
@@ -256,8 +271,8 @@ void *Split(void *argument)
 	  vlistIDs[index] = vlistID2;
 
 	  sprintf(filename+nchars, "%03d", tabnums[index]);
-	  if ( filesuffix[0] )
-	    sprintf(filename+nchars+3, "%s", filesuffix);
+	  gen_filename(filename, swap_obase, cdoStreamName(1), filesuffix);
+
 	  streamIDs[index] = streamOpenWrite(filename, cdoFiletype());
 
 	  streamDefVlist(streamIDs[index], vlistIDs[index]);
@@ -291,8 +306,8 @@ void *Split(void *argument)
 	  filename[nchars] = '\0';
 	  vlistInqVarName(vlistID1, varID, varname);
 	  strcat(filename, varname);
-	  if ( filesuffix[0] )
-	    strcat(filename, filesuffix);
+	  gen_filename(filename, swap_obase, cdoStreamName(1), filesuffix);
+
 	  streamIDs[index] = streamOpenWrite(filename, cdoFiletype());
 
 	  streamDefVlist(streamIDs[index], vlistID2);
@@ -344,8 +359,8 @@ void *Split(void *argument)
 	  vlistIDs[index] = vlistID2;
 
 	  sprintf(filename+nchars, "%06g", levels[index]);
-	  if ( filesuffix[0] )
-	    sprintf(filename+nchars+6, "%s", filesuffix);
+	  gen_filename(filename, swap_obase, cdoStreamName(1), filesuffix);
+   
 	  streamIDs[index] = streamOpenWrite(filename, cdoFiletype());
 
 	  streamDefVlist(streamIDs[index], vlistID2);
@@ -386,9 +401,9 @@ void *Split(void *argument)
 	  vlistCopyFlag(vlistID2, vlistID1);
 	  vlistIDs[index] = vlistID2;
 
-	  sprintf(filename+nchars, "%02d", gridIDs[index]+1);
-	  if ( filesuffix[0] )
-	    sprintf(filename+nchars+2, "%s", filesuffix);
+	  sprintf(filename+nchars, "%02d", vlistGridIndex(vlistID1, gridIDs[index])+1);
+	  gen_filename(filename, swap_obase, cdoStreamName(1), filesuffix);
+
 	  streamIDs[index] = streamOpenWrite(filename, cdoFiletype());
 
 	  streamDefVlist(streamIDs[index], vlistID2);
@@ -428,9 +443,9 @@ void *Split(void *argument)
 	  vlistCopyFlag(vlistID2, vlistID1);
 	  vlistIDs[index] = vlistID2;
 
-	  sprintf(filename+nchars, "%02d", zaxisIDs[index]+1);
-	  if ( filesuffix[0] )
-	    sprintf(filename+nchars+2, "%s", filesuffix);
+	  sprintf(filename+nchars, "%02d", vlistZaxisIndex(vlistID1, zaxisIDs[index])+1);
+	  gen_filename(filename, swap_obase, cdoStreamName(1), filesuffix);
+
 	  streamIDs[index] = streamOpenWrite(filename, cdoFiletype());
 
 	  streamDefVlist(streamIDs[index], vlistID2);
