@@ -133,14 +133,14 @@ void timpctl(int operatorID)
       
       nrecs = streamInqTimestep(streamID2, otsID);
       if ( nrecs != streamInqTimestep(streamID3, otsID) )
-        cdoAbort("Number of records in time step %d of %s and %s are different!", otsID+1, cdoStreamName(1), cdoStreamName(2));
+        cdoAbort("Number of records at time step %d of %s and %s differ!", otsID+1, cdoStreamName(1), cdoStreamName(2));
 
       vdate2 = taxisInqVdate(taxisID2);
       vtime2 = taxisInqVtime(taxisID2);
       vdate3 = taxisInqVdate(taxisID3);
       vtime3 = taxisInqVtime(taxisID3);
       if ( vdate2 != vdate3 || vtime2 != vtime3 )
-        cdoAbort("Verification dates for time step %d of %s and %s are different!", otsID+1, cdoStreamName(1), cdoStreamName(2));
+        cdoAbort("Verification dates at time step %d of %s and %s differ!", otsID+1, cdoStreamName(1), cdoStreamName(2));
       
       for ( recID = 0; recID < nrecs; recID++ )
         {
@@ -193,13 +193,13 @@ void timpctl(int operatorID)
       if ( nrecs == 0 && nsets == 0 ) break;
 
       if ( vdate2 != vdate4 )
-        cdoAbort("Verification dates for time step %d of %s, %s and %s are different!", otsID+1, cdoStreamName(1), cdoStreamName(2), cdoStreamName(3));
+        cdoAbort("Verification dates at time step %d of %s, %s and %s differ!", otsID+1, cdoStreamName(1), cdoStreamName(2), cdoStreamName(3));
       if ( vtime2 != vtime4 )
-        cdoAbort("Verification times for time step %d of %s, %s and %s are different!", otsID+1, cdoStreamName(1), cdoStreamName(2), cdoStreamName(3));
+        cdoAbort("Verification times at time step %d of %s, %s and %s differ!", otsID+1, cdoStreamName(1), cdoStreamName(2), cdoStreamName(3));
       
       for ( varID = 0; varID < nvars; varID++ )
 	{
-	  if ( vlistInqVarTime(vlistID1, varID) == TIME_CONSTANT ) continue;
+	  if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
 	  nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	  
 	  for ( levelID = 0; levelID < nlevels; levelID++ )
@@ -215,7 +215,7 @@ void timpctl(int operatorID)
 	  varID   = recVarID[recID];
 	  levelID = recLevelID[recID];
 
-	  if ( otsID && vlistInqVarTime(vlistID1, varID) == TIME_CONSTANT ) continue;
+	  if ( otsID && vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
 
 	  streamDefRecord(streamID4, varID, levelID);
 	  streamWriteRecord(streamID4, vars1[varID][levelID].ptr, vars1[varID][levelID].nmiss);
