@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -333,7 +333,7 @@ void *Wind(void *argument)
 	  if ( (varID1 != -1 && varID2 != -1) && (varID == varID1 || varID == varID2) )
 	    {
 	      streamReadRecord(streamID1, array1, &nmiss);
-	      if ( nmiss ) cdoAbort("missing values unsupported for spectral data!");
+	      if ( nmiss ) cdoAbort("Missing values unsupported for spectral data!");
 
 	      gridsize = gridInqSize(gridID1);
 	      offset = gridsize*levelID;
