@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.1
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 
       Zonstat    zonmin          Zonal minimum
       Zonstat    zonmax          Zonal maximum
+      Zonstat    zonrange        Zonal range
       Zonstat    zonsum          Zonal sum
       Zonstat    zonmean         Zonal mean
       Zonstat    zonavg          Zonal average
@@ -42,7 +43,8 @@ void *Zonstat(void *argument)
   int operfunc;
   int streamID1, streamID2;
   int vlistID1, vlistID2;
-  int gridID1, gridID2 = -1;
+  int gridID1 = -1, gridID2 = -1;
+  int zongridID = -1;
   int nlatmax;
   int index, ngrids;
   int recID, nrecs;
@@ -57,15 +59,16 @@ void *Zonstat(void *argument)
 
   cdoInitialize(argument);
 
-  cdoOperatorAdd("zonmin",  func_min,  0, NULL);
-  cdoOperatorAdd("zonmax",  func_max,  0, NULL);
-  cdoOperatorAdd("zonsum",  func_sum,  0, NULL);
-  cdoOperatorAdd("zonmean", func_mean, 0, NULL);
-  cdoOperatorAdd("zonavg",  func_avg,  0, NULL);
-  cdoOperatorAdd("zonvar",  func_var,  0, NULL);
-  cdoOperatorAdd("zonstd",  func_std,  0, NULL);
+  cdoOperatorAdd("zonmin",   func_min,   0, NULL);
+  cdoOperatorAdd("zonmax",   func_max,   0, NULL);
+  cdoOperatorAdd("zonrange", func_range, 0, NULL);
+  cdoOperatorAdd("zonsum",   func_sum,   0, NULL);
+  cdoOperatorAdd("zonmean",  func_mean,  0, NULL);
+  cdoOperatorAdd("zonavg",   func_avg,   0, NULL);
+  cdoOperatorAdd("zonvar",   func_var,   0, NULL);
+  cdoOperatorAdd("zonstd",   func_std,   0, NULL);
   /* RQ */
-  cdoOperatorAdd("zonpctl", func_pctl, 0, NULL);
+  cdoOperatorAdd("zonpctl",  func_pctl,  0, NULL);
   /* QR */
 
   operatorID = cdoOperatorID();
@@ -92,21 +95,34 @@ void *Zonstat(void *argument)
   vlistDefTaxis(vlistID2, taxisID2);
 
   ngrids = vlistNgrids(vlistID1);
+  for ( index = 0; index < ngrids; index++ )
+    {
+      if ( gridInqXsize(vlistGrid(vlistID1, index)) > 1 ) 
+	{
+	  if ( gridID1 == -1 ) gridID1 = vlistGrid(vlistID1, index);
+	}
+      else
+	{
+	  if ( zongridID == -1 ) zongridID = vlistGrid(vlistID1, index);
+	}
+    }
+
   ndiffgrids = 0;
-  for ( index = 1; index < ngrids; index++ )
-    if ( vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index))
-      ndiffgrids++;
-
+  for ( index = 0; index < ngrids; index++ )
+    {
+      if ( zongridID != -1 && zongridID == vlistGrid(vlistID1, index) ) continue;
+      if ( gridID1 != vlistGrid(vlistID1, index) ) ndiffgrids++;
+    }
   if ( ndiffgrids > 0 ) cdoAbort("Too many different grids!");
 
-  index = 0;
-  gridID1 = vlistGrid(vlistID1, index);
-
   if ( gridInqType(gridID1) == GRID_LONLAT   ||
        gridInqType(gridID1) == GRID_GAUSSIAN ||
        gridInqType(gridID1) == GRID_GENERIC )
     {
-      gridID2 = gridToZonal(gridID1);
+      if ( zongridID != -1 && gridInqYsize(zongridID) == gridInqYsize(gridID1) )
+	gridID2 = zongridID;
+      else
+	gridID2 = gridToZonal(gridID1);
     }
   else
     {
@@ -144,12 +160,18 @@ void *Zonstat(void *argument)
 	  field1.missval = vlistInqVarMissval(vlistID1, varID);
 	  field2.missval = vlistInqVarMissval(vlistID1, varID);
 
-	  /* RQ */
-	  if ( operfunc == func_pctl )
-	    zonpctl(field1, & field2, pn);
-	  else  
-	    zonfun(field1, &field2, operfunc);
-	  /* QR */  
+	  if ( zongridID != -1 && zongridID == field1.grid )
+	    {
+	      memcpy(field2.ptr, field1.ptr, nlatmax*sizeof(double));
+	      field2.nmiss = field1.nmiss;
+	    }
+	  else
+	    {
+	      if ( operfunc == func_pctl )
+		zonpctl(field1, & field2, pn);
+	      else  
+		zonfun(field1, &field2, operfunc);
+	    }
 
 	  streamDefRecord(streamID2, varID,  levelID);
 	  streamWriteRecord(streamID2, field2.ptr, field2.nmiss);
