@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -74,12 +74,14 @@ int cdoDefaultTimeType   = CDI_UNDEFID;
 int cdoDefaultByteorder  = CDI_UNDEFID;
 int cdoDefaultTableID    = CDI_UNDEFID;
 
+int cdoLockIO            = FALSE;
 int cdoCheckDatarange    = FALSE;
 
 int cdoDiag              = FALSE;
 int cdoDisableHistory    = FALSE;
 int cdoCompType          = COMPRESS_NONE;  // compression type
 int cdoCompLevel         = 0;              // compression level
+int cdoChunkType         = CDI_UNDEFID;
 int cdoLogOff            = FALSE;
 int cdoSilentMode        = FALSE;
 int cdoOverwriteMode     = FALSE;
@@ -92,6 +94,10 @@ int cdoInteractive       = FALSE;
 int cdoParIO             = FALSE;
 int cdoRegulargrid       = FALSE;
 
+#define MAX_NUM_VARNAMES 256
+int cdoNumVarnames       = 0;
+char **cdoVarnames       = NULL;
+
 char cdo_file_suffix[32];
 
 int cdoExpMode           = -1;
@@ -100,7 +106,6 @@ void exp_run(int argc, char *argv[], char *cdoExpName);
 
 
 int timer_total, timer_read, timer_write;
-int timer_remap, timer_remap_sort, timer_remap_con, timer_remap_con2, timer_remap_con3;
 
 
 #define PRINT_RLIMIT(resource) \
@@ -156,9 +161,18 @@ void cdo_version(void)
 #if defined (HAVE_LIBJASPER)
   fprintf(stderr, " JASPER");
 #endif
+#if defined (HAVE_LIBUDUNITS2)
+  fprintf(stderr, " UDUNITS2");
+#endif
 #if defined (HAVE_LIBPROJ)
   fprintf(stderr, " PROJ.4");
 #endif
+#if defined (HAVE_LIBXML2)
+  fprintf(stderr, " XML2");
+#endif
+#if defined (HAVE_LIBMAGICS)
+  fprintf(stderr, " MAGICS");
+#endif
 #if defined (HAVE_LIBDRMAA)
   fprintf(stderr, " DRMAA");
 #endif
@@ -202,6 +216,8 @@ void usage(void)
   fprintf(stderr, "\n");
   */
   /* fprintf(stderr, "    -l <level>     Level file\n"); */
+  fprintf(stderr, "    -k <chunktype> Chunk type: auto, grid or lines\n");
+  fprintf(stderr, "    -L             Lock IO\n");
   fprintf(stderr, "    -M             Switch to indicate that the I/O streams have missing values\n");
   fprintf(stderr, "    -m <missval>   Set the default missing value (default: %g)\n", cdiInqMissval());
   fprintf(stderr, "    -O             Overwrite existing output file, if checked\n");
@@ -225,7 +241,7 @@ void usage(void)
   fprintf(stderr, "    -v             Print extra details for some operators\n");
   fprintf(stderr, "    -z szip        SZIP compression of GRIB1 records\n");
   fprintf(stderr, "       jpeg        JPEG compression of GRIB2 records\n");
-  fprintf(stderr, "        zip        Deflate compression of netCDF4 variables\n");
+  fprintf(stderr, "        zip[_1-9]  Deflate compression of netCDF4 variables\n");
   fprintf(stderr, "\n");
 
   fprintf(stderr, "  Operators:\n");
@@ -242,7 +258,7 @@ static
 void cdoPrintHelp(char *phelp[]/*, char *xoperator*/)
 {
   if ( phelp == NULL )
-    printf("No help available for this operator!\n");
+    fprintf(stderr, "No help available for this operator!\n");
   else
     {
       int lprint;
@@ -253,7 +269,7 @@ void cdoPrintHelp(char *phelp[]/*, char *xoperator*/)
 	    if ( *(phelp+1) )
 	      if ( *(phelp+1)[0] == ' ' ) lprint = FALSE;
 	  
-	  if ( lprint ) printf("%s\n", *phelp);
+	  if ( lprint ) fprintf(stdout, "%s\n", *phelp);
 
 	  phelp++;
 	}
@@ -575,6 +591,44 @@ void setDefaultFileType(char *filetypestr, int labort)
     }
 }
 
+#if defined (malloc)
+#undef malloc
+#undef free
+#endif
+
+#define NTESTS 11
+static
+int getMemAlignment(void)
+{
+  int ma = -1;
+  int i, k;
+  double *ptr[NTESTS];
+  int64_t iptr;
+  size_t tsize[NTESTS] = {1, 3, 5, 9, 17, 33, 69, 121, 251, 510, 1025};
+  size_t ma_check[4] = {8, 16, 32, 64};
+  int ma_result[4] = {1, 1, 1, 1};
+
+  for ( i = 0; i < NTESTS; ++i )
+    {
+      ptr[i] = malloc(tsize[i]);
+      iptr = (int64_t) ptr[i];
+      for ( k = 0; k < 4; ++k ) if ( iptr%ma_check[k] ) ma_result[k] = 0; 
+    }
+  for ( i = 0; i < NTESTS; ++i ) free(ptr[i]);
+
+  for ( i = NTESTS-1; i >= 0; i-- )
+    {
+      ptr[i] = malloc(tsize[i]+5);
+      iptr = (int64_t) ptr[i];
+      for ( k = 0; k < 4; ++k ) if ( iptr%ma_check[k] ) ma_result[k] = 0; 
+    }
+  for ( i = 0; i < NTESTS; ++i ) free(ptr[i]);
+
+  for ( k = 0; k < 4; ++k ) if ( ma_result[k] ) ma = ma_check[k];
+
+  return (ma);
+}
+
 
 int cdoFiletype(void)
 {
@@ -608,13 +662,71 @@ void defineCompress(const char *arg)
       cdoCompType  = COMPRESS_GZIP;
       cdoCompLevel = 6;
     }
-  else if ( memcmp(arg, "zip", len) == 0 )
+  else if ( memcmp(arg, "zip", 3) == 0 )
     {
       cdoCompType  = COMPRESS_ZIP;
-      cdoCompLevel = 1;
+      if ( len == 5 && arg[3] == '_' && isdigit(arg[4]) )
+	cdoCompLevel = atoi(&arg[4]);
+      else
+	cdoCompLevel = 1;
     }
   else
-    fprintf(stderr, "%s compression unsupported!\n", arg);
+    {
+      fprintf(stderr, "Compression type '%s' unsupported!\n", arg);
+      exit(EXIT_FAILURE);
+    }
+}
+
+static
+void defineChunktype(const char *arg)
+{
+  if      ( strcmp("auto",  arg)   == 0 ) cdoChunkType = CHUNK_AUTO;
+  else if ( strcmp("grid",  arg)   == 0 ) cdoChunkType = CHUNK_GRID;
+  else if ( strcmp("lines", arg)   == 0 ) cdoChunkType = CHUNK_LINES;
+  else
+    {
+      fprintf(stderr, "Chunk type '%s' unsupported!\n", arg);
+      exit(EXIT_FAILURE);
+    }
+}
+
+static
+void defineVarnames(const char *arg)
+{
+  size_t len = strlen(arg);
+  size_t istart = 0;
+  char *pbuf;
+
+  while ( istart < len && (arg[istart] == ' ' || arg[istart] == ',') ) istart++;
+
+  len -= istart;
+
+  if ( len )
+    {
+      char *commapos;
+      
+      cdoVarnames = (char **) malloc(MAX_NUM_VARNAMES*sizeof(char *));
+
+      pbuf = strdup(arg+istart);
+      cdoVarnames[cdoNumVarnames++] = pbuf;    
+
+      commapos = pbuf;
+      while ( (commapos = strchr(commapos, ',')) != NULL )
+	{
+	  *commapos++ = '\0';
+	  if ( strlen(commapos) )
+	    {
+	      if ( cdoNumVarnames >= MAX_NUM_VARNAMES )
+		cdoAbort("Too many variable names (limit=%d)!", MAX_NUM_VARNAMES);
+
+	      cdoVarnames[cdoNumVarnames++] = commapos;
+	    }
+	}
+      /*
+      for ( int i = 0; i < cdoNumVarnames; ++i )
+	printf("varname %d: %s\n", i+1, cdoVarnames[i]);
+      */
+    }
 }
 
 static
@@ -715,7 +827,7 @@ int main(int argc, char *argv[])
 
   if ( noff ) setDefaultFileType(Progname+noff, 0);
 
-  while ( (c = cdoGetopt(argc, argv, "f:b:e:P:p:g:i:l:m:t:D:z:aBcdhMOQRrsSTuVvXZ")) != -1 )
+  while ( (c = cdoGetopt(argc, argv, "f:b:e:P:p:g:i:k:l:m:n:t:D:z:aBcdhLMOQRrsSTuVvXZ")) != -1 )
     {
       switch (c)
 	{
@@ -767,6 +879,12 @@ int main(int argc, char *argv[])
 	case 'i':
 	  defineInstitution(cdoOptarg);
 	  break;
+	case 'k':
+	  defineChunktype(cdoOptarg);
+	  break;
+	case 'L':	
+	  cdoLockIO = TRUE;
+	  break;
 	case 'l':
 	  defineZaxis(cdoOptarg);
 	  break;
@@ -776,6 +894,9 @@ int main(int argc, char *argv[])
 	case 'M':
 	  cdiDefGlobal("HAVE_MISSVAL", TRUE);
 	  break;
+	case 'n':
+	  defineVarnames(cdoOptarg);
+	  break;
 	case 'O':
 	  cdoOverwriteMode = TRUE;
 	  break;
@@ -865,6 +986,25 @@ int main(int argc, char *argv[])
       if ( envstr ) fprintf(stderr, "MACHTYPE            = %s\n", envstr);
       fprintf(stderr, "\n");
 
+#if defined (__SSE2__)
+      fprintf(stderr, "Predefined: __SSE2__\n");
+#endif 
+#if defined (__SSE3__)
+      fprintf(stderr, "Predefined: __SSE3__\n");
+#endif 
+#if defined (__SSE4_1__)
+      fprintf(stderr, "Predefined: __SSE4_1__\n");
+#endif 
+#if defined (__SSE4_2__)
+      fprintf(stderr, "Predefined: __SSE4_2__\n");
+#endif 
+#if defined (__AVX__)
+      fprintf(stderr, "Predefined: __AVX__\n");
+#endif 
+      fprintf(stderr, "\n");
+
+      fprintf(stderr, "mem alignment       = %d\n\n", getMemAlignment());
+
 #if defined (HAVE_MMAP)
       fprintf(stderr, "HAVE_MMAP\n");
 #endif
@@ -873,6 +1013,18 @@ int main(int argc, char *argv[])
 #endif
       fprintf(stderr, "\n");
 
+#if defined (_OPENACC)
+      fprintf(stderr, "OPENACC VERSION     = %d\n", _OPENACC);
+#endif
+#if defined (_OPENMP)
+      fprintf(stderr, "OPENMP VERSION      = %d\n", _OPENMP);
+#endif
+#if defined (__GNUC__)
+      fprintf(stderr, "GNUC VERSION        = %d\n", __GNUC__);
+#endif
+#if defined (__ICC)
+      fprintf(stderr, "ICC VERSION         = %d\n", __ICC);
+#endif
 #if defined (__STDC__)
       fprintf(stderr, "STD ANSI C          = %d\n", __STDC__);
 #endif
@@ -950,11 +1102,11 @@ int main(int argc, char *argv[])
 	      {
 		if ( status == 0 )
 		  {
-		    fprintf(stderr, "Set stack size to %ld\n", MIN_STACK_SIZE);
+		    fprintf(stderr, "Set stack size to %ld\n", (long) min_stack_size);
 		    PRINT_RLIMIT(RLIMIT_STACK);
 		  }
 		else
-		  fprintf(stderr, "Set stack size to %ld failed!\n", MIN_STACK_SIZE);
+		  fprintf(stderr, "Set stack size to %ld failed!\n", (long) min_stack_size);
 	      }
 	  }
       }
@@ -973,9 +1125,10 @@ int main(int argc, char *argv[])
   ompNumThreads = omp_get_max_threads();
   if ( omp_get_max_threads() > omp_get_num_procs() )
     fprintf(stderr, "Warning: Number of OMP threads is greater than number of CPUs=%d!\n", omp_get_num_procs());
+  if ( ompNumThreads < numThreads )
+    fprintf(stderr, "Warning: omp_get_max_threads() returns %d!\n", ompNumThreads);
   if ( cdoVerbose )
-    fprintf(stderr, " OpenMP:  num_procs = %d  max_threads = %d\n",
-	    omp_get_num_procs(), omp_get_max_threads());
+    fprintf(stderr, " OpenMP:  num_procs = %d  max_threads = %d\n", omp_get_num_procs(), omp_get_max_threads());
 #else
   if ( numThreads > 0 )
     {
@@ -1011,7 +1164,7 @@ int main(int argc, char *argv[])
 
   if ( Help )
     {
-      cdoPrintHelp(operatorHelp(operatorName)/*, operatorName*/);
+      cdoPrintHelp(operatorHelp(operatorName));
     }
   else if ( cdoExpMode == CDO_EXP_LOCAL )
     {
@@ -1019,28 +1172,26 @@ int main(int argc, char *argv[])
     }
   else
     {
-      if ( cdoTimer )
-	{
-	  timer_total      = timer_new("total");
-	  timer_read       = timer_new("read");
-	  timer_write      = timer_new("write");
-	  timer_remap      = timer_new("remap");
-	  timer_remap_sort = timer_new("remap sort");
-	  timer_remap_con  = timer_new("remap con");
-	  timer_remap_con2 = timer_new("remap con2");
-	  timer_remap_con3 = timer_new("remap con3");
-	}
+      timer_total      = timer_new("total");
+      timer_read       = timer_new("read");
+      timer_write      = timer_new("write");
 
-      if ( cdoTimer ) timer_start(timer_total);
+      timer_start(timer_total);
 
       operatorModule(operatorName)(argument);
 
-      if ( cdoTimer ) timer_stop(timer_total);
+      timer_stop(timer_total);
 
       if ( cdoTimer ) timer_report();
     }
 
   if ( argument ) free(argument);
+
+  if ( cdoVarnames )
+    {
+      if ( cdoNumVarnames ) free(cdoVarnames[0]);
+      free(cdoVarnames);
+    }
   /* problems with alias!!! if ( operatorName ) free(operatorName); */ 
 
   /* malloc_stats(); */
