@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,8 @@
 #define  MAX(a,b)  ((a) > (b) ? (a) : (b))
 #undef   NINT
 #define  NINT(x)   ((x) < 0 ? (int)((x)-0.5) : (int)((x)+0.5))
+#undef   NINTD
+#define  NINTD(x)   ((x) < 0 ? ((x)-0.5) : ((x)+0.5))
 
 #define  UNCHANGED_RECORD  (processSelf() == 0 && *cdoStreamName(0) != '-' && cdoRegulargrid == FALSE && cdoDefaultDataType == -1 && cdoDefaultByteorder == -1 )
 
@@ -51,6 +53,7 @@ extern int cdoDefaultByteorder;
 extern int cdoDefaultTableID;
 extern int cdoDefaultInstID;
 
+extern int cdoLockIO;
 extern int cdoCheckDatarange;
 
 extern int cdoSilentMode;
@@ -67,19 +70,25 @@ extern int cdoParIO;
 extern int cdoCompType;
 extern int cdoCompLevel;
 
+extern int cdoChunkType;
+
 extern int cdoExpMode;
 
 extern int cdoDiag;
 
+extern int cdoNumVarnames;
+extern char **cdoVarnames;
+
 void    cdiError(int cdiErrno, const char *fmt, ...);
 void    cdoAbort(const char *fmt, ...);
 void    cdoWarning(const char *fmt, ...);
 void    cdoPrint(const char *fmt, ...);
 
-int  timer_new(char *text);
+int  timer_new(const char *text);
 void timer_report(void);
 void timer_start(int it);
 void timer_stop(int it);
+double timer_val(int it);
 
 void    operatorInputArg(const char *enter);
 int     operatorArgc(void);
