@@ -22,7 +22,7 @@
 #include <stdarg.h> /* va_list */
 
 #if defined (HAVE_LIBPROJ)
-#  include "projects.h"
+#  include "proj_api.h"
 #endif
 
 #include <cdi.h>
@@ -354,7 +354,7 @@ static
 void sinusoidal_to_geo(int gridsize, double *xvals, double *yvals)
 {
 #if defined (HAVE_LIBPROJ)
-  PJ   *libProj;
+  projPJ   *libProj;
   char *params[20];
   int nbpar=0;
   projUV data, res;
@@ -391,7 +391,7 @@ static
 void laea_to_geo(int gridID, int gridsize, double *xvals, double *yvals)
 {
 #if defined (HAVE_LIBPROJ)
-  PJ   *libProj;
+  projPJ   *libProj;
   char *params[20];
   int nbpar=0;
   projUV data, res;
@@ -402,7 +402,7 @@ void laea_to_geo(int gridID, int gridsize, double *xvals, double *yvals)
 
   nbpar = 0;
   params[nbpar++] = gen_param("proj=laea");
-  params[nbpar++] = gen_param("a=%g", a);
+  if ( a > 0 ) params[nbpar++] = gen_param("a=%g", a);
   params[nbpar++] = gen_param("lon_0=%g", lon_0);
   params[nbpar++] = gen_param("lat_0=%g", lat_0);
 
@@ -435,7 +435,7 @@ static
 void lcc2_to_geo(int gridID, int gridsize, double *xvals, double *yvals)
 {
 #if defined (HAVE_LIBPROJ)
-  PJ   *libProj;
+  projPJ   *libProj;
   char *params[20];
   int nbpar=0;
   projUV data, res;
@@ -614,6 +614,24 @@ int gridToCurvilinear(int gridID1, int lbounds)
 	gridInqXunits(gridID1, xunits);
 	gridInqYunits(gridID1, yunits);
 
+	if ( gridtype == GRID_LAEA )
+	  {
+	    int lvalid_xunits = FALSE;
+	    int lvalid_yunits = FALSE;
+	    int len;
+	    len = (int) strlen(xunits);
+	    if ( len == 1 && memcmp(xunits, "m",  1) == 0 ) lvalid_xunits = TRUE;
+	    if ( len == 1 && memcmp(xunits, "km", 2) == 0 ) lvalid_xunits = TRUE;
+	    len = (int) strlen(yunits);
+	    if ( len == 1 && memcmp(yunits, "m",  1) == 0 ) lvalid_yunits = TRUE;
+	    if ( len == 1 && memcmp(yunits, "km", 2) == 0 ) lvalid_yunits = TRUE;
+
+	    if ( lvalid_xunits == FALSE )
+	      cdoWarning("Possibly wrong result! Invalid x-coordinate units: \"%s\" (expected \"m\" or \"km\")", xunits);
+	    if ( lvalid_yunits == FALSE )
+	      cdoWarning("Possibly wrong result! Invalid y-coordinate units: \"%s\" (expected \"m\" or \"km\")", yunits);
+	  }
+
 	if ( memcmp(xunits, "km", 2) == 0 ) xscale = 1000;
 	if ( memcmp(yunits, "km", 2) == 0 ) yscale = 1000;
 
@@ -676,9 +694,13 @@ int gridToCurvilinear(int gridID1, int lbounds)
 		    /* correct_sinxvals(nx, ny, xvals2D); */
 		  }
 		else if ( gridtype == GRID_LAEA )
-		  laea_to_geo(gridID1, gridsize, xvals2D, yvals2D);
+		  {
+		    laea_to_geo(gridID1, gridsize, xvals2D, yvals2D);
+		  }
 		else if ( gridtype == GRID_LCC2 )
-		  lcc2_to_geo(gridID1, gridsize, xvals2D, yvals2D);
+		  {
+		    lcc2_to_geo(gridID1, gridsize, xvals2D, yvals2D);
+		  }
 	      }
 	  }
 
@@ -1103,9 +1125,6 @@ int gridCurvilinearToRegular(int gridID1)
   xvals = (double *) malloc(nx*sizeof(double));
   yvals = (double *) malloc(ny*sizeof(double));
 
-  gridInqXvals(gridID1, xvals);
-  gridInqYvals(gridID1, yvals);
-
   for ( i = 0; i < nx; i++ ) xvals[i] = xvals2D[i];
   for ( j = 0; j < ny; j++ ) yvals[j] = yvals2D[j*nx];
 
@@ -1305,59 +1324,36 @@ double cell_area(long i, long nv, double *grid_center_lon, double *grid_center_l
   long k;
   double xa;
   double area;
-  double lim = 179*deg2rad;
   struct geo p1, p2, p3;
   struct cart c1, c2, c3;
 
   area = 0;
       
-  p3.lon = grid_center_lon[i]*deg2rad; 
-  p3.lat = grid_center_lat[i]*deg2rad;
+  p3.lon = grid_center_lon[i]; 
+  p3.lat = grid_center_lat[i];
   c3 = gc2cc(&p3);
       
   for ( k = 1; k < nv; ++k )
     {
-      p1.lon = grid_corner_lon[i*nv+k-1]*deg2rad; 
-      p1.lat = grid_corner_lat[i*nv+k-1]*deg2rad;
+      p1.lon = grid_corner_lon[i*nv+k-1]; 
+      p1.lat = grid_corner_lat[i*nv+k-1];
       c1 = gc2cc(&p1);
-      p2.lon = grid_corner_lon[i*nv+k]*deg2rad; 
-      p2.lat = grid_corner_lat[i*nv+k]*deg2rad;
+      p2.lon = grid_corner_lon[i*nv+k]; 
+      p2.lat = grid_corner_lat[i*nv+k];
       c2 = gc2cc(&p2);
 
       xa = areas(&c1, &c2, &c3);
-      /*
-      if ( xa > 0.001 )
-	if ( (fabs(p1.lon - p2.lon) > lim) ||
-	     (fabs(p2.lon - p3.lon) > lim) ||
-	     (fabs(p3.lon - p1.lon) > lim) )
-	  {
-	    printf("%ld %ld xa %g\n", i, k, xa);
-	    printf("  dlon %g %g %g %g\n", lim, fabs(p1.lon - p2.lon), fabs(p2.lon - p3.lon), fabs(p3.lon - p1.lon));
-	    *status = 2;
-	  }
-      */
       area += xa;
     }
 
-  p1.lon = grid_corner_lon[i*nv+0]*deg2rad; 
-  p1.lat = grid_corner_lat[i*nv+0]*deg2rad;
+  p1.lon = grid_corner_lon[i*nv+0]; 
+  p1.lat = grid_corner_lat[i*nv+0];
   c1 = gc2cc(&p1);
-  p2.lon = grid_corner_lon[i*nv+nv-1]*deg2rad; 
-  p2.lat = grid_corner_lat[i*nv+nv-1]*deg2rad;
+  p2.lon = grid_corner_lon[i*nv+nv-1]; 
+  p2.lat = grid_corner_lat[i*nv+nv-1];
   c2 = gc2cc(&p2);
 
   xa = areas(&c1, &c2, &c3);
-  /*
-  if ( xa > 0.001 )
-    if ( (fabs(p1.lon - p2.lon) > lim) ||
-	 (fabs(p2.lon - p3.lon) > lim) ||
-	 (fabs(p3.lon - p1.lon) > lim) )
-      {
-	printf("%ld %ld xa %g\n", i, k, xa);
-	printf("  dlon %g %g %g %g\n", lim, fabs(p1.lon - p2.lon), fabs(p2.lon - p3.lon), fabs(p3.lon - p1.lon));
-	*status = 2;
-      }
-  */
   area += xa;
 
   return (area);
@@ -1467,21 +1463,21 @@ int gridGenArea(int gridID, double *area)
 
     gridInqXunits(gridID, units);
 
-    if ( memcmp(units, "degree", 6) == 0 )
+    if ( memcmp(units, "radian", 6) == 0 )
       {
 	/* No conversion necessary */
       }
-    else if ( memcmp(units, "radian", 6) == 0 )
+    else if ( memcmp(units, "degree", 6) == 0 )
       {
 	for ( i = 0; i < gridsize; ++i )
 	  {
-	    grid_center_lon[i] *= rad2deg;
-	    grid_center_lat[i] *= rad2deg;
+	    grid_center_lon[i] *= deg2rad;
+	    grid_center_lat[i] *= deg2rad;
 	  }
 	for ( i = 0; i < gridsize*nv; ++i )
 	  {
-	    grid_corner_lon[i] *= rad2deg;
-	    grid_corner_lat[i] *= rad2deg;
+	    grid_corner_lon[i] *= deg2rad;
+	    grid_corner_lat[i] *= deg2rad;
 	  }
       }
     else
@@ -1501,6 +1497,13 @@ int gridGenArea(int gridID, double *area)
   for ( i = 0; i < gridsize; ++i )
     {
       area[i] = cell_area(i, nv, grid_center_lon, grid_center_lat, grid_corner_lon, grid_corner_lat, &status);
+      /*
+      if (  area[i] < 0.009 && fabs(grid_corner_lat[i*nv+1]) > 89)
+      printf("%d %g %g %g %g %g %g %g %g %g\n", i, area[i]*6371000*6371000, grid_center_lon[i], grid_center_lat[i],
+	     grid_corner_lon[i*nv], grid_corner_lat[i*nv],
+	     grid_corner_lon[i*nv+1], grid_corner_lat[i*nv+1],
+	     grid_corner_lon[i*nv+2], grid_corner_lat[i*nv+2]);
+      */
       //     total_area += area[i];
     }
 
