@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2011 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -118,7 +118,8 @@ double listGetFlt(LIST *list, int num)
 }
 
 
-static void split_intstring(const char *intstr, int *first, int *last, int *inc)
+static
+void split_intstring(const char *intstr, int *first, int *last, int *inc)
 {
   int i, start;
   int istrlen;
