@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2010 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <glob.h>
 
 #include "cdo.h"
 #include "cdo_int.h"
@@ -35,10 +36,11 @@
 #include "dmemory.h"
 
 
-#define  MAX_PROCESS   128
-#define  MAX_STREAM     64
-#define  MAX_OPERATOR  128
-#define  MAX_ARGC     4096
+#define  MAX_PROCESS    128
+#define  MAX_STREAM      64
+#define  MAX_OPERATOR   128
+#define  MAX_OARGC     4096
+#define  MAX_FILES    65536
 
 
 typedef struct {
@@ -52,6 +54,7 @@ operator_t;
 typedef struct {
 #if  defined  (HAVE_LIBPTHREAD)
   pthread_t threadID;
+  int       l_threadID;
 #endif
   short      nchild;
   short      nstream;
@@ -71,7 +74,7 @@ typedef struct {
   char      *operatorName;
   char      *operatorArg;
   int        oargc;
-  char      *oargv[MAX_ARGC];
+  char      *oargv[MAX_OARGC];
   char       prompt[64];
   short      noper;
   operator_t operator[MAX_OPERATOR];
@@ -82,6 +85,7 @@ process_t;
 static process_t Process[MAX_PROCESS];
 
 static int NumProcess = 0;
+static int NumProcessActive = 0;
 
 #if  defined  (HAVE_LIBPTHREAD)
 pthread_mutex_t processMutex = PTHREAD_MUTEX_INITIALIZER;
@@ -95,7 +99,10 @@ int processCreate(void)
 #if  defined  (HAVE_LIBPTHREAD)
   pthread_mutex_lock(&processMutex);
 #endif
+
   processID = NumProcess++;
+  NumProcessActive++;
+
 #if  defined  (HAVE_LIBPTHREAD)
   pthread_mutex_unlock(&processMutex);  
 #endif
@@ -105,6 +112,7 @@ int processCreate(void)
 
 #if  defined  (HAVE_LIBPTHREAD)
   Process[processID].threadID     = pthread_self();
+  Process[processID].l_threadID   = 1;
 #endif
   Process[processID].nstream      = 0;
   Process[processID].nchild       = 0;
@@ -134,7 +142,8 @@ int processSelf(void)
   pthread_mutex_lock(&processMutex);
 
   for ( processID = 0; processID < NumProcess; processID++ )
-    if ( pthread_equal(Process[processID].threadID, thID) ) break;
+    if ( Process[processID].l_threadID )
+      if ( pthread_equal(Process[processID].threadID, thID) ) break;
 
   if ( processID == NumProcess )
     {
@@ -170,6 +179,24 @@ int processNums(void)
 }
 
 
+int processNumsActive(void)
+{
+  int pnums = 0;
+
+#if  defined  (HAVE_LIBPTHREAD)
+  pthread_mutex_lock(&processMutex);
+#endif
+
+  pnums = NumProcessActive;
+
+#if  defined  (HAVE_LIBPTHREAD)
+  pthread_mutex_unlock(&processMutex);  
+#endif
+
+  return (pnums);
+}
+
+
 void processAddNvals(off_t nvals)
 {
   int processID = processSelf();
@@ -296,6 +323,34 @@ const char *processInqPrompt(void)
   return (Process[processID].prompt);
 }
 
+/* Convert a wildcard pattern into a list of blank-separated
+   filenames which match the wildcard.  */
+char *glob_pattern(const char *restrict wildcard)
+{
+  char *gfilename;
+  size_t cnt, length = 0;
+  glob_t glob_results;
+  char **p;
+
+  glob(wildcard, GLOB_NOCHECK, 0, &glob_results);
+
+  /* How much space do we need?  */
+  for ( p = glob_results.gl_pathv, cnt = glob_results.gl_pathc; cnt; p++, cnt-- )
+    length += strlen(*p) + 1;
+
+  /* Allocate the space and generate the list.  */
+  gfilename = (char *) calloc(length, sizeof(char));
+  for ( p = glob_results.gl_pathv, cnt = glob_results.gl_pathc; cnt; p++, cnt-- )
+    {
+      strcat(gfilename, *p);
+      if ( cnt > 1 ) strcat(gfilename, " ");
+    }
+
+  globfree(&glob_results);
+
+  return gfilename;
+}
+
 
 int cdoStreamCnt(void)
 {
@@ -457,7 +512,7 @@ void setStreamNames(int argc, char *argv[])
   int globArgc = 1;
   int globArgcStart;
   char *streamname;
-  size_t len;
+  int len;
 
   while ( globArgc < argc )
     {
@@ -472,9 +527,9 @@ void setStreamNames(int argc, char *argv[])
 	  for ( i = globArgcStart; i < globArgc; i++ ) len += strlen(argv[i]) + 1;
 	  streamname = (char *) malloc(len);
 	  memcpy(streamname, argv[globArgcStart], len);
-	  for ( i = 1; i < (int) len-1; i++ ) if ( streamname[i] == '\0' ) streamname[i] = ' ';
+	  for ( i = 1; i < len-1; i++ ) if ( streamname[i] == '\0' ) streamname[i] = ' ';
 	  Process[processID].streamNames[Process[processID].streamCnt++] = streamname;
-	  //	  printf("streamname1: %s\n", streamname);
+	  // printf("streamname1: %s\n", streamname);
 	}
       else
 	{
@@ -482,25 +537,95 @@ void setStreamNames(int argc, char *argv[])
 	  streamname = (char *) malloc(len);
 	  strcpy(streamname, argv[globArgc]);
 	  Process[processID].streamNames[Process[processID].streamCnt++] = streamname;
-	  //	  printf("streamname2: %s\n", streamname);
+	  // printf("streamname2: %s\n", streamname);
 	  globArgc++;
 	}
     }
 }
 
 static
-void checkStreamCnt(void)
+int expand_wildcards(int processID, int streamCnt)
+{
+  int i;
+  const char *streamname0 = Process[processID].streamNames[0];
+  int len = strlen(streamname0);
+
+  for ( i = 0; i < len; ++i ) if ( streamname0[i] == '?' || streamname0[i] == '*' ) break;
+  if ( i < len )
+    {
+      char *pattern = glob_pattern(streamname0);
+      if ( strcmp(streamname0, pattern) != 0 )
+	{
+	  int gargc = 0;
+	  char *gargv[MAX_FILES];
+	  int len;
+	  gargv[gargc++] = pattern;
+	  len = strlen(pattern);
+	  for ( i = 1; i < len; ++i )
+	    if ( pattern[i] == ' ' )
+	      {
+		pattern[i] = 0;
+		i++;
+		gargv[gargc++] = &pattern[i];
+		if ( gargc >= MAX_FILES )
+		  Error("Internal problem! More than %d arguments.", MAX_FILES);
+	      }
+
+	  // for ( i = 0; i < gargc; ++i ) printf("pattern %d >%s<\n", i+1, gargv[i]);
+
+	  streamCnt = streamCnt - 1 + gargc;
+	  
+	  /*
+	  for ( i = 0; i < Process[processID].streamCnt; ++i )
+	    printf("stream %d <%s>\n", i+1, Process[processID].streamNames[i]);
+	  */
+	  Process[processID].streamNames = (char **) realloc(Process[processID].streamNames, streamCnt*sizeof(char *));
+	      
+	  // move output streams to the end
+	  for ( i = 1; i < Process[processID].streamCnt; ++i )
+	    Process[processID].streamNames[i+gargc-1] = Process[processID].streamNames[i];
+
+	  free(Process[processID].streamNames[0]);
+
+	  for ( i = 0; i < gargc; ++i )
+	    {
+	      char *streamname;
+	      len = strlen(gargv[i]) + 1;
+	      streamname = (char *) malloc(len);
+	      strcpy(streamname, gargv[i]);
+	      Process[processID].streamNames[i] = streamname;
+	    }
+	  
+	  Process[processID].streamCnt = streamCnt;
+	  /*
+	  for ( i = 0; i < Process[processID].streamCnt; ++i )
+	    printf("stream %d <%s>\n", i+1, Process[processID].streamNames[i]);
+	  */
+	}
+      
+      free(pattern);
+    }
+
+  return 1;
+}
+
+static
+int checkStreamCnt(void)
 {
   int processID = processSelf();
   int streamInCnt, streamOutCnt;
+  int streamInCnt0;
   int streamCnt = 0;
   int i, j;
   int obase = FALSE;
+  int status = 0;
   const char *caller = processInqPrompt();
 
   streamInCnt  = operatorStreamInCnt(Process[processID].operatorName);
   streamOutCnt = operatorStreamOutCnt(Process[processID].operatorName);
 
+  streamInCnt0 = streamInCnt;
+
   if ( streamOutCnt == -1 )
     {
       streamOutCnt = 1;
@@ -510,6 +635,7 @@ void checkStreamCnt(void)
   if ( streamInCnt == -1 && streamOutCnt == -1 )
     Errorc("I/O stream counts unlimited no allowed!");
     
+  // printf(" streamInCnt, streamOutCnt %d %d\n", streamInCnt, streamOutCnt);
   if ( streamInCnt == -1 )
     {
       streamInCnt = Process[processID].streamCnt - streamOutCnt;
@@ -521,8 +647,10 @@ void checkStreamCnt(void)
       streamOutCnt = Process[processID].streamCnt - streamInCnt;
       if ( streamInCnt < 1 ) Errorc("Output streams missing!");
     }
+  // printf(" streamInCnt, streamOutCnt %d %d\n", streamInCnt, streamOutCnt);
 
   streamCnt = streamInCnt + streamOutCnt;
+  // printf(" streamCnt %d %d\n", Process[processID].streamCnt, streamCnt);
 
   if ( Process[processID].streamCnt > streamCnt )
     Errorc("Too many streams!"
@@ -547,10 +675,13 @@ void checkStreamCnt(void)
 	      Errorc("Output file name %s is equal to input file name"
 		     " on position %d!\n", Process[processID].streamNames[i], j+1);
 	}
-    }  
-}
+    }
 
-#define  MAX_ARGV  8192
+  if ( streamInCnt == 1 && streamInCnt0 == -1 )
+    status = expand_wildcards(processID, streamCnt);
+
+  return (status);
+}
 
 static
 void setStreams(const char *argument)
@@ -558,8 +689,9 @@ void setStreams(const char *argument)
   int processID = processSelf();
   int streamCnt;
   int i;
+  int status;
   int argc = 0;
-  char *argv[MAX_ARGV];
+  char *argv[MAX_FILES];
   char *string;
   size_t arglen;
 
@@ -574,8 +706,8 @@ void setStreams(const char *argument)
 	{
 	  string[i] = '\0';
 	  argv[argc++] = &string[i+1];
-	  if ( argc >= MAX_ARGV )
-	    Error("Internal problem! More than %d arguments.", argc);
+	  if ( argc >= MAX_FILES )
+	    Error("Internal problem! More than %d arguments.", MAX_FILES);
 	}
     }
 
@@ -592,10 +724,11 @@ void setStreams(const char *argument)
 
   setStreamNames(argc, argv);
 
-  checkStreamCnt();
+  status = checkStreamCnt();
 
-  if ( Process[processID].streamCnt != streamCnt )
+  if ( status == 0 && Process[processID].streamCnt != streamCnt )
     Error("Internal problem with stream count %d %d", Process[processID].streamCnt, streamCnt);
+
   /*
   for ( i = 0; i < streamCnt; i++ )
     fprintf(stderr, "stream %d %s\n", i+1, Process[processID].streamNames[i]);
@@ -613,7 +746,7 @@ void processDefArgument(const char *argument)
   int oargc = 0;
   char **oargv = Process[processID].oargv;
 
-  /*printf("argument: %s\n", argument);*/
+  /* printf("argument: %s\n", argument); */
   Process[processID].xoperator    = getOperator(argument);
   Process[processID].operatorName = getOperatorName(Process[processID].xoperator);
   Process[processID].operatorArg  = getOperatorArg(Process[processID].xoperator);
@@ -630,8 +763,8 @@ void processDefArgument(const char *argument)
 	  *commapos++ = '\0';
 	  if ( strlen(commapos) )
 	    {
-	      if ( oargc >= MAX_ARGC )
-		cdoAbort("Too many parameter (limit=%d)!", MAX_ARGC);
+	      if ( oargc >= MAX_OARGC )
+		cdoAbort("Too many parameter (limit=%d)!", MAX_OARGC);
 
 	      oargv[oargc++] = commapos;
 	    }
@@ -644,6 +777,7 @@ void processDefArgument(const char *argument)
   setStreams(argument);
 }
 
+
 void processDefVarNum(int nvars, int streamID)
 {
   int processID = processSelf();
@@ -688,7 +822,17 @@ void processDelete(void)
 {
   int processID = processSelf();
 
-  /* printf("delete processID %d\n", processID); */
+  //fprintf(stderr, "delete processID %d\n", processID);
+#if  defined  (HAVE_LIBPTHREAD)
+  pthread_mutex_lock(&processMutex);
+#endif
+
+  Process[processID].l_threadID = 0;
+  NumProcessActive--;
+
+#if  defined  (HAVE_LIBPTHREAD)
+  pthread_mutex_unlock(&processMutex);  
+#endif
 }
 
 
@@ -722,20 +866,17 @@ void operatorCheckArgc(int numargs)
 
 void operatorInputArg(const char *enter)
 {
-  char line[1024];
-  char *pline = line;
   int processID = processSelf();
-  size_t pos, len, linelen;
   int oargc;
-  int lreadline;
 
   oargc = Process[processID].oargc;
 
-  if ( oargc ) return;
-
-  while ( oargc == 0 )
+  if ( oargc == 0 )
     {
-      lreadline = 1;
+      char line[1024];
+      char *pline = line;
+      size_t pos, len, linelen;
+      int lreadline = 1;
 
       if ( enter ) fprintf(stderr, "%-16s : Enter %s > ", processInqPrompt(), enter);
 
@@ -772,9 +913,9 @@ void operatorInputArg(const char *enter)
 		break;
 	    }
 	}
-    }
 
-  Process[processID].oargc = oargc;
+      Process[processID].oargc = oargc;
+    }
 }
 
 
