@@ -65,21 +65,22 @@ static int pthreadScope = 0;
 
 static pthread_mutex_t streamOpenReadMutex  = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t streamOpenWriteMutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t streamMutex          = PTHREAD_MUTEX_INITIALIZER;
 
 static pthread_once_t _pstream_init_thread = PTHREAD_ONCE_INIT;
 static pthread_mutex_t _pstream_mutex;
 
-#  define PSTREAM_LOCK           pthread_mutex_lock(&_pstream_mutex);
-#  define PSTREAM_UNLOCK         pthread_mutex_unlock(&_pstream_mutex);
-#  define PSTREAM_INIT                               \
-   if ( _pstream_init == FALSE ) pthread_once(&_pstream_init_thread, pstream_initialize);
+#  define PSTREAM_LOCK()           pthread_mutex_lock(&_pstream_mutex)
+#  define PSTREAM_UNLOCK()         pthread_mutex_unlock(&_pstream_mutex)
+#  define PSTREAM_INIT()	  \
+   if ( _pstream_init == FALSE ) pthread_once(&_pstream_init_thread, pstream_initialize)
 
 #else
 
-#  define PSTREAM_LOCK
-#  define PSTREAM_UNLOCK
-#  define PSTREAM_INIT                               \
-   if ( _pstream_init == FALSE ) pstream_initialize();
+#  define PSTREAM_LOCK()
+#  define PSTREAM_UNLOCK()
+#  define PSTREAM_INIT()	  \
+   if ( _pstream_init == FALSE ) pstream_initialize()
 
 #endif
 
@@ -131,15 +132,15 @@ pstream_t *pstream_to_pointer(int idx)
 {
   pstream_t *pstreamptr = NULL;
 
-  PSTREAM_INIT
+  PSTREAM_INIT();
 
   if ( idx >= 0 && idx < _pstream_max )
     {
-      PSTREAM_LOCK
+      PSTREAM_LOCK();
 
       pstreamptr = _pstreamList[idx].ptr;
 
-      PSTREAM_UNLOCK
+      PSTREAM_UNLOCK();
     }
   else
     Error("pstream index %d undefined!", idx);
@@ -156,7 +157,7 @@ int pstream_from_pointer(pstream_t *ptr)
 
   if ( ptr )
     {
-      PSTREAM_LOCK
+      PSTREAM_LOCK();
 
       if ( _pstreamAvail )
 	{
@@ -172,7 +173,7 @@ int pstream_from_pointer(pstream_t *ptr)
       else
 	Warning("Too many open pstreams (limit is %d)!", _pstream_max);
 
-      PSTREAM_UNLOCK
+      PSTREAM_UNLOCK();
     }
   else
     Error("Internal problem (pointer %p undefined)", ptr);
@@ -225,7 +226,7 @@ void pstream_delete_entry(pstream_t *pstreamptr)
 
   idx = pstreamptr->self;
 
-  PSTREAM_LOCK
+  PSTREAM_LOCK();
 
   free(pstreamptr);
 
@@ -233,7 +234,7 @@ void pstream_delete_entry(pstream_t *pstreamptr)
   _pstreamList[idx].ptr  = 0;
   _pstreamAvail   	 = &_pstreamList[idx];
 
-  PSTREAM_UNLOCK
+  PSTREAM_UNLOCK();
 
   if ( PSTREAM_Debug )
     Message("Removed idx %d from pstream list", idx);
@@ -306,7 +307,7 @@ int pstreamOpenRead(const char *argument)
   int pstreamID;
   pstream_t *pstreamptr;
 
-  PSTREAM_INIT
+  PSTREAM_INIT();
 
   pstreamptr = pstream_new_entry();
   if ( ! pstreamptr ) Error("No memory");
@@ -323,7 +324,7 @@ int pstreamOpenRead(const char *argument)
       int rval;
       pthread_t thrID;
       pthread_attr_t attr;
-      struct sched_param param;
+      // struct sched_param param;
       size_t len;
       size_t stacksize;
       int status;
@@ -535,7 +536,10 @@ int pstreamOpenRead(const char *argument)
       if ( PSTREAM_Debug ) Message("file %s", filename);
 
 #if  defined  (HAVE_LIBPTHREAD)
-      pthread_mutex_lock(&streamOpenReadMutex);
+      if ( cdoLockIO )
+	pthread_mutex_lock(&streamMutex);
+      else
+	pthread_mutex_lock(&streamOpenReadMutex);
 #endif
       fileID = streamOpenRead(filename);
       if ( fileID < 0 ) cdiError(fileID, "Open failed on >%s<", filename);
@@ -548,7 +552,10 @@ int pstreamOpenRead(const char *argument)
       */
       cdoInqHistory(fileID);
 #if  defined  (HAVE_LIBPTHREAD)
-      pthread_mutex_unlock(&streamOpenReadMutex);
+      if ( cdoLockIO )
+	pthread_mutex_unlock(&streamMutex);
+      else
+	pthread_mutex_unlock(&streamOpenReadMutex);
 #endif
 
       pstreamptr->mode   = 'r';
@@ -627,7 +634,7 @@ int pstreamOpenWrite(const char *argument, int filetype)
   int ispipe;
   pstream_t *pstreamptr;
 
-  PSTREAM_INIT
+  PSTREAM_INIT();
 
   ispipe = memcmp(argument, "(pipe", 5) == 0;
 
@@ -670,15 +677,21 @@ int pstreamOpenWrite(const char *argument, int filetype)
 	  if ( rstatus != -1 ) query_user_exit(argument);
 	}
 
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_write);
 #if  defined  (HAVE_LIBPTHREAD)
-      pthread_mutex_lock(&streamOpenWriteMutex);
+      if ( cdoLockIO )
+	pthread_mutex_lock(&streamMutex);
+      else
+	pthread_mutex_lock(&streamOpenWriteMutex);
 #endif
-      if ( cdoTimer ) timer_start(timer_write);
       fileID = streamOpenWrite(argument, filetype);
-      if ( cdoTimer ) timer_stop(timer_write);
 #if  defined  (HAVE_LIBPTHREAD)
-      pthread_mutex_unlock(&streamOpenWriteMutex);
+      if ( cdoLockIO )
+	pthread_mutex_unlock(&streamMutex);
+      else
+	pthread_mutex_unlock(&streamOpenWriteMutex);
 #endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_write);
       if ( fileID < 0 ) cdiError(fileID, "Open failed on %s", argument);
 
       cdoDefHistory(fileID, commandLine());
@@ -755,9 +768,21 @@ int pstreamOpenAppend(const char *argument)
   
       if ( PSTREAM_Debug ) Message("file %s", argument);
 
-      if ( cdoTimer ) timer_start(timer_write);
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_write);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO )
+	pthread_mutex_lock(&streamMutex);
+      else
+	pthread_mutex_lock(&streamOpenReadMutex);
+#endif
       fileID = streamOpenAppend(argument);
-      if ( cdoTimer ) timer_stop(timer_write);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO )
+	pthread_mutex_unlock(&streamMutex);
+      else
+	pthread_mutex_unlock(&streamOpenReadMutex);
+#endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_write);
       if ( fileID < 0 ) cdiError(fileID, "Open failed on %s", argument);
       /*
       cdoInqHistory(fileID);
@@ -799,8 +824,7 @@ void pstreamClose(int pstreamID)
 	  pipe = pstreamptr->pipe;
 	  pthread_mutex_lock(pipe->mutex);
 	  pipe->EOP = TRUE;
-	  if ( PSTREAM_Debug )
-	    Message("%s read closed", pstreamptr->name);
+	  if ( PSTREAM_Debug ) Message("%s read closed", pstreamptr->name);
 	  pthread_mutex_unlock(pipe->mutex);     
 	  pthread_cond_signal(pipe->tsDef);
 	  pthread_cond_signal(pipe->tsInq);
@@ -823,8 +847,7 @@ void pstreamClose(int pstreamID)
 	  pipe = pstreamptr->pipe;
 	  pthread_mutex_lock(pipe->mutex);
 	  pipe->EOP = TRUE;
-	  if ( PSTREAM_Debug )
-	    Message("%s write closed", pstreamptr->name);
+	  if ( PSTREAM_Debug ) Message("%s write closed", pstreamptr->name);
 	  pthread_mutex_unlock(pipe->mutex);     
 	  pthread_cond_signal(pipe->tsDef);
 	  pthread_cond_signal(pipe->tsInq);
@@ -832,8 +855,7 @@ void pstreamClose(int pstreamID)
 	  pthread_mutex_lock(pipe->mutex);
 	  while ( pstreamptr->isopen )
 	    {
-	      if ( PSTREAM_Debug )
-		Message("wait of read close");
+	      if ( PSTREAM_Debug ) Message("wait of read close");
 	      pthread_cond_wait(pipe->isclosed, pipe->mutex);
 	    }
 	  pthread_mutex_unlock(pipe->mutex);
@@ -852,7 +874,13 @@ void pstreamClose(int pstreamID)
 	  processAddNvals(streamNvals(pstreamptr->fileID));
 	}
 
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
       streamClose(pstreamptr->fileID);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
 
       if ( cdoExpMode == CDO_EXP_REMOTE )
 	{
@@ -901,9 +929,15 @@ int pstreamInqVlist(int pstreamID)
     {
       extern int cdoDefaultTimeType;
 
-      if ( cdoTimer ) timer_start(timer_read);
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_read);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
       vlistID = streamInqVlist(pstreamptr->fileID);
-      if ( cdoTimer ) timer_stop(timer_read);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_read);
 
       if ( cdoDefaultTimeType != CDI_UNDEFID )
 	taxisDefType(vlistInqTaxis(vlistID), cdoDefaultTimeType);
@@ -1026,7 +1060,6 @@ void pstreamDefVlist(int pstreamID, int vlistID)
   else
 #endif
     {
-      extern int cdoDefaultDataType;
       if ( cdoDefaultDataType != CDI_UNDEFID )
 	{
 	  int varID, nvars = vlistNvars(vlistID);
@@ -1044,13 +1077,27 @@ void pstreamDefVlist(int pstreamID, int vlistID)
 	    }
 	}
 
+      if ( cdoChunkType != CDI_UNDEFID )
+	{
+	  int varID, nvars = vlistNvars(vlistID);
+
+	  for ( varID = 0; varID < nvars; ++varID )
+	    vlistDefVarChunkType(vlistID, varID, cdoChunkType);
+	}
+
       vlistDefAttTxt(vlistID, CDI_GLOBAL, "CDO", (int)strlen(cdoComment())+1, cdoComment());
 
       pstreamDefVarlist(pstreamptr, vlistID);
 
-      if ( cdoTimer ) timer_start(timer_write);
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_write);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
       streamDefVlist(pstreamptr->fileID, vlistID);
-      if ( cdoTimer ) timer_stop(timer_write);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_write);
     }
 }
 
@@ -1067,9 +1114,15 @@ int pstreamInqRecord(int pstreamID, int *varID, int *levelID)
   else
 #endif
     {
-      if ( cdoTimer ) timer_start(timer_read);
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_read);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
       streamInqRecord(pstreamptr->fileID, varID, levelID);
-      if ( cdoTimer ) timer_stop(timer_read);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_read);
     }
 
   return (0);
@@ -1092,9 +1145,15 @@ void pstreamDefRecord(int pstreamID, int varID, int levelID)
   else
 #endif
     {
-      if ( cdoTimer ) timer_start(timer_write);
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_write);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
       streamDefRecord(pstreamptr->fileID, varID, levelID);
-      if ( cdoTimer ) timer_stop(timer_write);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_write);
     }
 }
 
@@ -1113,9 +1172,15 @@ void pstreamReadRecord(int pstreamID, double *data, int *nmiss)
   else
 #endif
     {
-      if ( cdoTimer ) timer_start(timer_read);
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_read);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
       streamReadRecord(pstreamptr->fileID, data, nmiss);
-      if ( cdoTimer ) timer_stop(timer_read);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_read);
     }
 }
 
@@ -1156,32 +1221,36 @@ void pstreamCheckDatarange(pstream_t *pstreamptr, int varID, double *array, int
 	  if ( array[i] < arrmin ) arrmin = array[i];
 	  if ( array[i] > arrmax ) arrmax = array[i];
 	}
+      ivals = gridsize;
     }
 
-  smin = (arrmin - addoffset)/scalefactor;
-  smax = (arrmax - addoffset)/scalefactor;
-
-  if ( datatype == DATATYPE_INT8  || datatype == DATATYPE_UINT8 ||
-       datatype == DATATYPE_INT16 || datatype == DATATYPE_UINT16 )
+  if ( ivals > 0 )
     {
-      smin = NINT(smin);
-      smax = NINT(smax);
+      smin = (arrmin - addoffset)/scalefactor;
+      smax = (arrmax - addoffset)/scalefactor;
+
+      if ( datatype == DATATYPE_INT8  || datatype == DATATYPE_UINT8 ||
+	   datatype == DATATYPE_INT16 || datatype == DATATYPE_UINT16 )
+	{
+	  smin = NINTD(smin);
+	  smax = NINTD(smax);
+	}
+
+      if      ( datatype == DATATYPE_INT8   ) { vmin =        -128.; vmax =        127.; }
+      else if ( datatype == DATATYPE_UINT8  ) { vmin =           0.; vmax =        255.; }
+      else if ( datatype == DATATYPE_INT16  ) { vmin =      -32768.; vmax =      32767.; }
+      else if ( datatype == DATATYPE_UINT16 ) { vmin =           0.; vmax =      65535.; }
+      else if ( datatype == DATATYPE_INT32  ) { vmin = -2147483648.; vmax = 2147483647.; }
+      else if ( datatype == DATATYPE_UINT32 ) { vmin =           0.; vmax = 4294967295.; }
+      else if ( datatype == DATATYPE_FLT32  ) { vmin = -3.40282e+38; vmax = 3.40282e+38; }
+      else                                    { vmin =     -1.e+300; vmax =     1.e+300; }
+
+      if ( smin < vmin || smax > vmax )
+	cdoWarning("Some data values (min=%g max=%g) are outside the\n"
+		   "    valid range (%g - %g) of the used output precision!\n"
+		   "    Use the CDO option%s -b 64 to increase the output precision.",
+		   smin, smax, vmin, vmax, (datatype == DATATYPE_FLT32) ? "" : " -b 32 or");
     }
-
-  if      ( datatype == DATATYPE_INT8   ) { vmin =        -128.; vmax =        127.; }
-  else if ( datatype == DATATYPE_UINT8  ) { vmin =           0.; vmax =        255.; }
-  else if ( datatype == DATATYPE_INT16  ) { vmin =      -32768.; vmax =      32767.; }
-  else if ( datatype == DATATYPE_UINT16 ) { vmin =           0.; vmax =      65535.; }
-  else if ( datatype == DATATYPE_INT32  ) { vmin = -2147483648.; vmax = 2147483647.; }
-  else if ( datatype == DATATYPE_UINT32 ) { vmin =           0.; vmax = 4294967295.; }
-  else if ( datatype == DATATYPE_FLT32  ) { vmin = -3.40282e+38; vmax = 3.40282e+38; }
-  else                                    { vmin =     -1.e+300; vmax =     1.e+300; }
-
-  if ( smin < vmin || smax > vmax )
-    cdoWarning("Some data values (min=%g max=%g) are outside the\n"
-               "valid range (%g - %g) of the used output precision!\n"
-	       "Use the CDO option%s -b 64 to increase the output precision.",
-	       smin, smax, vmin, vmax, (datatype == DATATYPE_FLT32) ? "" : " -b 32 or");
 }
 
 
@@ -1202,14 +1271,57 @@ void pstreamWriteRecord(int pstreamID, double *data, int nmiss)
 #endif
     {
       int varID = pstreamptr->varID;
-      if ( cdoTimer ) timer_start(timer_write);
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_write);
 
       if ( pstreamptr->varlist )
 	if ( pstreamptr->varlist[varID].check_datarange )
 	  pstreamCheckDatarange(pstreamptr, varID, data, nmiss);
 
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
       streamWriteRecord(pstreamptr->fileID, data, nmiss);
-      if ( cdoTimer ) timer_stop(timer_write);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_write);
+    }
+}
+
+
+void pstreamWriteRecordF(int pstreamID, float *data, int nmiss)
+{
+  pstream_t *pstreamptr;
+
+  if ( data == NULL ) cdoAbort("Data pointer not allocated (pstreamWriteRecord)!");
+
+  pstreamptr = pstream_to_pointer(pstreamID);
+
+#if  defined  (HAVE_LIBPTHREAD)
+  if ( pstreamptr->ispipe )
+    {
+      cdoAbort("pipeWriteRecord not implemented for memtype float!");
+      //pipeWriteRecord(pstreamptr, data, nmiss);
+    }
+  else
+#endif
+    {
+      // int varID = pstreamptr->varID;
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_write);
+      /*
+      if ( pstreamptr->varlist )
+	if ( pstreamptr->varlist[varID].check_datarange )
+	  pstreamCheckDatarange(pstreamptr, varID, data, nmiss);
+      */
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
+      streamWriteRecordF(pstreamptr->fileID, data, nmiss);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_write);
     }
 }
 
@@ -1231,9 +1343,15 @@ int pstreamInqTimestep(int pstreamID, int tsID)
 
       if ( pstreamptr->mfiles ) tsID -= pstreamptr->tsID0;
 
-      if ( cdoTimer ) timer_start(timer_read);
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_read);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
       nrecs = streamInqTimestep(pstreamptr->fileID, tsID);
-      if ( cdoTimer ) timer_stop(timer_read);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_read);
 
       if ( nrecs == 0 && pstreamptr->mfiles &&
 	   (pstreamptr->nfiles < pstreamptr->mfiles) )
@@ -1255,19 +1373,25 @@ int pstreamInqTimestep(int pstreamID, int tsID)
 	  pstreamptr->nfiles++;
 
 #if  defined  (HAVE_LIBPTHREAD)
-	  pthread_mutex_lock(&streamOpenReadMutex);
+	  if ( cdoLockIO )
+	    pthread_mutex_lock(&streamMutex);
+	  else
+	    pthread_mutex_lock(&streamOpenReadMutex);
 #endif
 	  if ( cdoVerbose ) cdoPrint("Continuation file: %s", filename);
 
-	  if ( cdoTimer ) timer_start(timer_read);
+	  if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_read);
 	  fileID = streamOpenRead(filename);
 	  vlistIDnew = streamInqVlist(fileID);
-	  if ( cdoTimer ) timer_stop(timer_read);
+	  if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_read);
 
 	  vlistCompare(vlistIDold, vlistIDnew, CMP_HRD);
 	  vlistDestroy(vlistIDold);
 #if  defined  (HAVE_LIBPTHREAD)
-	  pthread_mutex_unlock(&streamOpenReadMutex);
+	  if ( cdoLockIO )
+	    pthread_mutex_unlock(&streamMutex);
+	  else
+	    pthread_mutex_unlock(&streamOpenReadMutex);
 #endif
 	  if ( fileID < 0 ) cdiError(fileID, "Open failed on >%s<", filename);
 
@@ -1276,9 +1400,15 @@ int pstreamInqTimestep(int pstreamID, int tsID)
 	  pstreamptr->name   = filename;
 	  pstreamptr->fileID = fileID;
 
-	  if ( cdoTimer ) timer_start(timer_read);
+	  if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_read);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
 	  nrecs = streamInqTimestep(pstreamptr->fileID, 0);
-	  if ( cdoTimer ) timer_stop(timer_read);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+	  if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_read);
 	}
 
       if ( tsID == 0 && cdoDefaultTimeType != CDI_UNDEFID )
@@ -1316,11 +1446,17 @@ void pstreamDefTimestep(int pstreamID, int tsID)
       	  taxisDefType(taxisID, cdoDefaultTimeType);
 	}
 
-      if ( cdoTimer ) timer_start(timer_write);
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_write);
       /* don't use sync -> very slow on GPFS */
       //  if ( tsID > 0 ) streamSync(pstreamptr->fileID);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
       streamDefTimestep(pstreamptr->fileID, tsID);
-      if ( cdoTimer ) timer_stop(timer_write);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_write);
     }
 }
 
@@ -1334,7 +1470,7 @@ void pstreamCopyRecord(int pstreamIDdest, int pstreamIDsrc)
 
   pstreamptr_dest = pstream_to_pointer(pstreamIDdest);
   pstreamptr_src  = pstream_to_pointer(pstreamIDsrc);
-
+  /*
 #if  defined  (HAVE_LIBPTHREAD)
   if ( pstreamptr_dest->ispipe || pstreamptr_src->ispipe )
     {
@@ -1342,8 +1478,15 @@ void pstreamCopyRecord(int pstreamIDdest, int pstreamIDsrc)
     }
   else
 #endif
+  */
     {
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
       streamCopyRecord(pstreamptr_dest->fileID, pstreamptr_src->fileID);
+#if  defined  (HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
     }
 }
 
@@ -1356,13 +1499,11 @@ void pstreamDebug(int debug)
 
 void cdoInitialize(void *argument)
 {
-  int processID;
-
 #if defined (_OPENMP)
   omp_set_num_threads(ompNumThreads); /* Have to be called for every module (pthread)! */
 #endif
 
-  processID = processCreate();
+  processCreate();
 
 #if  defined  (HAVE_LIBPTHREAD)
   if ( PSTREAM_Debug )
@@ -1404,7 +1545,7 @@ void cdoFinish(void)
 #if defined (_WIN32)
 	    fprintf(stderr, "%s: Processed %I64d value%s from %d variable%s",
 #else
-	    fprintf(stderr, "%s: Processed %lld value%s from %d variable%s",
+	    fprintf(stderr, "%s: Processed %jd value%s from %d variable%s",
 #endif
 		    processInqPrompt(),
 		    nvals, nvals > 1 ? "s" : "",
@@ -1415,15 +1556,21 @@ void cdoFinish(void)
 		    (long) nvals, nvals > 1 ? "s" : "",
 		    nvars, nvars > 1 ? "s" : "");
 	}
+      else if ( nvars > 0 )
+	{
+	  fprintf(stderr, "%s: Processed %d variable%s",
+		  processInqPrompt(),
+		  nvars, nvars > 1 ? "s" : "");
+	}
       else
-	fprintf(stderr, "%s: Processed %d variable%s",
-		processInqPrompt(),
-		nvars, nvars > 1 ? "s" : "");
+	{
+	  fprintf(stderr, "%s: ", processInqPrompt());
+	}
 
       if ( ntimesteps > 0 )
 	fprintf(stderr, " over %d timestep%s", ntimesteps, ntimesteps > 1 ? "s" : "");
 
-      fprintf(stderr, ".");
+      //  fprintf(stderr, ".");
     }
   /*
     fprintf(stderr, "%s: Processed %d variable%s %d timestep%s.",
@@ -1527,10 +1674,31 @@ int pstreamInqFiletype(int pstreamID)
 }
 
 
+int pstreamInqByteorder(int pstreamID)
+{
+  int byteorder;
+  pstream_t *pstreamptr;
+
+  pstreamptr = pstream_to_pointer(pstreamID);
+
+#if  defined  (HAVE_LIBPTHREAD)
+  if ( pstreamptr->ispipe )
+    byteorder = pstreamptr->filetype;
+  else
+#endif
+    byteorder = streamInqByteorder(pstreamptr->fileID);
+
+  return (byteorder);
+}
+
+
 void openLock(void)
 {
 #if  defined  (HAVE_LIBPTHREAD)
-  pthread_mutex_lock(&streamOpenReadMutex);
+  if ( cdoLockIO )
+    pthread_mutex_lock(&streamMutex);
+  else
+    pthread_mutex_lock(&streamOpenReadMutex);
 #endif  
 }
 
@@ -1538,6 +1706,9 @@ void openLock(void)
 void openUnlock(void)
 {
 #if  defined  (HAVE_LIBPTHREAD)
-  pthread_mutex_unlock(&streamOpenReadMutex);
+  if ( cdoLockIO )
+    pthread_mutex_unlock(&streamMutex);
+  else
+    pthread_mutex_unlock(&streamOpenReadMutex);
 #endif
 }
