@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2010 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -19,11 +19,13 @@
 #include <string.h>
 #include <ctype.h>   /* tolower */
 
+#include "cdi.h"
 #include "cdo.h"
 #include "cdo_int.h"
 #include "modules.h"
 #include "util.h"
 
+
 char *getProgname(char *string)
 {
   char *progname;
@@ -211,7 +213,7 @@ void get_season_name(const char *seas_name[4])
 #include <sys/stat.h>
 //#include <unistd.h>
 
-int fileExist(const char *filename)
+int fileExists(const char *filename)
 {
   int status = 0;
   struct stat buf;
@@ -227,17 +229,25 @@ int fileExist(const char *filename)
 
 int userFileOverwrite(const char *filename)
 {
-  int status = 0;
+  int status = 0, len;
   char line[1024], *pline;
 
-  fprintf(stderr, "File %s already exist, overwrite? (yes/no): ", filename);
+  fprintf(stderr, "File %s already exists, overwrite? (yes/no): ", filename);
   readline(stdin, line, 1024);
   pline = line;
   while ( isspace((int) *pline) ) pline++;
-  if ( pline[0] == 'y' && pline[1] == 'e' && pline[2] == 's' )
-    status = 1;
-  else if ( pline[0] == 'Y' && pline[1] == 'E' && pline[2] == 'S' )
-    status = 1;
+  len = strlen(pline);
+  if ( len == 3 )
+    {
+      if ( pline[0] == 'y' && pline[1] == 'e' && pline[2] == 's' )
+	status = 1;
+      else if ( pline[0] == 'Y' && pline[1] == 'E' && pline[2] == 'S' )
+	status = 1;
+    }
+  else if ( len == 1 )
+    {
+      if ( pline[0] == 'y' ) status = 1;
+    }
 
   return (status);
 }
@@ -303,3 +313,56 @@ void progressStatus(double offset, double refval, double curval)
       fflush(stdout);
     }
 }
+
+
+int datatype2str(int datatype, char *datatypestr)
+{
+  int status = 0;
+
+  if      ( datatype == DATATYPE_PACK   ) strcpy(datatypestr, "P0");
+  else if ( datatype > 0 && datatype <= 32  ) sprintf(datatypestr, "P%d", datatype);
+  else if ( datatype == DATATYPE_CPX32  ) strcpy(datatypestr, "C32");
+  else if ( datatype == DATATYPE_CPX64  ) strcpy(datatypestr, "C64");
+  else if ( datatype == DATATYPE_FLT32  ) strcpy(datatypestr, "F32");
+  else if ( datatype == DATATYPE_FLT64  ) strcpy(datatypestr, "F64");
+  else if ( datatype == DATATYPE_INT8   ) strcpy(datatypestr, "I8");
+  else if ( datatype == DATATYPE_INT16  ) strcpy(datatypestr, "I16");
+  else if ( datatype == DATATYPE_INT32  ) strcpy(datatypestr, "I32");
+  else if ( datatype == DATATYPE_UINT8  ) strcpy(datatypestr, "U8");
+  else if ( datatype == DATATYPE_UINT16 ) strcpy(datatypestr, "U16");
+  else if ( datatype == DATATYPE_UINT32 ) strcpy(datatypestr, "U32");
+  else                                  { strcpy(datatypestr, "-1"); status = -1;}
+
+  return (status);
+}
+
+
+int str2datatype(const char *datatypestr)
+{
+  int datatype = -1;
+  size_t len;
+
+  len = strlen(datatypestr);
+
+  if ( len > 1 )
+    {
+      int ilen = atoi(datatypestr+1);
+      if      ( memcmp(datatypestr, "P0",  len) == 0 ) datatype = DATATYPE_PACK;
+      else if ( memcmp(datatypestr, "P",     1) == 0 &&
+		ilen > 0 && ilen <= 32 )               datatype = atoi(datatypestr+1);
+      else if ( memcmp(datatypestr, "C32", len) == 0 ) datatype = DATATYPE_CPX32;
+      else if ( memcmp(datatypestr, "C64", len) == 0 ) datatype = DATATYPE_CPX64;
+      else if ( memcmp(datatypestr, "F32", len) == 0 ) datatype = DATATYPE_FLT32;
+      else if ( memcmp(datatypestr, "F64", len) == 0 ) datatype = DATATYPE_FLT64;
+      else if ( memcmp(datatypestr, "I8",  len) == 0 ) datatype = DATATYPE_INT8;
+      else if ( memcmp(datatypestr, "I16", len) == 0 ) datatype = DATATYPE_INT16;
+      else if ( memcmp(datatypestr, "I32", len) == 0 ) datatype = DATATYPE_INT32;
+      else if ( memcmp(datatypestr, "U8",  len) == 0 ) datatype = DATATYPE_UINT8;
+      else if ( memcmp(datatypestr, "U16", len) == 0 ) datatype = DATATYPE_UINT16;
+      else if ( memcmp(datatypestr, "U32", len) == 0 ) datatype = DATATYPE_UINT32;
+      else if ( memcmp(datatypestr, "real",   len) == 0 ) datatype = DATATYPE_FLT32;
+      else if ( memcmp(datatypestr, "double", len) == 0 ) datatype = DATATYPE_FLT64;
+    }
+
+  return (datatype);
+}
