@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2010 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2012 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,11 @@ void init_is_tty(void);
 void progressInit(void);
 void progressStatus(double offset, double refval, double curval);
 
-int fileExist(const char *filename);
+int fileExists(const char *filename);
 int userFileOverwrite(const char *filename);
 
+/* convert a CDI datatype to string */
+int datatype2str(int datatype, char *datatypestr);
+int str2datatype(const char *datatypestr);
+
 #endif  /* _UTIL_H */
